@@ -237,6 +237,12 @@ class Serve(_BkServe):
                 index = args.index if args.index.endswith(ext) else f'{args.index}{ext}'
                 if any(f.endswith(index) for f in files):
                     found = True
+            # Check for directory style applications
+            for f in files:
+                if '.' in os.path.basename(f): # Skip files with extension
+                    continue
+                if args.index == os.path.basename(f) or args.index == f:
+                    found = True
             if not found:
                 raise ValueError(
                     "The --index argument must either specify a jinja2 "
@@ -400,7 +406,7 @@ class Serve(_BkServe):
                         "base64-encoded bytes."
                     )
                 config.oauth_encryption_key = encryption_key
-            else:
+            elif not config.oauth_encryption_key:
                 print("WARNING: OAuth has not been configured with an "
                       "encryption key and will potentially leak "
                       "credentials in cookies and a JWT token embedded "
@@ -429,7 +435,7 @@ class Serve(_BkServe):
                 )
             elif args.cookie_secret:
                 config.cookie_secret = args.cookie_secret
-            else:
+            elif not config.cookie_secret:
                 raise ValueError(
                     "When enabling an OAuth provider you must supply "
                     "a valid cookie_secret either using the --cookie-secret "
