@@ -9,6 +9,7 @@ import inspect
 import os
 import sys
 
+from concurrent.futures import ThreadPoolExecutor
 from contextlib import contextmanager
 from weakref import WeakKeyDictionary
 
@@ -18,6 +19,7 @@ from pyviz_comms import (
     JupyterCommManager as _JupyterCommManager, extension as _pyviz_extension
 )
 
+from .io.logging import panel_log_handler
 from .io.notebook import load_notebook
 from .io.state import state
 
@@ -69,16 +71,24 @@ class _base_config(param.Parameterized):
 
 class _config(_base_config):
     """
-    Holds global configuration options for Panel. The options can be
-    set directly on the global config instance, via keyword arguments
-    in the extension or via environment variables. For example to set
-    the embed option the following approaches can be used:
+    Holds global configuration options for Panel.
+
+    The options can be set directly on the global config instance, via
+    keyword arguments in the extension or via environment variables.
+
+    For example to set the embed option the following approaches can be used:
 
         pn.config.embed = True
 
         pn.extension(embed=True)
 
         os.environ['PANEL_EMBED'] = 'True'
+
+    Reference: Currently none
+
+    :Example:
+
+    >>> pn.config.loading_spinner = 'bar'
     """
 
     apply_signatures = param.Boolean(default=True, doc="""
@@ -98,6 +108,13 @@ class _config(_base_config):
     loading_max_height = param.Integer(default=400, doc="""
         Maximum height of the loading indicator.""")
 
+    notifications = param.Boolean(default=False, doc="""
+        Whether to enable notifications functionality.""")
+
+    profiler = param.Selector(default=None, allow_None=True, objects=[
+        'pyinstrument', 'snakeviz'], doc="""
+        The profiler engine to enable.""")
+
     safe_embed = param.Boolean(default=False, doc="""
         Ensure all bokeh property changes trigger events which are
         embedded. Useful when only partial updates are made in an
@@ -123,6 +140,8 @@ class _config(_base_config):
     throttled = param.Boolean(default=False, doc="""
         If sliders and inputs should be throttled until release of mouse.""")
 
+    _admin = param.Boolean(default=False, doc="Whether the admin panel was enabled.")
+
     _comms = param.ObjectSelector(
         default='default', objects=['default', 'ipywidgets', 'vscode', 'colab'], doc="""
         Whether to render output in Jupyter with the default Jupyter
@@ -153,9 +172,14 @@ class _config(_base_config):
         Where to save json files for embedded state.""")
 
     _log_level = param.Selector(
-        default=None, objects=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
+        default='WARNING', objects=['DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL'],
         doc="Log level of Panel loggers")
 
+    _nthreads = param.Integer(default=None, doc="""
+        When set to a non-None value a thread pool will be started.
+        Whenever an event arrives from the frontend it will be
+        dispatched to the thread pool to be processed.""")
+
     _oauth_provider = param.ObjectSelector(
         default=None, allow_None=True, objects=[], doc="""
         Select between a list of authentification providers.""")
@@ -185,6 +209,13 @@ class _config(_base_config):
         Whether to inline JS and CSS resources. If disabled, resources
         are loaded from CDN if one is available.""")
 
+    _globals = [
+        'autoreload', 'comms', 'cookie_secret', 'nthreads',
+        'oauth_provider', 'oauth_expiry', 'oauth_key', 'oauth_secret',
+        'oauth_jwt_user', 'oauth_redirect_uri','oauth_encryption_key',
+        'oauth_extra_params'
+    ]
+
     _truthy = ['True', 'true', '1', True, 1]
 
     _session_config = WeakKeyDictionary()
@@ -193,19 +224,44 @@ class _config(_base_config):
         super().__init__(**params)
         self._validating = False
         for p in self.param:
-            if p.startswith('_'):
+            if p.startswith('_') and p[1:] not in self._globals:
                 setattr(self, p+'_', None)
+        if self.log_level:
+            panel_log_handler.setLevel(self.log_level)
+
+    @param.depends('_nthreads', watch=True, on_init=True)
+    def _set_thread_pool(self):
+        from .io.state import state
+        if self.nthreads is None:
+            if state._thread_pool is not None:
+                state._thread_pool.shutdown(wait=False)
+            state._thread_pool = None
+            return
+        if state._thread_pool:
+            raise RuntimeError("Thread pool already running")
+        threads = self.nthreads if self.nthreads else None
+        state._thread_pool = ThreadPoolExecutor(max_workers=threads)
+
+    @param.depends('notifications', watch=True)
+    def _enable_notifications(self):
+        from .io.notifications import NotificationArea
+        from .io.state import state
+        if not state.curdoc:
+            state._notification = NotificationArea()
 
     @contextmanager
     def set(self, **kwargs):
-        values = [(k, v) for k, v in self.param.get_param_values() if k != 'name']
-        overrides = [(k, getattr(self, k+'_')) for k in self.param if k.startswith('_')]
+        values = [(k, v) for k, v in self.param.values().items() if k != 'name']
+        overrides = [
+            (k, getattr(self, k+'_')) for k in self.param
+            if k.startswith('_') and k[1:] not in self._globals
+        ]
         for k, v in kwargs.items():
             setattr(self, k, v)
         try:
             yield
         finally:
-            self.param.set_param(**dict(values))
+            self.param.update(**dict(values))
             for k, v in overrides:
                 setattr(self, k+'_', v)
 
@@ -214,7 +270,9 @@ class _config(_base_config):
         if not getattr(self, 'initialized', False) or (attr.startswith('_') and attr.endswith('_')) or attr == '_validating':
             return super().__setattr__(attr, value)
         value = getattr(self, f'_{attr}_hook', lambda x: x)(value)
-        if state.curdoc is not None:
+        if attr in self._globals:
+            super().__setattr__(attr if attr in self.param else f'_{attr}', value)
+        elif state.curdoc is not None:
             if attr in self.param:
                 validate_config(self, attr, value)
             elif f'_{attr}' in self.param:
@@ -230,9 +288,14 @@ class _config(_base_config):
         else:
             super().__setattr__(attr, value)
 
+    @param.depends('_log_level', watch=True)
+    def _update_log_level(self):
+        panel_log_handler.setLevel(self._log_level)
+
     def __getattribute__(self, attr):
         from .io.state import state
         init = super().__getattribute__('initialized')
+        global_params = super().__getattribute__('_globals')
         if init and not attr.startswith('__'):
             params = super().__getattribute__('param')
         else:
@@ -244,7 +307,9 @@ class _config(_base_config):
             state.curdoc and attr not in session_config[state.curdoc]):
             new_obj = copy.copy(super().__getattribute__(attr))
             setattr(self, attr, new_obj)
-        if state.curdoc and state.curdoc in session_config and attr in session_config[state.curdoc]:
+        if attr in global_params:
+            return super().__getattribute__(attr)
+        elif state.curdoc and state.curdoc in session_config and attr in session_config[state.curdoc]:
             return session_config[state.curdoc][attr]
         elif f'_{attr}' in params and getattr(self, f'_{attr}_') is not None:
             return super().__getattribute__(f'_{attr}_')
@@ -275,7 +340,7 @@ class _config(_base_config):
 
     @property
     def comms(self):
-        return os.environ.get('PANEL_COMMS', _config._comms)
+        return os.environ.get('PANEL_COMMS', self._comms)
 
     @property
     def embed_json(self):
@@ -302,49 +367,54 @@ class _config(_base_config):
         log_level = os.environ.get('PANEL_LOG_LEVEL', self._log_level)
         return log_level.upper() if log_level else None
 
+    @property
+    def nthreads(self):
+        nthreads = os.environ.get('PANEL_NUM_THREADS', self._nthreads)
+        return None if nthreads is None else int(nthreads)
+
     @property
     def oauth_provider(self):
-        provider = os.environ.get('PANEL_OAUTH_PROVIDER', _config._oauth_provider)
+        provider = os.environ.get('PANEL_OAUTH_PROVIDER', self._oauth_provider)
         return provider.lower() if provider else None
 
     @property
     def oauth_expiry(self):
-        provider = os.environ.get('PANEL_OAUTH_EXPIRY', _config._oauth_expiry)
+        provider = os.environ.get('PANEL_OAUTH_EXPIRY', self._oauth_expiry)
         return float(provider)
 
     @property
     def oauth_key(self):
-        return os.environ.get('PANEL_OAUTH_KEY', _config._oauth_key)
+        return os.environ.get('PANEL_OAUTH_KEY', self._oauth_key)
 
     @property
     def cookie_secret(self):
         return os.environ.get(
             'PANEL_COOKIE_SECRET',
-            os.environ.get('BOKEH_COOKIE_SECRET', _config._cookie_secret)
+            os.environ.get('BOKEH_COOKIE_SECRET', self._cookie_secret)
         )
 
     @property
     def oauth_secret(self):
-        return os.environ.get('PANEL_OAUTH_SECRET', _config._oauth_secret)
+        return os.environ.get('PANEL_OAUTH_SECRET', self._oauth_secret)
 
     @property
     def oauth_redirect_uri(self):
-        return os.environ.get('PANEL_OAUTH_REDIRECT_URI', _config._oauth_redirect_uri)
+        return os.environ.get('PANEL_OAUTH_REDIRECT_URI', self._oauth_redirect_uri)
 
     @property
     def oauth_jwt_user(self):
-        return os.environ.get('PANEL_OAUTH_JWT_USER', _config._oauth_jwt_user)
+        return os.environ.get('PANEL_OAUTH_JWT_USER', self._oauth_jwt_user)
 
     @property
     def oauth_encryption_key(self):
-        return os.environ.get('PANEL_OAUTH_ENCRYPTION', _config._oauth_encryption_key)
+        return os.environ.get('PANEL_OAUTH_ENCRYPTION', self._oauth_encryption_key)
 
     @property
     def oauth_extra_params(self):
         if 'PANEL_OAUTH_EXTRA_PARAMS' in os.environ:
             return ast.literal_eval(os.environ['PANEL_OAUTH_EXTRA_PARAMS'])
         else:
-            return _config._oauth_extra_params
+            return self._oauth_extra_params
 
 
 if hasattr(_config.param, 'objects'):
@@ -358,8 +428,29 @@ config = _config(**{k: None if p.allow_None else getattr(_config, k)
 
 class panel_extension(_pyviz_extension):
     """
-    Initializes the pyviz notebook extension to allow plotting with
-    bokeh and enable comms.
+    Initializes and configures Panel. You should always run `pn.extension`.
+    This will
+
+    - Initialize the `pyviz` notebook extension to enable bi-directional
+    communication and for example plotting with Bokeh.
+    - Load `.js` libraries (positional arguments).
+    - Update the global configuration `pn.config`
+    (keyword arguments).
+
+    Reference: https://github.com/holoviz/panel/issues/3404
+
+    :Example:
+
+    >>> import panel as pn
+    >>> pn.extension("plotly", sizing_mode="stretch_width", template="fast")
+
+    This will
+
+    - Initialize the `pyviz` notebook extension.
+    - Enable you to use the `Plotly` pane by loading `plotly.js`.
+    - Set the default `sizing_mode` to `stretch_width` instead of `fixed`.
+    - Set the global configuration `pn.config.template` to `fast`, i.e. you
+    will be using the `FastListTemplate`.
     """
 
     _loaded = False
@@ -377,7 +468,8 @@ class panel_extension(_pyviz_extension):
         'perspective': 'panel.models.perspective',
         'terminal': 'panel.models.terminal',
         'tabulator': 'panel.models.tabulator',
-        'gridstack': 'panel.layout.gridstack'
+        'texteditor': 'panel.models.quill',
+        'jsoneditor': 'panel.models.json_editor'
     }
 
     # Check whether these are loaded before rendering (if any item
@@ -399,16 +491,25 @@ class panel_extension(_pyviz_extension):
     _loaded_extensions = []
 
     def __call__(self, *args, **params):
-        # Load ipywidget support if requested in environment variable
-        if 'ipywidgets' in sys.modules and os.environ.get("PANEL_IPYWIDGET"):
-            args += ('ipywidgets',)
-    
+        from .reactive import ReactiveHTML, ReactiveHTMLMetaclass
+        reactive_exts = {
+            v._extension_name: v for k, v in param.concrete_descendents(ReactiveHTML).items()
+        }
         for arg in args:
-            if arg not in self._imports:
+            if arg in self._imports:
+                try:
+                    if (arg == 'ipywidgets' and get_ipython() and # noqa (get_ipython)
+                        not "PANEL_IPYWIDGET" in os.environ):
+                        continue
+                except Exception:
+                    pass
+                __import__(self._imports[arg])
+                self._loaded_extensions.append(arg)
+            elif arg in reactive_exts:
+                ReactiveHTMLMetaclass._loaded_extensions.add(arg)
+            else:
                 self.param.warning('%s extension not recognized and '
                                    'will be skipped.' % arg)
-            else:
-                __import__(self._imports[arg])
 
         for k, v in params.items():
             if k in ['raw_css', 'css_files']:
@@ -422,7 +523,7 @@ class panel_extension(_pyviz_extension):
             else:
                 setattr(config, k, v)
 
-        if config.apply_signatures and sys.version_info.major >= 3:
+        if config.apply_signatures:
             self._apply_signatures()
 
         loaded = self._loaded
@@ -477,16 +578,16 @@ class panel_extension(_pyviz_extension):
             # In embedded mode the ipywidgets_bokeh model must be loaded
             __import__(self._imports['ipywidgets'])
 
-        nb_load = False
+        nb_loaded = getattr(self, '_repeat_execution_in_cell', False)
         if 'holoviews' in sys.modules:
             if getattr(hv.extension, '_loaded', False):
                 return
             with param.logging_level('ERROR'):
                 hv.plotting.Renderer.load_nb(config.inline)
                 if hasattr(hv.plotting.Renderer, '_render_with_panel'):
-                    nb_load = True
+                    nb_loaded = True
 
-        if not nb_load and hasattr(ip, 'kernel'):
+        if not nb_loaded and hasattr(ip, 'kernel'):
             load_notebook(config.inline)
         panel_extension._loaded = True
 
@@ -505,6 +606,12 @@ class panel_extension(_pyviz_extension):
         if "VSCODE_PID" in os.environ:
             config.comms = "vscode"
 
+        if "pyodide" in sys.modules:
+            config.comms = "ipywidgets"
+
+        if config.notifications:
+            display(state.notifications) # noqa
+
     def _apply_signatures(self):
         from inspect import Parameter, Signature
         from .viewable import Viewable
@@ -573,3 +680,5 @@ def _cleanup_server(server_id):
 panel_extension.add_delete_action(_cleanup_panel)
 if hasattr(panel_extension, 'add_server_delete_action'):
     panel_extension.add_server_delete_action(_cleanup_server)
+
+__all__ = ['config', 'panel_extension']
