@@ -6,7 +6,6 @@ import asyncio
 import inspect
 import logging
 import time
-import sys
 
 import param
 
@@ -38,6 +37,9 @@ class PeriodicCallback(param.Parameterized):
         Number of times the callback will be executed, by default
         this is unlimited.""")
 
+    log = param.Boolean(default=True, doc="""
+        Whether the periodic callback should log its actions.""")
+
     period = param.Integer(default=500, doc="""
         Period in milliseconds at which the callback is executed.""")
 
@@ -87,7 +89,7 @@ class PeriodicCallback(param.Parameterized):
 
     def _post_callback(self):
         cbname = function_name(self.callback)
-        if self._doc:
+        if self._doc and self.log:
             _periodic_logger.info(
                 LOG_PERIODIC_END, id(self._doc), cbname, self._counter
             )
@@ -105,7 +107,7 @@ class PeriodicCallback(param.Parameterized):
         with edit_readonly(state):
             state.busy = True
         cbname = function_name(self.callback)
-        if self._doc:
+        if self._doc and self.log:
             _periodic_logger.info(
                 LOG_PERIODIC_START, id(self._doc), cbname, self._counter
             )
@@ -142,12 +144,10 @@ class PeriodicCallback(param.Parameterized):
         """
         while True:
             start = time.monotonic()
-            self._cb = asyncio.ensure_future(func())
-            await self._cb
+            await func()
             timeout = (self.period/1000.) - (time.monotonic()-start)
             if timeout > 0:
-                self._cb = asyncio.ensure_future(asyncio.sleep(timeout))
-                await self._cb
+                await asyncio.sleep(timeout)
 
     def _cleanup(self, session_context):
         self.stop()
@@ -165,12 +165,10 @@ class PeriodicCallback(param.Parameterized):
             finally:
                 self._updating = False
         self._start_time = time.time()
-        if '_pyodide' in sys.modules:
-            event_loop = asyncio.get_running_loop()
-            task = asyncio.create_task(
+        if state._is_pyodide:
+            self._cb = asyncio.create_task(
                 self._async_repeat(self._periodic_callback)
             )
-            event_loop.call_soon(task)
         elif state.curdoc:
             self._doc = state.curdoc
             self._cb = self._doc.add_periodic_callback(self._periodic_callback, self.period)
@@ -178,10 +176,6 @@ class PeriodicCallback(param.Parameterized):
             from tornado.ioloop import PeriodicCallback
             self._cb = PeriodicCallback(lambda: asyncio.create_task(self._periodic_callback()), self.period)
             self._cb.start()
-        try:
-            state.on_session_destroyed(self._cleanup)
-        except Exception:
-            pass
 
     def stop(self):
         """
@@ -195,7 +189,7 @@ class PeriodicCallback(param.Parameterized):
                 self._updating = False
         self._counter = 0
         self._timeout = None
-        if '_pyodide' in sys.modules:
+        if state._is_pyodide:
             self._cb.cancel()
         elif self._doc:
             if self._doc._session_context:
