@@ -1,45 +1,89 @@
 """
 Utilities for manipulating bokeh models.
 """
-from __future__ import absolute_import, division, unicode_literals
+from __future__ import annotations
 
 import textwrap
+
 from contextlib import contextmanager
+from typing import (
+    TYPE_CHECKING, Any, List, Iterable, Optional
+)
+from typing_extensions import Literal
+
+import numpy as np
 
 from bokeh.document import Document
-from bokeh.document.events import ColumnDataChangedEvent
+from bokeh.document.events import (
+    ColumnDataChangedEvent, DocumentPatchedEvent, ModelChangedEvent
+)
+from bokeh.model import DataModel
 from bokeh.models import Box, ColumnDataSource, Model
 from bokeh.protocol import Protocol
 
 from .state import state
 
+if TYPE_CHECKING:
+    from bokeh.core.enums import HoldPolicyType
+    from bokeh.document.events import DocumentChangedEvent
+    from bokeh.protocol.message import Message
+    from pyviz_comms import Comm
+
+#---------------------------------------------------------------------
+# Private API
+#---------------------------------------------------------------------
+
+class comparable_array(np.ndarray):
+    """
+    Array subclass that allows comparisons.
+    """
+
+    def __eq__(self, other: Any) -> bool:
+        return super().__eq__(other).all().item()
+
+    def __ne__(self, other: Any) -> bool:
+        return super().__ne__(other).all().item()
+
+def monkeypatch_events(events: List['DocumentChangedEvent']) -> None:
+    """
+    Patch events applies patches to events that are to be dispatched
+    avoiding various issues in Bokeh.
+    """
+    for e in events:
+        # Patch ColumnDataChangedEvents which reference non-existing columns
+        if isinstance(getattr(e, 'hint', None), ColumnDataChangedEvent):
+            e.hint.cols = None # type: ignore
+        # Patch ModelChangedEvents which change an array property (see https://github.com/bokeh/bokeh/issues/11735)
+        elif (isinstance(e, ModelChangedEvent) and isinstance(e.model, DataModel) and
+              isinstance(e.new, np.ndarray)):
+                new_array = comparable_array(e.new.shape, e.new.dtype, e.new)
+                e.new = new_array
+                e.serializable_new = new_array
+
 #---------------------------------------------------------------------
 # Public API
 #---------------------------------------------------------------------
 
-def diff(doc, binary=True, events=None):
+def diff(
+    doc: 'Document', binary: bool = True, events: Optional[List['DocumentChangedEvent']] = None
+) -> Message[Any] | None:
     """
     Returns a json diff required to update an existing plot with
     the latest plot data.
     """
-    events = list(doc._held_events) if events is None else events
+    if events is None:
+        events = list(doc.callbacks._held_events)
     if not events or state._hold:
         return None
 
-    # Filter ColumnDataChangedEvents which reference non-existing
-    # columns, later event will include the changes
-    fixed_events = []
-    for e in events:
-        if (hasattr(e, 'hint') and isinstance(e.hint, ColumnDataChangedEvent)
-            and e.hint.cols is not None):
-            e.hint.cols = None
-        fixed_events.append(e)
-    msg = Protocol().create("PATCH-DOC", events, use_buffers=binary)
-    doc._held_events = [e for e in doc._held_events if e not in events]
+    patch_events = [event for event in events if isinstance(event, DocumentPatchedEvent)]
+    monkeypatch_events(events)
+    msg_type: Literal["PATCH-DOC"] = "PATCH-DOC"
+    msg = Protocol().create(msg_type, patch_events, use_buffers=binary)
+    doc.callbacks._held_events = [e for e in doc.callbacks._held_events if e not in events]
     return msg
 
-
-def remove_root(obj, replace=None):
+def remove_root(obj: 'Model', replace: Optional['Document'] = None) -> None:
     """
     Removes the document from any previously displayed bokeh object
     """
@@ -51,20 +95,19 @@ def remove_root(obj, replace=None):
         if replace:
             model._document = replace
 
-
-def add_to_doc(obj, doc, hold=False):
+def add_to_doc(obj: 'Model', doc: 'Document', hold: bool = False):
     """
     Adds a model to the supplied Document removing it from any existing Documents.
     """
     # Add new root
     remove_root(obj)
     doc.add_root(obj)
-    if doc._hold is None and hold:
+    if doc.callbacks.hold_value is None and hold:
         doc.hold()
 
 @contextmanager
-def hold(doc, policy='combine'):
-    held = doc._hold
+def hold(doc: 'Document', policy: 'HoldPolicyType' = 'combine', comm: Optional['Comm'] = None):
+    held = doc.callbacks.hold_value
     try:
         if policy is None:
             doc.unhold()
@@ -73,11 +116,13 @@ def hold(doc, policy='combine'):
         yield
     finally:
         if held:
-            doc._hold = held
+            doc.callbacks._hold = held
         else:
+            if comm is not None:
+                from .notebook import push
+                push(doc, comm)
             doc.unhold()
 
-
 def patch_cds_msg(model, msg):
     """
     Required for handling messages containing JSON serialized typed
@@ -93,10 +138,9 @@ def patch_cds_msg(model, msg):
             if isinstance(values, dict):
                 event['new'][col] = [v for _, v in sorted(values.items())]
 
-
 _DEFAULT_IGNORED_REPR = frozenset(['children', 'text', 'name', 'toolbar', 'renderers', 'below', 'center', 'left', 'right'])
 
-def bokeh_repr(obj, depth=0, ignored=None):
+def bokeh_repr(obj: 'Model', depth: int = 0, ignored: Optional[Iterable[str]] = None) -> str:
     """
     Returns a string repr for a bokeh model, useful for recreating
     panel objects using pure bokeh.
@@ -110,7 +154,7 @@ def bokeh_repr(obj, depth=0, ignored=None):
 
     r = ""
     cls = type(obj).__name__
-    properties = sorted(obj.properties_with_values(False).items())
+    properties = sorted(obj.properties_with_values(include_defaults=False).items())
     props = []
     for k, v in properties:
         if k in ignored:
@@ -122,12 +166,12 @@ def bokeh_repr(obj, depth=0, ignored=None):
         if len(v) > 30:
             v = v[:30] + '...'
         props.append('%s=%s' % (k, v))
-    props = ', '.join(props)
+    props_repr = ', '.join(props)
     if isinstance(obj, Box):
         r += '{cls}(children=[\n'.format(cls=cls)
-        for obj in obj.children:
+        for obj in obj.children: # type: ignore
             r += textwrap.indent(bokeh_repr(obj, depth=depth+1) + ',\n', '  ')
-        r += '], %s)' % props
+        r += '], %s)' % props_repr
     else:
-        r += '{cls}({props})'.format(cls=cls,  props=props)
+        r += '{cls}({props})'.format(cls=cls, props=props_repr)
     return r
