@@ -44,6 +44,8 @@ def conffilter(value):
     return json.dumps(OrderedDict(value)).replace('"', '\'')
 
 _env = get_env()
+_env.trim_blocks = True
+_env.lstrip_blocks = True
 _env.filters['json'] = lambda obj: Markup(json.dumps(obj))
 _env.filters['conffilter'] = conffilter
 
@@ -204,20 +206,22 @@ def bundle_resources(roots, resources):
 
 class Resources(BkResources):
 
+    def __init__(self, *args, absolute=False, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.absolute = absolute
+
     @classmethod
     def from_bokeh(cls, bkr, absolute=False):
         kwargs = {}
         if bkr.mode.startswith("server"):
             kwargs['root_url'] = bkr.root_url
-        inst = cls(
+        return cls(
             mode=bkr.mode, version=bkr.version, minified=bkr.minified,
             legacy=bkr.legacy, log_level=bkr.log_level,
             path_versioner=bkr.path_versioner,
             components=bkr._components, base_dir=bkr.base_dir,
-            root_dir=bkr.root_dir, **kwargs
+            root_dir=bkr.root_dir, absolute=absolute, **kwargs
         )
-        inst.absolute = absolute
-        return inst
 
     def extra_resources(self, resources, resource_type):
         """
@@ -313,8 +317,8 @@ class Resources(BkResources):
         from ..config import config
         modules = list(config.js_modules.values())
         self.extra_resources(modules, '__javascript_modules__')
-        
-        
+
+
         return modules
 
     @property
