@@ -1,9 +1,14 @@
 """
 Utilities for creating bokeh Server instances.
 """
+from __future__ import annotations
+
+import asyncio
 import datetime as dt
+import gc
 import html
-import inspect
+import importlib
+import logging
 import os
 import pathlib
 import signal
@@ -11,16 +16,21 @@ import sys
 import traceback
 import threading
 import uuid
+import warnings
 
 from collections import OrderedDict
 from contextlib import contextmanager
 from functools import partial, wraps
 from types import FunctionType, MethodType
+from typing import (
+    TYPE_CHECKING, Any, Callable, Dict, Mapping, Optional, Union
+)
 from urllib.parse import urljoin, urlparse
 
 import param
 import bokeh
 import bokeh.command.util
+import tornado
 
 # Bokeh imports
 from bokeh.application import Application as BkApplication
@@ -28,29 +38,46 @@ from bokeh.application.handlers.code import CodeHandler, _monkeypatch_io, patch_
 from bokeh.application.handlers.function import FunctionHandler
 from bokeh.command.util import build_single_handler_application
 from bokeh.core.templates import AUTOLOAD_JS
-from bokeh.document.events import ModelChangedEvent
 from bokeh.embed.bundle import Script
 from bokeh.embed.elements import html_page_for_render_items, script_for_render_items
 from bokeh.embed.util import RenderItem
 from bokeh.io import curdoc
 from bokeh.server.server import Server
-from bokeh.server.urls import per_app_patterns, toplevel_patterns
+from bokeh.server.urls import per_app_patterns
 from bokeh.server.views.autoload_js_handler import AutoloadJsHandler as BkAutoloadJsHandler
 from bokeh.server.views.doc_handler import DocHandler as BkDocHandler
-from bokeh.server.views.root_handler import RootHandler as BkRootHandler
 from bokeh.server.views.static_handler import StaticHandler
 
 # Tornado imports
 from tornado.ioloop import IOLoop
-from tornado.websocket import WebSocketHandler
-from tornado.web import RequestHandler, StaticFileHandler, authenticated
+from tornado.web import HTTPError, RequestHandler, StaticFileHandler, authenticated
 from tornado.wsgi import WSGIContainer
 
 # Internal imports
-from ..util import edit_readonly
+from ..util import edit_readonly, fullpath
+from .document import init_doc, with_lock, unlocked # noqa
+from .logging import LOG_SESSION_CREATED, LOG_SESSION_DESTROYED, LOG_SESSION_LAUNCHING
+from .profile import profile_ctx
 from .reload import autoreload_watcher
-from .resources import BASE_TEMPLATE, Resources, bundle_resources
-from .state import state
+from .resources import (
+    BASE_TEMPLATE, COMPONENT_PATH, Resources, bundle_resources, component_rel_path
+)
+from .state import set_curdoc, state
+
+logger = logging.getLogger(__name__)
+
+if TYPE_CHECKING:
+    from bokeh.document import Document
+    from bokeh.server.contexts import BokehSessionContext
+    from bokeh.server.session import ServerSession
+    from jinja2 import Template
+
+    from ..template.base import BaseTemplate
+    from ..viewable import Viewable, Viewer
+    from .location import Location
+
+    TViewable = Union[Viewable, Viewer, BaseTemplate]
+    TViewableOrFunc = Union[TViewable, Callable[[], TViewable]]
 
 #---------------------------------------------------------------------
 # Private API
@@ -58,18 +85,18 @@ from .state import state
 
 INDEX_HTML = os.path.join(os.path.dirname(__file__), '..', '_templates', "index.html")
 
-def _origin_url(url):
+def _origin_url(url: str) -> str:
     if url.startswith("http"):
         url = url.split("//")[1]
     return url
 
-def _server_url(url, port):
+def _server_url(url: str, port: int) -> str:
     if url.startswith("http"):
         return '%s:%d%s' % (url.rsplit(':', 1)[0], port, "/")
     else:
         return 'http://%s:%d%s' % (url.split(':')[0], port, "/")
 
-def _eval_panel(panel, server_id, title, location, doc):
+def _eval_panel(panel: 'TViewableOrFunc', server_id: str, title: str, location, doc: 'Document'):
     from ..template import BaseTemplate
     from ..pane import panel as as_panel
 
@@ -82,14 +109,14 @@ def _eval_panel(panel, server_id, title, location, doc):
             doc = as_panel(panel)._modify_doc(server_id, title, doc, location)
         return doc
 
-def async_execute(func):
+def async_execute(func: Callable[..., None]) -> None:
     """
     Wrap async event loop scheduling to ensure that with_lock flag
     is propagated from function to partial wrapping it.
     """
     if not state.curdoc or not state.curdoc.session_context:
         ioloop = IOLoop.current()
-        event_loop = ioloop.asyncio_loop
+        event_loop = ioloop.asyncio_loop # type: ignore
         if event_loop.is_running():
             ioloop.add_callback(func)
         else:
@@ -97,7 +124,7 @@ def async_execute(func):
         return
 
     if isinstance(func, partial) and hasattr(func.func, 'lock'):
-        unlock = not func.func.lock
+        unlock = not func.func.lock # type: ignore
     else:
         unlock = not getattr(func, 'lock', False)
     curdoc = state.curdoc
@@ -106,22 +133,23 @@ def async_execute(func):
         with set_curdoc(curdoc):
             return await func(*args, **kw)
     if unlock:
-        wrapper.nolock = True
+        wrapper.nolock = True # type: ignore
     state.curdoc.add_next_tick_callback(wrapper)
 
 param.parameterized.async_executor = async_execute
 
-def _initialize_session_info(session_context):
+def _initialize_session_info(session_context: 'BokehSessionContext'):
     from ..config import config
     session_id = session_context.id
     sessions = state.session_info['sessions']
-    if config.session_history == 0 or session_id in sessions:
+    history = -1 if config._admin else config.session_history
+    if not config._admin and (history == 0 or session_id in sessions):
         return
 
     state.session_info['total'] += 1
-    if config.session_history > 0 and len(sessions) >= config.session_history:
+    if history > 0 and len(sessions) >= history:
         old_history = list(sessions.items())
-        sessions = OrderedDict(old_history[-(config.session_history-1):])
+        sessions = OrderedDict(old_history[-(history-1):])
         state.session_info['sessions'] = sessions
     sessions[session_id] = {
         'launched': dt.datetime.now().timestamp(),
@@ -130,6 +158,7 @@ def _initialize_session_info(session_context):
         'ended': None,
         'user_agent': session_context.request.headers.get('User-Agent')
     }
+    state.param.trigger('session_info')
 
 state.on_session_created(_initialize_session_info)
 
@@ -137,8 +166,11 @@ state.on_session_created(_initialize_session_info)
 # Bokeh patches
 #---------------------------------------------------------------------
 
-def server_html_page_for_session(session, resources, title, template=BASE_TEMPLATE,
-                                 template_variables=None):
+def server_html_page_for_session(
+    session: 'ServerSession', resources: 'Resources', title: str,
+    template: str | Template = BASE_TEMPLATE,
+    template_variables: Optional[Dict[str, Any]] = None
+) -> str:
     render_item = RenderItem(
         token = session.token,
         roots = session.document.roots,
@@ -161,6 +193,28 @@ def autoload_js_script(doc, resources, token, element_id, app_path, absolute_url
 
     return AUTOLOAD_JS.render(bundle=bundle, elementid=element_id)
 
+
+def destroy_document(self, session):
+    """
+    Override for Document.destroy() without calling gc.collect directly.
+    The gc.collect() call is scheduled as a task, ensuring that when
+    multiple documents are destroyed in quick succession we do not
+    schedule excessive garbage collection.
+    """
+    self.remove_on_change(session)
+    del self._roots
+    del self._theme
+    del self._template
+    self._session_context = None
+
+    self.callbacks.destroy()
+    self.models.destroy()
+    self.modules.destroy()
+
+    at = dt.datetime.now() + dt.timedelta(seconds=5)
+    state.schedule_task('gc.collect', gc.collect, at=at)
+
+
 # Patch Application to handle session callbacks
 class Application(BkApplication):
 
@@ -175,7 +229,7 @@ class Application(BkApplication):
             template = state._templates[doc]
             template.server_doc(title=template.title, location=True, doc=doc)
 
-bokeh.command.util.Application = Application
+bokeh.command.util.Application = Application # type: ignore
 
 
 class SessionPrefixHandler:
@@ -192,7 +246,7 @@ class SessionPrefixHandler:
         # Handle autoload.js absolute paths
         abs_url = self.get_argument('bokeh-absolute-url', default=None)
         if abs_url is not None:
-            app_path = self.get_argument('bokeh-app-path', default=None)
+            app_path = self.get_argument('bokeh-app-path', default='')
             rel_path = abs_url.replace(app_path, '')
 
         with edit_readonly(state):
@@ -213,6 +267,7 @@ class DocHandler(BkDocHandler, SessionPrefixHandler):
         with self._session_prefix():
             session = await self.get_session()
             state.curdoc = session.document
+            logger.info(LOG_SESSION_CREATED, id(session.document))
             try:
                 resources = Resources.from_bokeh(self.application.resources())
                 page = server_html_page_for_session(
@@ -233,7 +288,7 @@ class AutoloadJsHandler(BkAutoloadJsHandler, SessionPrefixHandler):
 
     '''
 
-    async def get(self, *args, **kwargs):
+    async def get(self, *args, **kwargs) -> None:
         element_id = self.get_argument("bokeh-autoload-element", default=None)
         if not element_id:
             self.send_error(status_code=400, reason='No bokeh-autoload-element query parameter')
@@ -264,23 +319,100 @@ class AutoloadJsHandler(BkAutoloadJsHandler, SessionPrefixHandler):
 
 per_app_patterns[3] = (r'/autoload.js', AutoloadJsHandler)
 
-class RootHandler(BkRootHandler):
 
-    @authenticated
-    async def get(self, *args, **kwargs):
-        if self.index and not self.index.endswith('.html'):
-            prefix = "" if self.prefix is None else self.prefix
-            redirect_to = prefix + '.'.join(self.index.split('.')[:-1])
-            self.redirect(redirect_to)
-        await super().get(*args, **kwargs)
+class ComponentResourceHandler(StaticFileHandler):
+    """
+    A handler that serves local resources relative to a Python module.
+    The handler resolves a specific Panel component by module reference
+    and name, then resolves an attribute on that component to check
+    if it contains the requested resource path.
 
-toplevel_patterns[0] = (r'/?', RootHandler)
-bokeh.server.tornado.RootHandler = RootHandler
+    /<endpoint>/<module>/<class>/<attribute>/<path>
+    """
 
-def modify_document(self, doc):
+    _resource_attrs = [
+        '__css__', '__javascript__', '__js_module__',  '_resources',
+        '_css', '_js', 'base_css', 'css'
+    ]
+
+    def initialize(self, path: Optional[str] = None, default_filename: Optional[str] = None):
+        self.root = path
+        self.default_filename = default_filename
+
+    def parse_url_path(self, path: str) -> str:
+        """
+        Resolves the resource the URL pattern refers to.
+        """
+        parts = path.split('/')
+        if len(parts) < 4:
+            raise HTTPError(400, 'Malformed URL')
+        mod, cls, rtype, *subpath = parts
+        try:
+            module = importlib.import_module(mod)
+        except ModuleNotFoundError:
+            raise HTTPError(404, 'Module not found')
+        try:
+            component = getattr(module, cls)
+        except AttributeError:
+            raise HTTPError(404, 'Component not found')
+
+        # May only access resources listed in specific attributes
+        if rtype not in self._resource_attrs:
+            raise HTTPError(403, 'Requested resource type not valid.')
+
+        try:
+            resources = getattr(component, rtype)
+        except AttributeError:
+            raise HTTPError(404, 'Resource type not found')
+
+        # Handle template resources
+        if rtype == '_resources':
+            rtype = subpath[0]
+            subpath = subpath[1:]
+            if rtype not in resources:
+                raise HTTPError(404, 'Resource type not found')
+            resources = resources[rtype]
+            rtype = f'_resources/{rtype}'
+
+        if isinstance(resources, dict):
+            resources = list(resources.values())
+        elif isinstance(resources, (str, pathlib.PurePath)):
+            resources = [resources]
+        resources = [
+            component_rel_path(component, resource).replace(os.path.sep, '/')
+            for resource in resources
+        ]
+
+        rel_path = '/'.join(subpath)
+
+        # Important: May only access resources explicitly listed on the component
+        # Otherwise this potentially exposes all files to the web
+        if rel_path not in resources:
+            raise HTTPError(403, 'Requested resource was not listed.')
+
+        if not module.__file__:
+            raise HTTPError(404, 'Requested module does not reference a file.')
+
+        return str(pathlib.Path(module.__file__).parent / rel_path)
+
+    @classmethod
+    def get_absolute_path(cls, root: str, path: str) -> str:
+        return path
+
+    def validate_absolute_path(self, root: str, absolute_path: str) -> str:
+        if not os.path.exists(absolute_path):
+            raise HTTPError(404)
+        if not os.path.isfile(absolute_path):
+            raise HTTPError(403, "%s is not a file", self.path)
+        return absolute_path
+
+
+def modify_document(self, doc: 'Document'):
     from bokeh.io.doc import set_curdoc as bk_set_curdoc
     from ..config import config
 
+    logger.info(LOG_SESSION_LAUNCHING, id(doc))
+
     if config.autoreload:
         path = self._runner.path
         argv = self._runner._argv
@@ -302,13 +434,18 @@ def modify_document(self, doc):
     sys.modules[module.__name__] = module
     doc.modules._modules.append(module)
 
-    old_doc = curdoc()
+    try:
+        old_doc = curdoc()
+    except RuntimeError:
+        old_doc = None
     bk_set_curdoc(doc)
 
     if config.autoreload:
         set_curdoc(doc)
         state.onload(autoreload_watcher)
 
+    sessions = []
+
     try:
         def post_check():
             newdoc = curdoc()
@@ -344,13 +481,31 @@ def modify_document(self, doc):
 
         if config.autoreload:
             bokeh.application.handlers.code_runner.handle_exception = handle_exception
+
+        state._launching.append(doc)
         with _monkeypatch_io(self._loggers):
             with patch_curdoc(doc):
-                self._runner.run(module, post_check)
-    finally:
-        bk_set_curdoc(old_doc)
+                with profile_ctx(config.profiler) as sessions:
+                    self._runner.run(module, post_check)
 
-CodeHandler.modify_document = modify_document
+        def _log_session_destroyed(session_context):
+            logger.info(LOG_SESSION_DESTROYED, id(doc))
+
+        doc.on_session_destroyed(_log_session_destroyed)
+        doc.destroy = partial(destroy_document, doc) # type: ignore
+    finally:
+        state._launching.remove(doc)
+        if config.profiler:
+            try:
+                path = doc.session_context.request.path
+                state._profiles[(path, config.profiler)] += sessions
+                state.param.trigger('_profiles')
+            except Exception:
+                pass
+        if old_doc is not None:
+            bk_set_curdoc(old_doc)
+
+CodeHandler.modify_document = modify_document # type: ignore
 
 # Copied from bokeh 2.4.0, to fix directly in bokeh at some point.
 def create_static_handler(prefix, key, app):
@@ -365,107 +520,29 @@ def create_static_handler(prefix, key, app):
 
 bokeh.server.tornado.create_static_handler = create_static_handler
 
+# Bokeh 2.4.x patches the asyncio event loop policy but Tornado 6.1
+# support the WindowsProactorEventLoopPolicy so we restore it,
+# unless we detect we are running on jupyter_server.
+if (
+    sys.platform == 'win32' and
+    sys.version_info[:3] >= (3, 8, 0) and
+    tornado.version_info >= (6, 1) and
+    type(asyncio.get_event_loop_policy()) is asyncio.WindowsSelectorEventLoopPolicy and
+    'jupyter_server' not in sys.modules
+):
+    asyncio.set_event_loop_policy(asyncio.WindowsProactorEventLoopPolicy())
+
 #---------------------------------------------------------------------
 # Public API
 #---------------------------------------------------------------------
 
-def init_doc(doc):
-    doc = doc or curdoc()
-    if not doc.session_context:
-        return doc
-
-    session_id = doc.session_context.id
-    sessions = state.session_info['sessions']
-    if session_id not in sessions:
-        return doc
-
-    sessions[session_id].update({
-        'started': dt.datetime.now().timestamp()
-    })
-    doc.on_event('document_ready', state._init_session)
-    return doc
-
-@contextmanager
-def set_curdoc(doc):
-    state.curdoc = doc
-    yield
-    state.curdoc = None
-
-def with_lock(func):
-    """
-    Wrap a callback function to execute with a lock allowing the
-    function to modify bokeh models directly.
-
-    Arguments
-    ---------
-    func: callable
-      The callable to wrap
-
-    Returns
-    -------
-    wrapper: callable
-      Function wrapped to execute without a Document lock.
-    """
-    if inspect.iscoroutinefunction(func):
-        @wraps(func)
-        async def wrapper(*args, **kw):
-            return await func(*args, **kw)
-    else:
-        @wraps(func)
-        def wrapper(*args, **kw):
-            return func(*args, **kw)
-    wrapper.lock = True
-    return wrapper
-
-
-@contextmanager
-def unlocked():
-    """
-    Context manager which unlocks a Document and dispatches
-    ModelChangedEvents triggered in the context body to all sockets
-    on current sessions.
-    """
-    curdoc = state.curdoc
-    if curdoc is None or curdoc.session_context is None or curdoc.session_context.session is None:
-        yield
-        return
-    connections = curdoc.session_context.session._subscribed_connections
-
-    hold = curdoc.callbacks.hold_value
-    if hold:
-        old_events = list(curdoc.callbacks._held_events)
-    else:
-        old_events = []
-        curdoc.hold()
-    try:
-        yield
-        events = []
-        for conn in connections:
-            socket = conn._socket
-            if hasattr(socket, 'write_lock') and socket.write_lock._block._value == 0:
-                state._locks.add(socket)
-            locked = socket in state._locks
-            for event in curdoc.callbacks._held_events:
-                if (isinstance(event, ModelChangedEvent) and event not in old_events
-                    and hasattr(socket, 'write_message') and not locked):
-                    msg = conn.protocol.create('PATCH-DOC', [event])
-                    WebSocketHandler.write_message(socket, msg.header_json)
-                    WebSocketHandler.write_message(socket, msg.metadata_json)
-                    WebSocketHandler.write_message(socket, msg.content_json)
-                    for header, payload in msg._buffers:
-                        WebSocketHandler.write_message(socket, header)
-                        WebSocketHandler.write_message(socket, payload, binary=True)
-                elif event not in events:
-                    events.append(event)
-        curdoc.callbacks._held_events = events
-    finally:
-        if not hold:
-            curdoc.unhold()
-
-
-def serve(panels, port=0, address=None, websocket_origin=None, loop=None,
-          show=True, start=True, title=None, verbose=True, location=True,
-          threaded=False, **kwargs):
+def serve(
+    panels: 'TViewableOrFunc' | Mapping[str, 'TViewableOrFunc'], port: int = 0,
+    address: Optional[str] = None, websocket_origin: Optional[str | list[str]] = None,
+    loop: Optional[IOLoop] = None, show: bool = True, start: bool = True,
+    title: Optional[str] = None, verbose: bool = True, location: bool = True,
+    threaded: bool = False, **kwargs
+) -> threading.Thread | 'Server':
     """
     Allows serving one or more panel objects on a single server.
     The panels argument should be either a Panel object or a function
@@ -474,6 +551,8 @@ def serve(panels, port=0, address=None, websocket_origin=None, loop=None,
     each app is served, e.g. `serve({'app': panel1, 'app2': panel2})`
     will serve apps at /app and /app2 on the server.
 
+    Reference: https://panel.holoviz.org/user_guide/Deploy_and_Export.html#serving-multiple-apps
+
     Arguments
     ---------
     panel: Viewable, function or {str: Viewable or function}
@@ -515,7 +594,6 @@ def serve(panels, port=0, address=None, websocket_origin=None, loop=None,
         location=location
     ))
     if threaded:
-        from tornado.ioloop import IOLoop
         kwargs['loop'] = loop = IOLoop() if loop is None else loop
         server = StoppableThread(
             target=get_server, io_loop=loop, args=(panels,), kwargs=kwargs
@@ -524,7 +602,7 @@ def serve(panels, port=0, address=None, websocket_origin=None, loop=None,
         state._threads[server_id] = server
         server.start()
     else:
-        server = get_server(panels, **kwargs)
+        return get_server(panels, **kwargs)
     return server
 
 
@@ -557,21 +635,28 @@ def get_static_routes(static_dirs):
         if slug == '/static':
             raise ValueError("Static file route may not use /static "
                              "this is reserved for internal use.")
-        path = os.path.abspath(path)
+        path = fullpath(path)
         if not os.path.isdir(path):
             raise ValueError("Cannot serve non-existent path %s" % path)
         patterns.append(
             (r"%s/(.*)" % slug, StaticFileHandler, {"path": path})
         )
+    patterns.append((
+        f'/{COMPONENT_PATH}(.*)', ComponentResourceHandler, {}
+    ))
     return patterns
 
-
-def get_server(panel, port=0, address=None, websocket_origin=None,
-               loop=None, show=False, start=False, title=None,
-               verbose=False, location=True, static_dirs={},
-               oauth_provider=None, oauth_key=None, oauth_secret=None,
-               oauth_extra_params={}, cookie_secret=None,
-               oauth_encryption_key=None, session_history=None, **kwargs):
+def get_server(
+    panel: 'TViewableOrFunc' | Mapping[str, 'TViewableOrFunc'], port: int = 0,
+    address: Optional[str] = None, websocket_origin: Optional[str | list[str]] = None,
+    loop: Optional[IOLoop] = None, show: bool = False, start: bool = False,
+    title: bool = None, verbose: bool = False, location: bool | Location = True,
+    static_dirs: Mapping[str, str] = {}, oauth_provider: Optional[str] = None,
+    oauth_key: Optional[str] = None, oauth_secret: Optional[str] = None,
+    oauth_extra_params: Mapping[str, str] = {}, cookie_secret: Optional[str] = None,
+    oauth_encryption_key: Optional[str] = None, session_history: Optional[int] = None,
+    **kwargs
+) -> Server:
     """
     Returns a Server instance with this panel attached as the root
     app.
@@ -670,6 +755,8 @@ def get_server(panel, port=0, address=None, websocket_origin=None,
             if (isinstance(app, str) and (app.endswith(".py") or app.endswith(".ipynb"))
                 and os.path.isfile(app)):
                 apps[slug] = build_single_handler_application(app)
+            elif isinstance(app, Application):
+                apps[slug] = app
             else:
                 handler = FunctionHandler(partial(_eval_panel, app, server_id, title_, location))
                 apps[slug] = Application(handler)
@@ -706,17 +793,16 @@ def get_server(panel, port=0, address=None, websocket_origin=None,
 
     # Configure OAuth
     from ..config import config
-    if config.oauth_provider:
-        from ..auth import OAuthProvider
-        opts['auth_provider'] = OAuthProvider()
     if oauth_provider:
-        config.oauth_provider = oauth_provider
+        from ..auth import OAuthProvider
+        config.oauth_provider = oauth_provider # type: ignore
+        opts['auth_provider'] = OAuthProvider()
     if oauth_key:
-        config.oauth_key = oauth_key
+        config.oauth_key = oauth_key # type: ignore
     if oauth_extra_params:
-        config.oauth_extra_params = oauth_extra_params
+        config.oauth_extra_params = oauth_extra_params # type: ignore
     if cookie_secret:
-        config.cookie_secret = cookie_secret
+        config.cookie_secret = cookie_secret # type: ignore
     opts['cookie_secret'] = config.cookie_secret
 
     server = Server(apps, port=port, **opts)
@@ -749,21 +835,26 @@ def get_server(panel, port=0, address=None, websocket_origin=None,
             server.io_loop.start()
         except RuntimeError:
             pass
+        except TypeError:
+            warnings.warn(
+                "IOLoop couldn't be started. Ensure it is started by "
+                "process invoking the panel.io.server.serve."
+            )
     return server
 
 
 class StoppableThread(threading.Thread):
     """Thread class with a stop() method."""
 
-    def __init__(self, io_loop=None, **kwargs):
+    def __init__(self, io_loop: IOLoop, **kwargs):
         super().__init__(**kwargs)
         self.io_loop = io_loop
 
-    def run(self):
+    def run(self) -> None:
         if hasattr(self, '_target'):
-            target, args, kwargs = self._target, self._args, self._kwargs
+            target, args, kwargs = self._target, self._args, self._kwargs # type: ignore
         else:
-            target, args, kwargs = self._Thread__target, self._Thread__args, self._Thread__kwargs
+            target, args, kwargs = self._Thread__target, self._Thread__args, self._Thread__kwargs # type: ignore
         if not target:
             return
         bokeh_server = None
@@ -771,11 +862,14 @@ class StoppableThread(threading.Thread):
             bokeh_server = target(*args, **kwargs)
         finally:
             if isinstance(bokeh_server, Server):
-                bokeh_server.stop()
+                try:
+                    bokeh_server.stop()
+                except Exception:
+                    pass
             if hasattr(self, '_target'):
-                del self._target, self._args, self._kwargs
+                del self._target, self._args, self._kwargs # type: ignore
             else:
-                del self._Thread__target, self._Thread__args, self._Thread__kwargs
+                del self._Thread__target, self._Thread__args, self._Thread__kwargs # type: ignore
 
-    def stop(self):
+    def stop(self) -> None:
         self.io_loop.add_callback(self.io_loop.stop)
