@@ -7,7 +7,24 @@ from .card import Card
 
 
 class Accordion(NamedListPanel):
-    
+    """
+    The `Accordion` layout is a type of `Card` layout that allows switching
+    between multiple objects by clicking on the corresponding card header.
+
+    The labels for each card will default to the `name` parameter of the card’s
+    contents, but may also be defined explicitly as part of a tuple.
+
+    Like `Column` and `Row`, `Accordion` has a list-like API that allows
+    interactively updating and modifying the cards using the methods `append`,
+    `extend`, `clear`, `insert`, `pop`, `remove` and `__setitem__`.
+
+    Reference: https://panel.holoviz.org/reference/layouts/Accordion.html
+
+    :Example:
+
+    >>> pn.Accordion(some_pane_with_a_name, ("Plot", some_plot))
+    """
+
     active_header_background = param.String(default='#ccc', doc="""
         Color for currently active headers.""")
 
@@ -24,7 +41,7 @@ class Accordion(NamedListPanel):
         Whether to toggle between active cards or allow multiple cards""")
 
     _bokeh_model = BkColumn
-    
+
     _rename = {'active': None, 'active_header_background': None,
                'header_background': None, 'objects': 'children',
                'dynamic': None, 'toggle': None, 'header_color': None}
@@ -43,7 +60,8 @@ class Accordion(NamedListPanel):
     ]
 
     def __init__(self, *objects, **params):
-        super(Accordion, self).__init__(*objects, **params)
+        super().__init__(*objects, **params)
+        self._updating_active = False
         self.param.watch(self._update_active, ['active'])
         self.param.watch(self._update_cards, self._synced_properties)
 
@@ -67,11 +85,12 @@ class Accordion(NamedListPanel):
             if obj not in self.objects:
                 self._panels[id(obj)]._cleanup(root)
 
-        params = {k: v for k, v in self.param.get_param_values()
+        params = {k: v for k, v in self.param.values().items()
                   if k in self._synced_properties}
 
         ref = root.ref['id']
         current_objects = list(self)
+        self._updating_active = True
         for i, (name, pane) in enumerate(zip(self._names, self)):
             params.update(self._apply_style(i))
             if id(pane) in self._panels:
@@ -82,8 +101,9 @@ class Accordion(NamedListPanel):
                     header_css_classes=['accordion-header'],
                     margin=self.margin
                 )
+                card.param.watch(self._set_active, ['collapsed'])
                 self._panels[id(pane)] = card
-            card.param.set_param(**params)
+            card.param.update(**params)
             if ref in card._models:
                 panel = card._models[ref][0]
             else:
@@ -94,8 +114,10 @@ class Accordion(NamedListPanel):
                         panel.js_on_change('collapsed', cb)
                 except RerenderError:
                     return self._get_objects(model, current_objects[:i], doc, root, comm)
-            
             new_models.append(panel)
+
+        self._updating_active = False
+        self._set_active()
         self._update_cards()
         self._update_active()
         return new_models
@@ -103,7 +125,7 @@ class Accordion(NamedListPanel):
     def _cleanup(self, root):
         for panel in self._panels.values():
             panel._cleanup(root)
-        super(Accordion, self)._cleanup(root)
+        super()._cleanup(root)
 
     def _apply_style(self, i):
         if i == 0:
@@ -114,14 +136,40 @@ class Accordion(NamedListPanel):
             margin = (0, 5, 0, 5)
         return dict(margin=margin, collapsed = i not in self.active)
 
+    def _set_active(self, *events):
+        if self._updating_active:
+            return
+        self._updating_active = True
+        try:
+            if self.toggle and events and not events[0].new:
+                active = [list(self._panels.values()).index(events[0].obj)]
+            else:
+                active = []
+                for i, pane in enumerate(self.objects):
+                    if id(pane) not in self._panels:
+                        continue
+                    elif not self._panels[id(pane)].collapsed:
+                        active.append(i)
+
+            if not self.toggle or active:
+                self.active = active
+        finally:
+            self._updating_active = False
+
     def _update_active(self, *events):
-        for i, pane in enumerate(self.objects):
-            if id(pane) not in self._panels:
-                continue
-            self._panels[id(pane)].collapsed = i not in self.active
+        if self._updating_active:
+            return
+        self._updating_active = True
+        try:
+            for i, pane in enumerate(self.objects):
+                if id(pane) not in self._panels:
+                    continue
+                self._panels[id(pane)].collapsed = i not in self.active
+        finally:
+            self._updating_active = False
 
     def _update_cards(self, *events):
-        params = {k: v for k, v in self.param.get_param_values()
+        params = {k: v for k, v in self.param.values().items()
                   if k in self._synced_properties}
         for panel in self._panels.values():
-            panel.param.set_param(**params)
+            panel.param.update(**params)
