@@ -18,8 +18,21 @@ from .base import _col, _row, ListPanel, Panel
 
 class GridBox(ListPanel):
     """
-    List-like Grid which wraps depending on the specified number of
-    rows or columns.
+    The `GridBox` is a list-like layout (unlike `GridSpec`) that wraps objects
+    into a grid according to the specified `nrows` and `ncols` parameters.
+
+    It has a list-like API with methods to `append`, `extend`, `clear`,
+    `insert`, `pop`, `remove` and `__setitem__`, which makes it possible to
+    interactively update and modify the layout.
+
+    Reference: https://panel.holoviz.org/reference/layouts/GridBox.html
+
+    :Example:
+
+    >>> pn.GridBox(
+    ...    python_object_1, python_object_2, ...,
+    ...    python_object_24, ncols=6
+    ... )
     """
 
     nrows = param.Integer(default=None, bounds=(0, None), doc="""
@@ -142,6 +155,7 @@ class GridBox(ListPanel):
         from ..io import state
 
         msg = dict(msg)
+        preprocess = any(self._rename.get(k, k) in self._preprocess_params for k in msg)
         if self._rename['objects'] in msg or 'ncols' in msg or 'nrows' in msg:
             if 'objects' in events:
                 old = events['objects'].old
@@ -160,7 +174,7 @@ class GridBox(ListPanel):
                 if update:
                     return
                 ref = root.ref['id']
-                if ref in state._views:
+                if ref in state._views and preprocess:
                     state._views[ref][0]._preprocess(root)
             finally:
                 Panel._batch_update = update
@@ -190,6 +204,8 @@ class GridSpec(Panel):
 
     _rename = {'objects': 'children', 'mode': None, 'ncols': None, 'nrows': None}
 
+    _preprocess_params = ['objects']
+
     def __init__(self, **params):
         if 'objects' not in params:
             params['objects'] = OrderedDict()
@@ -202,7 +218,7 @@ class GridSpec(Panel):
     @param.depends('nrows', watch=True)
     def _update_nrows(self):
         if not self._updating:
-            self._rows_fixed = self.nrows is not None
+            self._rows_fixed = bool(self.nrows)
 
     @param.depends('ncols', watch=True)
     def _update_ncols(self):
@@ -335,7 +351,7 @@ class GridSpec(Panel):
         -------
         Cloned GridSpec object
         """
-        p = dict(self.param.get_param_values(), **params)
+        p = dict(self.param.values(), **params)
         if not self._cols_fixed:
             del p['ncols']
         if not self._rows_fixed:
@@ -369,9 +385,9 @@ class GridSpec(Panel):
 
         subgrid = self._object_grid[yidx, xidx]
         if isinstance(subgrid, np.ndarray):
-            params = dict(self.param.get_param_values())
+            params = self.param.values()
             params['objects'] = OrderedDict([list(o)[0] for o in subgrid.flatten()])
-            gspec = GridSpec(**params)
+            gspec = type(self)(**params)
             xoff, yoff = gspec._xoffset, gspec._yoffset
             adjusted = []
             for (y0, x0, y1, x1), obj in gspec.objects.items():
