@@ -10,8 +10,25 @@ from .grid import GridSpec
 
 class GridStack(ReactiveHTML, GridSpec):
     """
-    The GridStack layout builds on the GridSpec component and
-    gridstack.js to allow resizing and dragging items in the grid.
+    The `GridStack` layout allows arranging multiple Panel objects in a grid
+    using a simple API to assign objects to individual grid cells or to a grid
+    span.
+
+    Other layout containers function like lists, but a `GridSpec` has an API
+    similar to a 2D array, making it possible to use 2D assignment to populate,
+    index, and slice the grid.
+
+    Reference: https://panel.holoviz.org/reference/layouts/GridStack.html
+
+    :Example:
+
+    >>> pn.extension('gridstack')
+    >>> gstack = GridStack(sizing_mode='stretch_both')
+    >>> gstack[ : , 0: 3] = pn.Spacer(background='red',    margin=0)
+    >>> gstack[0:2, 3: 9] = pn.Spacer(background='green',  margin=0)
+    >>> gstack[2:4, 6:12] = pn.Spacer(background='orange', margin=0)
+    >>> gstack[4:6, 3:12] = pn.Spacer(background='blue',   margin=0)
+    >>> gstack[0:2, 9:12] = pn.Spacer(background='purple', margin=0)
     """
 
     allow_resize = param.Boolean(default=True, doc="""
@@ -28,6 +45,8 @@ class GridStack(ReactiveHTML, GridSpec):
 
     height = param.Integer(default=None)
 
+    _extension_name = 'gridstack'
+
     _template = """
     <div id="grid" class="grid-stack">
     {% for key, obj in objects.items() %}
@@ -156,5 +175,7 @@ class GridStack(ReactiveHTML, GridSpec):
                     properties['height'] = h*height
                 elif 'height' in self.sizing_mode:
                     properties['width'] = w*width
-            obj.param.set_param(**{k: v for k, v in properties.items()
-                                   if not obj.param[k].readonly})
+            obj.param.update(**{
+                k: v for k, v in properties.items()
+                if not obj.param[k].readonly
+            })
