@@ -15,6 +15,7 @@ function unpress(btn_list: HTMLButtonElement[]): void {
 export class PlayerView extends WidgetView {
   model: Player
 
+  protected buttonEl: HTMLDivElement
   protected groupEl: HTMLDivElement
   protected sliderEl: HTMLInputElement
   protected loop_state: HTMLFormElement
@@ -22,20 +23,37 @@ export class PlayerView extends WidgetView {
   protected _toggle_reverse: CallableFunction
   protected _toogle_pause: CallableFunction
   protected _toggle_play: CallableFunction
+  protected _changing: boolean = false
 
   connect_signals(): void {
     super.connect_signals()
-    this.connect(this.model.change, () => this.render())
+    this.connect(this.model.properties.direction.change, () => this.set_direction())
     this.connect(this.model.properties.value.change, () => this.render())
     this.connect(this.model.properties.loop_policy.change, () => this.set_loop_state(this.model.loop_policy))
+    this.connect(this.model.properties.disabled.change, () => this.toggle_disable())
     this.connect(this.model.properties.show_loop_controls.change, () => {
       if (this.model.show_loop_controls && this.loop_state.parentNode != this.groupEl)
         this.groupEl.appendChild(this.loop_state)
       else if (!this.model.show_loop_controls && this.loop_state.parentNode == this.groupEl)
-        this.el.removeChild(this.loop_state)
+        this.groupEl.removeChild(this.loop_state)
     })
   }
 
+  toggle_disable() {
+    this.sliderEl.disabled = this.model.disabled
+    for (const el of this.buttonEl.children){
+      const anyEl = <any>el
+      anyEl.disabled = this.model.disabled
+    }
+
+    for (const el of this.loop_state.children) {
+      if (el.tagName == "input"){
+        const anyEl = <any>el
+        anyEl.disabled = this.model.disabled
+      }
+    }
+  }
+
   get_height(): number {
     return 250
   }
@@ -67,6 +85,7 @@ export class PlayerView extends WidgetView {
 
     // Buttons
     const button_div = div() as any
+    this.buttonEl = button_div
     button_div.style.cssText = "margin: 0 auto; display: flex; padding: 5px; align-items: stretch; width: 100%;"
 
     const button_style_small = "text-align: center; min-width: 20px; flex-grow: 1; margin: 2px";
@@ -187,6 +206,8 @@ export class PlayerView extends WidgetView {
     this.groupEl.appendChild(button_div)
     if (this.model.show_loop_controls)
       this.groupEl.appendChild(this.loop_state)
+
+    this.toggle_disable()
     this.el.appendChild(this.groupEl)
   }
 
@@ -282,29 +303,46 @@ export class PlayerView extends WidgetView {
     }
   }
 
+  set_direction(): void {
+    if (this._changing)
+      return
+    else if (this.model.direction === 0)
+      this.pause_animation()
+    else if(this.model.direction === 1)
+      this.play_animation()
+    else if(this.model.direction === -1)
+      this.reverse_animation()
+  }
+
   pause_animation(): void {
     this._toogle_pause()
-    this.model.direction = 0;
+    this._changing = true
+    this.model.direction = 0
+    this._changing = false
     if (this.timer) {
-      clearInterval(this.timer);
-      this.timer = null;
+      clearInterval(this.timer)
+      this.timer = null
     }
   }
 
   play_animation(): void {
-    this.pause_animation();
+    this.pause_animation()
     this._toggle_play()
-    this.model.direction = 1;
+    this._changing = true
+    this.model.direction = 1
+    this._changing = false
     if (!this.timer)
-      this.timer = setInterval(() => this.anim_step_forward(), this.model.interval);
+      this.timer = setInterval(() => this.anim_step_forward(), this.model.interval)
   }
 
   reverse_animation(): void {
-    this.pause_animation();
+    this.pause_animation()
     this._toggle_reverse()
-    this.model.direction = -1;
+    this._changing = true
+    this.model.direction = -1
+    this._changing = false
     if (!this.timer)
-      this.timer = setInterval(() => this.anim_step_reverse(), this.model.interval);
+      this.timer = setInterval(() => this.anim_step_reverse(), this.model.interval)
   }
 }
 
