@@ -31,7 +31,10 @@ export class QuillInputView extends PanelHTMLBoxView {
       this.quill.enable(!this.model.disabled)
       this._editing = false
     })
-    const {mode, toolbar} = this.model.properties
+    const {mode, toolbar, placeholder} = this.model.properties
+    this.on_change([placeholder], () => {
+      this.quill.root.setAttribute('data-placeholder', this.model.placeholder)
+    })
     this.on_change([mode, toolbar], () => {
       this.render()
       this._layout_toolbar()
@@ -101,7 +104,7 @@ export class QuillInput extends HTMLBox {
   }
 
   static __module__ = "panel.models.quill"
-  
+
   static init_QuillInput(): void {
     this.prototype.default_view = QuillInputView
 
