@@ -11,10 +11,10 @@ from bokeh.model import DataModel
 from bokeh.events import ModelEvent
 
 
-endfor = '{% endfor %}'
-list_iter_re = r'{% for (\s*[A-Za-z_]\w*\s*) in (\s*[A-Za-z_]\w*\s*) %}'
-items_iter_re = r'{% for \s*[A-Za-z_]\w*\s*, (\s*[A-Za-z_]\w*\s*) in (\s*[A-Za-z_]\w*\s*)\.items\(\) %}'
-values_iter_re = r'{% for (\s*[A-Za-z_]\w*\s*) in (\s*[A-Za-z_]\w*\s*)\.values\(\) %}'
+endfor = '{%-? endfor -?%}'
+list_iter_re = r'{%-? for (\s*[A-Za-z_]\w*\s*) in (\s*[A-Za-z_]\w*\s*) -?%}'
+items_iter_re = r'{%-? for \s*[A-Za-z_]\w*\s*, (\s*[A-Za-z_]\w*\s*) in (\s*[A-Za-z_]\w*\s*)\.items\(\) -?%}'
+values_iter_re = r'{%-? for (\s*[A-Za-z_]\w*\s*) in (\s*[A-Za-z_]\w*\s*)\.values\(\) -?%}'
 
 
 class ReactiveHTMLParser(HTMLParser):
@@ -117,9 +117,11 @@ class ReactiveHTMLParser(HTMLParser):
                                  f'template section:\n\n{data}')
             self.loop_map[var] = obj
 
-        if '{% for ' in data:
+        open_for = re.search(r'{%-? for', data)
+        end_for = re.search(endfor, data)
+        if open_for:
             self._open_for = True
-        if endfor in data and (not nloops or data.index(endfor) > data.index('{% for ')):
+        if end_for and (not nloops or end_for.start() > open_for.start()):
             self._open_for = False
 
         if not (self._current_node and matches):
