@@ -237,13 +237,13 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   _selection_updating: boolean = false
   _initializing: boolean
   _lastVerticalScrollbarTopPosition: number = 0;
-  _timeout_queue: any[] = []
+  _applied_styles: boolean = false
 
   connect_signals(): void {
     super.connect_signals()
 
     const {configuration, layout, columns, theme, groupby} = this.model.properties;
-    this.on_change([configuration, layout, columns, groupby], debounce(() => this.render(), 20, false))
+    this.on_change([configuration, layout, columns, groupby], debounce(() => this.invalidate_render(), 20, false))
 
     this.on_change([theme], () => this.setCSS())
 
@@ -261,7 +261,11 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       }
     })
 
-    this.connect(this.model.properties.styles.change, () => this.redraw())
+    this.connect(this.model.properties.styles.change, () => {
+      if (this._applied_styles)
+	this.tabulator.redraw(true)
+      this.setStyles()
+    })
     this.connect(this.model.properties.hidden_columns.change, () => this.setHidden())
     this.connect(this.model.properties.page_size.change, () => this.setPageSize())
     this.connect(this.model.properties.page.change, () => {
@@ -277,7 +281,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       const inds = this.model.source.selected.indices
       this.updateOrAddData();
       this.tabulator.rowManager.element.scrollTop = this._lastVerticalScrollbarTopPosition;
-      // Restore indices since updating data may have reset checkbox column 
+      // Restore indices since updating data may have reset checkbox column
       this.model.source.selected.indices = inds;
     })
     this.connect(this.model.source.selected.change, () => this.setSelection())
@@ -286,7 +290,9 @@ export class DataTabulatorView extends PanelHTMLBoxView {
 
   get sorters(): any[] {
     const sorters = []
-    for (const sort of this.model.sorters) {
+    if (this.model.sorters.length)
+      sorters.push({column: '_index', dir: 'asc'})
+    for (const sort of this.model.sorters.reverse()) {
       if (sort.column === undefined)
 	sort.column = sort.field
       sorters.push(sort)
@@ -294,20 +300,22 @@ export class DataTabulatorView extends PanelHTMLBoxView {
     return sorters
   }
 
+  invalidate_render(): void {
+    this.tabulator.destroy()
+    this.tabulator = null
+    this.render()
+    this.relayout()
+  }
+
   renderComplete(): void {
     // Only have to set up styles after initial render subsequent
     // styling is handled by change event on styles property
-    if (this._timeout_queue.length) {
-      const timeout = this._timeout_queue.shift()
-      clearTimeout(timeout)
-    }
-
     if (this._initializing) {
-      this.setStyles()
       this.setSelection()
       this.relayout()
       this._initializing = false
-      this.redraw()
+      if (this._lastVerticalScrollbarTopPosition)
+	this.tabulator.rowManager.element.scrollTop = this._lastVerticalScrollbarTopPosition;
     }
   }
 
@@ -354,9 +362,6 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       this.setFrozen()
 
     this.el.appendChild(container)
-
-    // Ensure renderComplete is run eventually
-    this._timeout_queue.push(setTimeout(() => this.renderComplete(), 500))
   }
 
   /*
@@ -471,7 +476,12 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       try {
         if (page != null && sorters != null) {
 	  this._updating_sort = true
-          this.model.sorters = sorters
+	  const sorts = []
+	  for (const s of sorters) {
+	    if (s.field !== '_index')
+	      sorts.push({field: s.field, dir: s.dir})
+	  }
+          this.model.sorters = sorts
 	  this._updating_sort = false
           this._updating_page = true
           try {
@@ -517,7 +527,8 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       dataSorting: (sorters: any[]) => {
 	const sorts = []
 	for (const s of sorters) {
-	  sorts.push({field: s.field, dir: s.dir})
+	  if (s.field !== '_index')
+	    sorts.push({field: s.field, dir: s.dir})
 	}
 	if (this.model.pagination !== 'remote') {
 	  this._updating_sort = true
@@ -638,6 +649,8 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       }
     }
     this.model.expanded = expanded
+    const icon = expanded.indexOf(index) < 0 ? "\u25ba" : "\u25bc"
+    cell._cell.element.innerText = icon
     if (expanded.indexOf(index) < 0)
       return
     let ready = true
@@ -661,6 +674,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   getColumns(): any {
     const config_columns: (any[] | undefined) = this.model.configuration?.columns;
     let columns = []
+    columns.push({field: '_index', frozen: true})
     if (config_columns != null) {
       for (const column of config_columns)
         if (column.columns != null) {
@@ -937,6 +951,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   setStyles(): void {
     if (this.tabulator == null || this.tabulator.getDataCount() == 0)
       return
+    this._applied_styles = false
     for (const r in this.model.styles.data) {
       const row_style = this.model.styles.data[r]
       const row = this.tabulator.getRow(r)
@@ -958,6 +973,7 @@ export class DataTabulatorView extends PanelHTMLBoxView {
           else
             [prop, value] = s.split(':')
           element.style.setProperty(prop, value.trimLeft())
+	  this._applied_styles = true
         }
       }
     }
@@ -965,7 +981,8 @@ export class DataTabulatorView extends PanelHTMLBoxView {
 
   setHidden(): void {
     for (const column of this.tabulator.getColumns()) {
-      if (this.model.hidden_columns.indexOf(column._column.field) > -1)
+      const col = column._column
+      if ((col.field == '_index') || (this.model.hidden_columns.indexOf(col.field) > -1))
         column.hide()
       else
         column.show()
@@ -1006,9 +1023,15 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   }
 
   // Update model
-
   rowClicked(e: any, row: any) {
-    if (this._selection_updating || this._initializing || (typeof this.model.select_mode) === 'string' || this.model.select_mode === false || this.model.configuration.dataTree)
+    if (
+        this._selection_updating ||
+	this._initializing ||
+	(typeof this.model.select_mode) === 'string' ||
+	this.model.select_mode === false ||  // selection disabled
+	this.model.configuration.dataTree || // dataTree does not support selection
+	e.srcElement?.innerText === "►"      // expand button
+    )
       return
     let indices: number[] = []
     const selected = this.model.source.selected
@@ -1054,7 +1077,13 @@ export class DataTabulatorView extends PanelHTMLBoxView {
   }
 
   rowSelectionChanged(data: any, _: any): void {
-    if (this._selection_updating || this._initializing || (typeof this.model.select_mode) === 'boolean' || (typeof this.model.select_mode) === 'number' || this.model.configuration.dataTree)
+    if (
+        this._selection_updating ||
+	this._initializing ||
+	(typeof this.model.select_mode) === 'boolean' ||
+	(typeof this.model.select_mode) === 'number' ||
+	this.model.configuration.dataTree
+    )
       return
     const indices: number[] = data.map((row: any) => row._index)
     const filtered = this._filter_selected(indices)
@@ -1076,6 +1105,8 @@ export class DataTabulatorView extends PanelHTMLBoxView {
       this._tabulator_cell_updating = false
     }
     this.model.trigger_event(new TableEditEvent(field, index))
+    this.setSorters()
+    this.tabulator.scrollToRow(index, "top", false)
   }
 }
 
