@@ -1,16 +1,45 @@
 import * as p from "@bokehjs/core/properties"
+import {ModelEvent, JSON} from "@bokehjs/core/bokeh_events"
 import {isArray} from "@bokehjs/core/util/types"
 import {HTMLBox, HTMLBoxView} from "@bokehjs/models/layouts/html_box"
 
+import {debounce} from  "debounce"
+
+export class VegaEvent extends ModelEvent {
+  event_name: string = "vega_event"
+
+  constructor(readonly data: any) {
+    super()
+  }
+
+  protected _to_json(): JSON {
+    return {model: this.origin, data: this.data}
+  }
+}
+
 export class VegaPlotView extends HTMLBoxView {
   model: VegaPlot
   vega_view: any
+  _callbacks: string[]
   _connected: string[]
 
   connect_signals(): void {
     super.connect_signals()
-    this.connect(this.model.properties.data.change, this._plot)
+    const {data, show_actions, theme} = this.model.properties
+    this.on_change([data, show_actions, theme], () => {
+      this._plot()
+    })
     this.connect(this.model.properties.data_sources.change, () => this._connect_sources())
+    this.connect(this.model.properties.events.change, () => {
+      for (const event of this.model.events) {
+        if (this._callbacks.indexOf(event) > -1)
+          continue
+        this._callbacks.push(event)
+	const callback = (name: string, value: any) => this._dispatch_event(name, value)
+	const timeout = this.model.throttle[event] || 20
+        this.vega_view.addSignalListener(event, debounce(callback, timeout, false))
+      }
+    })
     this._connected = []
     this._connect_sources()
   }
@@ -25,6 +54,16 @@ export class VegaPlotView extends HTMLBoxView {
     }
   }
 
+  _dispatch_event(name: string, value: any): void {
+    if ('vlPoint' in value && value.vlPoint.or != null) {
+      const indexes = []
+      for (const index of value.vlPoint.or)
+	indexes.push(index._vgsid_)
+      value = indexes
+    }
+    this.model.trigger_event(new VegaEvent({type: name, value: value}))
+  }
+
   _fetch_datasets() {
     const datasets: any = {}
     for (const ds in this.model.data_sources) {
@@ -45,6 +84,7 @@ export class VegaPlotView extends HTMLBoxView {
 
   render(): void {
     super.render()
+    this._callbacks = []
     this._plot()
   }
 
@@ -69,12 +109,19 @@ export class VegaPlotView extends HTMLBoxView {
       }
       this.model.data['datasets'] = datasets
     }
-    (window as any).vegaEmbed(this.el, this.model.data, {actions: false}).then((result: any) => {
+    const config: any = {actions: this.model.show_actions, theme: this.model.theme};
+    (window as any).vegaEmbed(this.el, this.model.data, config).then((result: any) => {
       this.vega_view = result.view
       this.relayout()
       if (this.vega_view._viewHeight <= 0 || this.vega_view._viewWidth <= 0) {
 	(window as any).dispatchEvent(new Event('resize'));
       }
+      const callback = (name: string, value: any) => this._dispatch_event(name, value)
+      for (const event of this.model.events) {
+	this._callbacks.push(event)
+	const timeout = this.model.throttle[event] || 20
+	this.vega_view.addSignalListener(event, debounce(callback, timeout, false))
+      }
     })
   }
 
@@ -104,6 +151,10 @@ export namespace VegaPlot {
   export type Props = HTMLBox.Props & {
     data: p.Property<any>
     data_sources: p.Property<any>
+    events: p.Property<string[]>
+    show_actions: p.Property<boolean>
+    theme: p.Property<string | null>
+    throttle: p.Property<any>
   }
 }
 
@@ -121,9 +172,13 @@ export class VegaPlot extends HTMLBox {
   static init_VegaPlot(): void {
     this.prototype.default_view = VegaPlotView
 
-    this.define<VegaPlot.Props>(({Any}) => ({
-      data:         [ Any, {} ],
-      data_sources: [ Any, {} ],
+    this.define<VegaPlot.Props>(({Any, Array, Boolean, String}) => ({
+      data:         [ Any,           {} ],
+      data_sources: [ Any,           {} ],
+      events:       [ Array(String), [] ],
+      show_actions: [ Boolean,    false ],
+      theme:        [ String,           ],
+      throttle:     [ Any,           {} ]
     }))
   }
 }
