@@ -1,15 +1,14 @@
 """
 Custom bokeh Widget models.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 from bokeh.core.enums import ButtonType
 
 from bokeh.core.properties import (
-    Int, Float, Override, Enum, Any, Bool, Dict, String, List, Either, Tuple
+    Any, Bool, Dict, Either, Enum, Float, Int, List, Nullable,
+    Override, String, Tuple
 )
 from bokeh.models.layouts import HTMLBox
-from bokeh.models.widgets import InputWidget, Widget
+from bokeh.models.widgets import InputWidget, Select, Widget
 
 
 class Player(Widget):
@@ -17,9 +16,9 @@ class Player(Widget):
     The Player widget provides controls to play through a number of frames.
     """
 
-    start = Int(help="Lower bound of the Player slider")
+    start = Int(0, help="Lower bound of the Player slider")
 
-    end = Int(help="Upper bound of the Player slider")
+    end = Int(10, help="Upper bound of the Player slider")
 
     value = Int(0, help="Current value of the player app")
 
@@ -46,6 +45,10 @@ class SingleSelect(InputWidget):
 
     '''
 
+    disabled_options = List(Any, default=[], help="""
+    List of options to disable.
+    """)
+
     options = List(Either(String, Tuple(String, String)), help="""
     Available selection options. Options may be provided either as a list of
     possible string values, or as a list of tuples, each of the form
@@ -68,6 +71,10 @@ class Audio(HTMLBox):
 
     paused = Bool(False, help="""Whether the audio is paused""")
 
+    muted = Bool(False, help="""Whether the audio is muted""")
+
+    autoplay = Bool(False, help="""Whether the audio is playing automatically""")
+
     time = Float(0, help="""
         The current time stamp of the audio playback""")
 
@@ -85,6 +92,10 @@ class Video(HTMLBox):
 
     paused = Bool(False, help="""Whether the video is paused""")
 
+    muted = Bool(False, help="""Whether the video is muted""")
+
+    autoplay = Bool(False, help="""Whether the video is playing automatically""")
+
     time = Float(0, help="""
         The current time stamp of the video playback""")
 
@@ -93,7 +104,7 @@ class Video(HTMLBox):
 
     value = Any(help="Encoded file data")
 
-    volume = Int(0, help="""The volume of the video player.""")
+    volume = Int(help="""The volume of the video player.""")
 
 
 class VideoStream(HTMLBox):
@@ -104,7 +115,7 @@ class VideoStream(HTMLBox):
 
     snapshot = Bool(False, help="""On change generate a snapshot of the current video frame""")
 
-    timeout = Float(None, help="""
+    timeout = Nullable(Int, help="""
         The timeout between snapshots (if None snapshot only generated
         when snapshot property is changed""")
 
@@ -124,7 +135,7 @@ class Progress(HTMLBox):
 
     max = Int(100, help="""Maximum value""")
 
-    value = Int(help="""Current value""")
+    value = Nullable(Int, help="""Current value""")
 
     style = Dict(String, Any, default={}, help="""
     Raw CSS style declaration. Note this may be web browser dependent.
@@ -154,3 +165,13 @@ class FileDownload(InputWidget):
     """)
 
     title = Override(default='')
+
+
+class CustomSelect(Select):
+    ''' Custom widget that extends the base Bokeh Select
+    by adding a parameter to disable one or more options.
+
+    '''
+    disabled_options = List(Any, default=[], help="""
+    List of options to disable.
+    """)
