@@ -2,35 +2,61 @@
 Defines the PaneBase class defining the API for panes which convert
 objects to a visual representation expressed as a bokeh model.
 """
+from __future__ import annotations
+
+import warnings
+
 from functools import partial
+from typing import (
+    TYPE_CHECKING, Any, Callable, List, Optional, TypeVar, Type
+)
 
 import param
 
-from bokeh.models.layouts import GridBox as _BkGridBox
+from bokeh.models.layouts import (
+    GridBox as _BkGridBox, Tabs as _BkTabs, Panel as _BkPanel
+)
 
 from ..io import init_doc, push, state, unlocked
 from ..layout import Panel, Row
 from ..links import Link
 from ..models import ReactiveHTML as _BkReactiveHTML
 from ..reactive import Reactive
-from ..viewable import Layoutable, Viewable
 from ..util import param_reprs
+from ..viewable import Layoutable, Viewable, Viewer
+
+if TYPE_CHECKING:
+    from bokeh.document import Document
+    from bokeh.model import Model
+    from pyviz_comms import Comm
 
 
-def Pane(obj, **kwargs):
+def Pane(obj: Any, **kwargs) -> 'PaneBase':
     """
     Converts any object to a Pane if a matching Pane class exists.
     """
+    warnings.warn(
+        'panel.Pane(...) is deprecated, use panel.panel(...) instead.',
+        DeprecationWarning
+    )
     if isinstance(obj, Viewable):
         return obj
     return PaneBase.get_pane_type(obj, **kwargs)(obj, **kwargs)
 
 
-def panel(obj, **kwargs):
+def panel(obj: Any, **kwargs) -> Viewable:
     """
     Creates a panel from any supplied object by wrapping it in a pane
     and returning a corresponding Panel.
 
+    If you provide a "reactive function" as `obj` and set
+    `loading_indicator=True`, then Panel will display a loading indicator
+    when invoking the function.
+
+    Reference: https://panel.holoviz.org/user_guide/Components.html#panes
+
+    >>> pn.panel(some_python_object, width=500, background="whitesmoke")
+
     Arguments
     ---------
     obj: object
@@ -46,6 +72,8 @@ def panel(obj, **kwargs):
     if isinstance(obj, Viewable):
         return obj
     elif hasattr(obj, '__panel__'):
+        if not isinstance(obj, Viewer) and issubclass(obj, Viewer):
+            return panel(obj().__panel__())
         return panel(obj.__panel__())
     if kwargs.get('name', False) is None:
         kwargs.pop('name')
@@ -60,11 +88,14 @@ class RerenderError(RuntimeError):
     Error raised when a pane requests re-rendering during initial render.
     """
 
+T = TypeVar('T', bound='PaneBase')
 
 class PaneBase(Reactive):
     """
     PaneBase is the abstract baseclass for all atomic displayable units
-    in the panel library. Pane defines an extensible interface for
+    in the panel library. We call any child class of `PaneBase` a `Pane`.
+
+    Panes defines an extensible interface for
     wrapping arbitrary objects and transforming them into Bokeh models.
 
     Panes are reactive in the sense that when the object they are
@@ -88,19 +119,19 @@ class PaneBase(Reactive):
     # numerical priority is selected. The default is an intermediate value.
     # If set to None, applies method will be called to get a priority
     # value for a specific object type.
-    priority = 0.5
+    priority: float | bool | None = 0.5
 
     # Whether applies requires full set of keywords
     _applies_kw = False
 
     # Whether the Pane layout can be safely unpacked
-    _unpack = True
+    _unpack: bool = True
 
     # Declares whether Pane supports updates to the Bokeh model
-    _updates = False
+    _updates: bool = False
 
     # List of parameters that trigger a rerender of the Bokeh model
-    _rerender_params = ['object']
+    _rerender_params: List[str] = ['object']
 
     __abstract = True
 
@@ -137,15 +168,17 @@ class PaneBase(Reactive):
     #----------------------------------------------------------------
 
     @property
-    def _linkable_params(self):
+    def _linkable_params(self) -> List[str]:
         return [p for p in self._synced_params if self._rename.get(p, False) is not None]
 
     @property
-    def _synced_params(self):
+    def _synced_params(self) -> List[str]:
         ignored_params = ['name', 'default_layout', 'loading']+self._rerender_params
         return [p for p in self.param if p not in ignored_params]
 
-    def _update_object(self, ref, doc, root, parent, comm):
+    def _update_object(
+        self, ref: str, doc: 'Document', root: 'Model', parent: 'Model', comm: Optional['Comm']
+    ) -> None:
         old_model = self._models[ref][0]
         if self._updates:
             self._update(ref, old_model)
@@ -153,8 +186,10 @@ class PaneBase(Reactive):
             new_model = self._get_model(doc, root, parent, comm)
             try:
                 if isinstance(parent, _BkGridBox):
-                    indexes = [i for i, child in enumerate(parent.children)
-                               if child[0] is old_model]
+                    indexes = [
+                        i for i, child in enumerate(parent.children)
+                        if child[0] is old_model
+                    ]
                     if indexes:
                         index = indexes[0]
                     else:
@@ -167,6 +202,8 @@ class PaneBase(Reactive):
                             new_models = list(children)
                             new_models[index] = new_model
                             break
+                elif isinstance(parent, _BkTabs):
+                    index = [tab.child for tab in parent.tabs].index(old_model)
                 else:
                     index = parent.children.index(old_model)
             except ValueError:
@@ -178,6 +215,10 @@ class PaneBase(Reactive):
             else:
                 if isinstance(parent, _BkReactiveHTML):
                     parent.children[node] = new_models
+                elif isinstance(parent, _BkTabs):
+                    old_tab = parent.tabs[index]
+                    props = dict(old_tab.properties_with_values(), child=new_model)
+                    parent.tabs[index] = _BkPanel(**props)
                 else:
                     parent.children[index] = new_model
 
@@ -186,7 +227,7 @@ class PaneBase(Reactive):
         if ref in state._views:
             state._views[ref][0]._preprocess(root)
 
-    def _update_pane(self, *events):
+    def _update_pane(self, *events) -> None:
         for ref, (_, parent) in self._models.items():
             if ref not in state._views or ref in state._fake_roots:
                 continue
@@ -203,7 +244,7 @@ class PaneBase(Reactive):
                 else:
                     cb()
 
-    def _update(self, ref=None, model=None):
+    def _update(self, ref: Optional[str] = None, model: Optional['Model'] = None) -> None:
         """
         If _updates=True this method is used to update an existing
         Bokeh model instead of replacing the model entirely. The
@@ -216,44 +257,50 @@ class PaneBase(Reactive):
     #----------------------------------------------------------------
 
     @classmethod
-    def applies(cls, obj):
+    def applies(cls, obj: Any) -> float | bool | None:
         """
-        Given the object return a boolean indicating whether the Pane
-        can render the object. If the priority of the pane is set to
-        None, this method may also be used to define a priority
+        Returns boolean or float indicating whether the Pane
+        can render the object.
+
+        If the priority of the pane is set to
+        `None`, this method may also be used to define a float priority
         depending on the object being rendered.
         """
         return None
 
-    def clone(self, object=None, **params):
+    def clone(self: T, object: Optional[Any] = None, **params) -> T:
         """
         Makes a copy of the Pane sharing the same parameters.
 
         Arguments
         ---------
+        object: Optional new object to render
         params: Keyword arguments override the parameters on the clone.
 
         Returns
         -------
         Cloned Pane object
         """
-        params = dict(self.param.get_param_values(), **params)
+        params = dict(self.param.values(), **params)
         old_object = params.pop('object')
         if object is None:
             object = old_object
         return type(self)(object, **params)
 
-    def get_root(self, doc=None, comm=None, preprocess=True):
+    def get_root(
+        self, doc: Optional['Document'] = None, comm: Optional['Comm'] = None,
+        preprocess: bool = True
+    ) -> 'Model':
         """
         Returns the root model and applies pre-processing hooks
 
         Arguments
         ---------
-        doc: bokeh.Document
-          Bokeh document the bokeh model will be attached to.
+        doc: bokeh.document.Document
+          Optional Bokeh document the bokeh model will be attached to.
         comm: pyviz_comms.Comm
           Optional pyviz_comms when working in notebook
-        preprocess: boolean (default=True)
+        preprocess: bool (default=True)
           Whether to run preprocessing hooks
 
         Returns
@@ -272,7 +319,7 @@ class PaneBase(Reactive):
         return root
 
     @classmethod
-    def get_pane_type(cls, obj, **kwargs):
+    def get_pane_type(cls, obj: Any, **kwargs) -> Type['PaneBase']:
         """
         Returns the applicable Pane type given an object by resolving
         the precedence of all types whose applies method declares that
@@ -280,7 +327,7 @@ class PaneBase(Reactive):
 
         Arguments
         ---------
-        obj (object): The object type to return a Pane for
+        obj (object): The object type to return a Pane type for
 
         Returns
         -------
@@ -329,11 +376,11 @@ class ReplacementPane(PaneBase):
     on.
     """
 
-    _updates = True
+    _updates: bool = True
 
     __abstract = True
 
-    def __init__(self, object=None, **params):
+    def __init__(self, object: Any=None, **params):
         self._kwargs =  {p: params.pop(p) for p in list(params)
                          if p not in self.param}
         super().__init__(object, **params)
@@ -354,13 +401,13 @@ class ReplacementPane(PaneBase):
         """
 
     @classmethod
-    def _update_from_object(cls, object, old_object, was_internal, **kwargs):
+    def _update_from_object(cls, object: Any, old_object: Any, was_internal: bool, **kwargs):
         pane_type = cls.get_pane_type(object)
         try:
             links = Link.registry.get(object)
         except TypeError:
             links = []
-        custom_watchers = False
+        custom_watchers = []
         if isinstance(object, Reactive):
             watchers = [
                 w for pwatchers in object._param_watchers.values()
@@ -373,10 +420,10 @@ class ReplacementPane(PaneBase):
             # If the object has not external referrers we can update
             # it inplace instead of replacing it
             if isinstance(object, Reactive):
-                pvals = dict(old_object.param.get_param_values())
-                new_params = {k: v for k, v in object.param.get_param_values()
+                pvals = old_object.param.values()
+                new_params = {k: v for k, v in object.param.values().items()
                               if k != 'name' and v is not pvals[k]}
-                old_object.param.set_param(**new_params)
+                old_object.param.update(**new_params)
             else:
                 old_object.object = object
         else:
@@ -397,8 +444,8 @@ class ReplacementPane(PaneBase):
                 internal = object is not old_object
         return pane, internal
 
-    def _update_inner(self, new_object):
-        kwargs = dict(self.param.get_param_values(), **self._kwargs)
+    def _update_inner(self, new_object: Any) -> None:
+        kwargs = dict(self.param.values(), **self._kwargs)
         del kwargs['object']
         new_pane, internal = self._update_from_object(
             new_object, self._pane, self._internal, **kwargs
@@ -410,7 +457,10 @@ class ReplacementPane(PaneBase):
         self._inner_layout[0] = self._pane
         self._internal = internal
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
+    def _get_model(
+        self, doc: 'Document', root: Optional['Model'] = None,
+        parent: Optional['Model'] = None, comm: Optional['Comm'] = None
+    ) -> 'Model':
         if root:
             ref = root.ref['id']
             if ref in self._models:
@@ -421,18 +471,18 @@ class ReplacementPane(PaneBase):
         self._models[ref] = (model, parent)
         return model
 
-    def _cleanup(self, root=None):
+    def _cleanup(self, root: 'Model' | None = None) -> None:
         self._inner_layout._cleanup(root)
         super()._cleanup(root)
 
-    def select(self, selector=None):
+    def select(self, selector: type | Callable | None = None) -> List[Viewable]:
         """
         Iterates over the Viewable and any potential children in the
         applying the Selector.
 
         Arguments
         ---------
-        selector: type or callable or None
+        selector: (type | callable | None)
           The selector allows selecting a subset of Viewables by
           declaring a type or callable function to filter by.
 
