@@ -47,15 +47,26 @@ def _wrap_callback(cb, wrapped, doc, comm, callbacks):
 
 class Bokeh(PaneBase):
     """
-    Bokeh panes allow including any Bokeh model in a panel.
+    The Bokeh pane allows displaying any displayable Bokeh model inside a
+    Panel app.
+
+    Reference: https://panel.holoviz.org/reference/panes/Bokeh.html
+
+    :Example:
+
+    >>> Bokeh(some_bokeh_figure)
     """
 
+    autodispatch = param.Boolean(default=True, doc="""
+        Whether to automatically dispatch events inside bokeh on_change
+        and on_event callbacks in the notebook.""")
+
     theme = param.ClassSelector(default=None, class_=(Theme, str), doc="""
         Bokeh theme to apply to the plot.""")
 
     priority = 0.8
 
-    _rename = {'theme': None}
+    _rename = {'autodispatch': None, 'theme': None}
 
     @classmethod
     def applies(cls, obj):
@@ -93,7 +104,7 @@ class Bokeh(PaneBase):
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
         if root is None:
-            return self._get_root(doc, comm)
+            return self.get_root(doc, comm)
 
         if self.object is None:
             model = BkSpacer()
@@ -101,13 +112,13 @@ class Bokeh(PaneBase):
             model = self.object
 
         properties = {}
-        for p, value in self.param.get_param_values():
+        for p, value in self.param.values().items():
             if (p not in Layoutable.param or p == 'name' or
                 value is self.param[p].default):
                 continue
             properties[p] = value
         model.update(**properties)
-        if comm:
+        if comm and self.autodispatch:
             self._wrap_bokeh_callbacks(root, model, doc, comm)
 
         ref = root.ref['id']
@@ -127,11 +138,20 @@ class Bokeh(PaneBase):
 
 class Matplotlib(PNG, IPyWidget):
     """
-    A Matplotlib pane renders a matplotlib figure to png and wraps the
-    base64 encoded data in a bokeh Div model. The size of the image in
-    pixels is determined by scaling the size of the figure in inches
-    by a dpi of 72, increasing the dpi therefore controls the
-    resolution of the image not the displayed size.
+    The `Matplotlib` pane allows displaying any displayable Matplotlib figure
+    inside a Panel app.
+
+    - It will render the plot to PNG at the declared DPI and then embed it.
+    - If you find the figure to be clipped on the edges, you can set `tight=True`
+    to automatically resize objects to fit within the pane.
+    - If you have installed `ipympl` you will also be able to use the
+    interactive backend.
+
+    Reference: https://panel.holoviz.org/reference/panes/Matplotlib.html
+
+    :Example:
+
+    >>> Matplotlib(some_matplotlib_figure, dpi=144)
     """
 
     dpi = param.Integer(default=144, bounds=(1, None), doc="""
@@ -164,7 +184,7 @@ class Matplotlib(PNG, IPyWidget):
         self._managers = {}
 
     def _get_widget(self, fig):
-        import matplotlib
+        import matplotlib.backends
         old_backend = getattr(matplotlib.backends, 'backend', 'agg')
 
         from ipympl.backend_nbagg import FigureManager, Canvas, is_interactive
@@ -193,6 +213,10 @@ class Matplotlib(PNG, IPyWidget):
         props = self._process_param_change(self._init_params())
         kwargs = {k: v for k, v in props.items()
                   if k not in self._rerender_params+['interactive']}
+        w, h = self.object.get_size_inches()
+        kwargs['width'] = self.width or int(self.dpi * w)
+        kwargs['height'] = self.height or int(self.dpi * h)
+        kwargs['sizing_mode'] = self.sizing_mode
         model = self._get_ipywidget(manager.canvas, doc, root, comm,
                                     **kwargs)
         if root is None:
@@ -216,11 +240,6 @@ class Matplotlib(PNG, IPyWidget):
             manager.canvas.handle_resize(event)
         manager.canvas.draw_idle()
 
-    def _imgshape(self, data):
-        """Calculate and return image width,height"""
-        w, h = self.object.get_size_inches()
-        return int(w*72), int(h*72)
-
     def _data(self):
         self.object.set_dpi(self.dpi)
         b = BytesIO()
