@@ -13,14 +13,24 @@ from ..util import isdatetime, lazy_load
 from ..viewable import Layoutable
 
 
-
 class Plotly(PaneBase):
     """
-    Plotly panes allow rendering plotly Figures and traces.
+    The `Plotly` pane renders Plotly plots inside a panel.
 
-    For efficiency any array objects found inside a Figure are added
-    to a ColumnDataSource which allows using binary transport to sync
-    the figure on bokeh server and via Comms.
+    Note that
+
+    - the Panel `extension` has to be loaded with `plotly` as an argument to
+    ensure that Plotly.js is initialized.
+    - it supports click, hover and selection events.
+    - it optimizes the plot rendering by using binary serialization for any
+    array data found on the Plotly object.
+
+    Reference: https://panel.holoviz.org/reference/panes/Plotly.html
+
+    :Example:
+
+    >>> pn.extension('plotly')
+    >>> Plotly(some_plotly_figure, width=500, height=500)
     """
 
     click_data = param.Dict(doc="Click callback data")
@@ -45,7 +55,7 @@ class Plotly(PaneBase):
         * "mouseup": updates are synchronized when mouse button is
           released after panning
         * "continuous": updates are synchronized continually while panning
-        * "throttle": updates are synchronized while panning, at 
+        * "throttle": updates are synchronized while panning, at
           intervals determined by the viewport_update_throttle parameter
         """, objects=["mouseup", "continuous", "throttle"])
 
@@ -69,6 +79,7 @@ class Plotly(PaneBase):
     def __init__(self, object=None, **params):
         super().__init__(object, **params)
         self._figure = None
+        self._event = None
         self._update_figure()
 
     def _to_figure(self, obj):
@@ -97,7 +108,7 @@ class Plotly(PaneBase):
     @staticmethod
     def _get_sources_for_trace(json, data, parent_path=''):
         for key, value in list(json.items()):
-            full_path = key if not parent_path else (parent_path + '.' + key)
+            full_path = key if not parent_path else "{}.{}".format(parent_path, key)
             if isinstance(value, np.ndarray):
                 # Extract numpy array
                 data[full_path] = [json.pop(key)]
@@ -116,24 +127,69 @@ class Plotly(PaneBase):
     def _update_figure(self):
         import plotly.graph_objs as go
 
-        if (self.object is None or
-                type(self.object) is not go.Figure or
-                self.object is self._figure):
+        if (self.object is None or type(self.object) is not go.Figure or
+            self.object is self._figure):
             return
 
         # Monkey patch the message stubs used by FigureWidget.
         # We only patch `Figure` objects (not subclasses like FigureWidget) so
         # we don't interfere with subclasses that override these methods.
         fig = self.object
-        fig._send_addTraces_msg = lambda *_, **__: self.param.trigger('object')
-        fig._send_moveTraces_msg = lambda *_, **__: self.param.trigger('object')
-        fig._send_deleteTraces_msg = lambda *_, **__: self.param.trigger('object')
-        fig._send_restyle_msg = lambda *_, **__: self.param.trigger('object')
-        fig._send_relayout_msg = lambda *_, **__: self.param.trigger('object')
-        fig._send_update_msg = lambda *_, **__: self.param.trigger('object')
-        fig._send_animate_msg = lambda *_, **__: self.param.trigger('object')
+        fig._send_addTraces_msg = lambda *_, **__: self._update_from_figure('add')
+        fig._send_moveTraces_msg = lambda *_, **__: self._update_from_figure('move')
+        fig._send_deleteTraces_msg = lambda *_, **__: self._update_from_figure('delete')
+        fig._send_restyle_msg = self._send_restyle_msg
+        fig._send_relayout_msg = self._send_relayout_msg
+        fig._send_update_msg = self._send_update_msg
+        fig._send_animate_msg = lambda *_, **__: self._update_from_figure('animate')
         self._figure = fig
 
+    def _send_relayout_msg(self, relayout_data, source_view_id=None):
+        self._send_update_msg({}, relayout_data, None, source_view_id)
+
+    def _send_restyle_msg(self, restyle_data, trace_indexes=None, source_view_id=None):
+        self._send_update_msg(restyle_data, {}, trace_indexes, source_view_id)
+
+    @param.depends('restyle_data', watch=True)
+    def _update_figure_style(self):
+        if self._figure is None or self.restyle_data is None:
+            return
+        self._figure.plotly_restyle(*self.restyle_data)
+
+    @param.depends('relayout_data', watch=True)
+    def _update_figure_layout(self):
+        if self._figure is None or self.relayout_data is None:
+            return
+        relayout_data = self._clean_relayout_data(self.relayout_data)
+        self._figure.plotly_relayout(relayout_data)
+
+    @staticmethod
+    def _clean_relayout_data(relayout_data):
+        return {
+            key: val for key, val in relayout_data.items() if not key.endswith("._derived")
+        }
+
+    def _send_update_msg(
+        self, restyle_data, relayout_data, trace_indexes=None, source_view_id=None
+    ):
+        if source_view_id:
+            return
+        trace_indexes = self._figure._normalize_trace_indexes(trace_indexes)
+        msg = {}
+        if relayout_data:
+            msg['relayout'] = relayout_data
+        if restyle_data:
+            msg['restyle'] = {'data': restyle_data, 'traces': trace_indexes}
+        for ref, (m, _) in self._models.items():
+            self._apply_update([], msg, m, ref)
+
+    def _update_from_figure(self, event, *args, **kwargs):
+        self._event = event
+        try:
+            self.param.trigger('object')
+        finally:
+            self._event = None
+
     def _update_data_sources(self, cds, trace):
         trace_arrays = {}
         Plotly._get_sources_for_trace(trace, trace_arrays)
@@ -171,21 +227,17 @@ class Plotly(PaneBase):
                 if isdatetime(data[idx][key]):
                     arr = data[idx][key]
                     if isinstance(arr, np.ndarray):
-                        arr = arr.astype(str) 
+                        arr = arr.astype(str)
                     else:
                         arr = [str(v) for v in arr]
                     data[idx][key] = arr
         return json
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
-        """
-        Should return the bokeh model to be rendered.
-        """
-        PlotlyPlot = lazy_load('panel.models.plotly', 'PlotlyPlot', isinstance(comm, JupyterComm))
+    def _init_params(self):
         viewport_params = [p for p in self.param if 'viewport' in p]
-        params = list(Layoutable.param)+viewport_params
-        properties = {p : getattr(self, p) for p in params
-                      if getattr(self, p) is not None}
+        parameters = list(Layoutable.param)+viewport_params
+        params = {p: getattr(self, p) for p in parameters
+                  if getattr(self, p) is not None}
 
         if self.object is None:
             json, sources = {}, []
@@ -194,33 +246,22 @@ class Plotly(PaneBase):
             json = self._plotly_json_wrapper(fig)
             sources = Plotly._get_sources(json)
 
-        data = json.get('data', [])
-        layout = json.get('layout', {})
+        params['_render_count'] = self._render_count
+        params['config'] = self.config or {}
+        params['data'] = json.get('data', [])
+        params['data_sources'] = sources
+        params['layout'] = layout = json.get('layout', {})
+        params['frames'] = json.get('frames', [])
         if layout.get('autosize') and self.sizing_mode is self.param.sizing_mode.default:
-            properties['sizing_mode'] = 'stretch_both'
-
-        model = PlotlyPlot(
-            data=data, layout=layout, config=self.config or {},
-            data_sources=sources, _render_count=self._render_count,
-            **properties
-        )
-
-        if root is None:
-            root = model
-
-        self._link_props(
-            model, [
-                'config', 'relayout_data', 'restyle_data', 'click_data',  'hover_data',
-                'clickannotation_data', 'selected_data', 'viewport',
-                'viewport_update_policy', 'viewport_update_throttle', '_render_count'
-            ],
-            doc,
-            root,
-            comm
-        )
+            params['sizing_mode'] = 'stretch_both'
+        return params
 
+    def _get_model(self, doc, root=None, parent=None, comm=None):
+        PlotlyPlot = lazy_load('panel.models.plotly', 'PlotlyPlot', isinstance(comm, JupyterComm), root)
+        model = PlotlyPlot(**self._init_params())
         if root is None:
             root = model
+        self._link_props(model, self._linkable_params, doc, root, comm)
         self._models[root.ref['id']] = (model, parent)
         return model
 
@@ -233,6 +274,7 @@ class Plotly(PaneBase):
         fig = self._to_figure(self.object)
         json = self._plotly_json_wrapper(fig)
         layout = json.get('layout')
+        frames = json.get('frames')
 
         traces = json['data']
         new_sources = []
@@ -245,6 +287,8 @@ class Plotly(PaneBase):
                 new_sources.append(cds)
 
             update_sources = self._update_data_sources(cds, trace) or update_sources
+
+        # Determine if layout needs update
         try:
             update_layout = model.layout != layout
         except Exception:
@@ -259,28 +303,42 @@ class Plotly(PaneBase):
                 try:
                     update_data = (
                         {k: v for k, v in new.items() if k != 'uid'} !=
-                        {k: v for k, v in old.items() if k != 'uid'})
+                        {k: v for k, v in old.items() if k != 'uid'}
+                    )
                 except Exception:
                     update_data = True
                 if update_data:
                     break
 
+        # Determine if frames needs update
+        try:
+            update_frames = model.frames != frames
+        except Exception:
+            update_frames = True
+
+        updates = {}
         if self.sizing_mode is self.param.sizing_mode.default and 'autosize' in layout:
             autosize = layout.get('autosize')
             if autosize and model.sizing_mode != 'stretch_both':
-                model.sizing_mode = 'stretch_both'
+                updates['sizing_mode'] = 'stretch_both'
             elif not autosize and model.sizing_mode != 'fixed':
-                model.sizing_mode = 'fixed'
+                updates['sizing_mode'] = 'fixed'
 
         if new_sources:
-            model.data_sources += new_sources
+            updates['data_sources'] = model.data_sources + new_sources
 
         if update_data:
-            model.data = json.get('data')
+            updates['data'] = json.get('data')
 
         if update_layout:
-            model.layout = layout
+            updates['layout'] = layout
+
+        if update_frames:
+            updates['frames'] = frames or []
+
+        if updates:
+            model.update(**updates)
 
         # Check if we should trigger rendering
-        if new_sources or update_sources or update_data or update_layout:
+        if updates or update_sources:
             model._render_count += 1
