@@ -65,23 +65,25 @@ def poly_data_builder(state, zf, register):
     if 'points' in state['properties']:
         points = state['properties']['points']
         vtkpoints = vtk.vtkPoints()
-        data_arr = ARRAY_TYPES[points['dataType']]()
-        fill_array(data_arr, points, zf)
-        vtkpoints.SetData(data_arr)
+        points_data_arr = ARRAY_TYPES[points['dataType']]()
+        fill_array(points_data_arr, points, zf)
+        vtkpoints.SetData(points_data_arr)
         instance.SetPoints(vtkpoints)
     for cell_type in ['verts', 'lines', 'polys', 'strips']:
         if cell_type in state['properties']:
             cell_arr = vtk.vtkCellArray()
-            fill_array(cell_arr.GetData(), state['properties'][cell_type], zf)
+            cell_data_arr = vtk.vtkIdTypeArray()
+            fill_array(cell_data_arr, state['properties'][cell_type], zf)
+            cell_arr.ImportLegacyFormat(cell_data_arr)
             getattr(instance, 'Set' + capitalize(cell_type))(cell_arr)
-    
+
     # datasets
     fields = state['properties']['fields']
     for dataset in fields:
         data_arr = ARRAY_TYPES[dataset['dataType']]()
         fill_array(data_arr, dataset, zf)
         location = getattr(instance, 'Get' + capitalize(dataset['location']))()
-        getattr(location, capitalize(dataset['registration']))(data_arr)
+        getattr(location, capitalize(dataset.get("registration", "addArray")))(data_arr)
 
 def volume_mapper_builder(state, zf, register):
     instance = generic_builder(state, zf, register)
@@ -120,13 +122,16 @@ def generic_builder(state, zf, register=None):
             else:
                 method = METHODS_RENAME[capitalize(call[0])]
             if method is None: continue
-            getattr(instance, method)(*args)
+            if method == "SetInputData" and len(args)==2:
+                getattr(instance, method + "Object")(*args[::-1])
+            else:
+                getattr(instance, method)(*args)
     arrays = state.get('arrays', None)
     if arrays:
         for array_meta in arrays:
             vtk_array = ARRAY_TYPES[array_meta['dataType']]()
             fill_array(vtk_array, array_meta, zf)
-            location = (instance 
+            location = (instance
                 if 'location' not in array_meta
                 else getattr(instance, 'Get'+capitalize(array_meta['location']))())
             getattr(location, capitalize(array_meta['registration']))(vtk_array)
@@ -156,7 +161,8 @@ def make_type_handlers():
         'vtkActor': ['vtkOpenGLActor', 'vtkPVLODActor'],
         'vtkLight': ['vtkOpenGLLight', 'vtkPVLight'],
         'vtkTexture': ['vtkOpenGLTexture'],
-        'vtkVolumeMapper': ['vtkFixedPointVolumeRayCastMapper', 'vtkSmartVolumeMapper']
+        'vtkVolumeMapper': ['vtkFixedPointVolumeRayCastMapper', 'vtkSmartVolumeMapper'],
+        "vtkGlyph3DMapper": ["vtkOpenGLGlyph3DMapper"],
     }
 
     type_handlers = {
