@@ -1,4 +1,3 @@
-# coding: utf-8
 """
 Defines a VTKPane which renders a vtk plot using VTKPlot bokeh model.
 """
@@ -8,7 +7,6 @@ import base64
 import zipfile
 
 from abc import abstractmethod
-from six import string_types
 from urllib.request import urlopen
 
 import param
@@ -146,9 +144,20 @@ class SyncHelpers:
 
 class VTK:
     """
-    Class factory: allows to switch between VTKJS, VTKRenderWindow, and
-    VTKRenderWindowSynchronized pane in function of the object type and
-    when the serialisation of the vtkRenderWindow occurs.
+    The VTK pane renders a VTK scene inside a panel, making it possible to
+    interact with complex geometries in 3D.
+
+    Reference: https://panel.holoviz.org/reference/panes/VTK.html
+
+    :Example:
+
+    >>> pn.extension('vtk')
+    >>> VTK(some_vtk_object, width=500, height=500)
+
+    This is a Class factory and allows to switch between VTKJS,
+    VTKRenderWindow, and VTKRenderWindowSynchronized pane as a function of the
+    object type and when the serialisation of the vtkRenderWindow occurs.
+
     Once a pane is returned by this class (inst = VTK(object)), one can
     use pn.help(inst) to see parameters available for the current pane
     """
@@ -482,7 +491,15 @@ class VTKRenderWindowSynchronized(BaseVTKRenderWindow, SyncHelpers):
         old_camera = self.vtk_camera
         new_camera = vtk.vtkCamera()
         self.vtk_camera = new_camera
-        exclude_properties = ['mtime']
+        exclude_properties = [
+            'mtime',
+            'projectionMatrix',
+            'viewMatrix',
+            'physicalTranslation',
+            'physicalScale',
+            'physicalViewUp',
+            'physicalViewNorth'
+        ]
         if self.camera is not None:
             for k, v in self.camera.items():
                 if k not in exclude_properties:
@@ -496,6 +513,24 @@ class VTKRenderWindowSynchronized(BaseVTKRenderWindow, SyncHelpers):
 
 
 class VTKVolume(AbstractVTK):
+    """
+    The `VTKVolume` pane renders 3d volumetric data defined on regular grids.
+    It may be constructed from a 3D NumPy array or a vtkVolume.
+
+    The pane provides a number of interactive control which can be set either
+    through callbacks from Python or Javascript callbacks.
+
+    Reference: https://panel.holoviz.org/reference/panes/VTKVolume.html
+
+    :Example:
+
+    >>> pn.extension('vtk')
+    >>> VTKVolume(
+    ...    data_matrix, spacing=(3,2,1), interpolation='nearest',
+    ...    edge_gradient=0, sampling=0,
+    ...    sizing_mode='stretch_width', height=400,
+    ... )
+    """
 
     ambient = param.Number(default=0.2, step=1e-2, doc="""
         Value to control the ambient lighting. It is the light an
@@ -539,6 +574,9 @@ class VTKVolume(AbstractVTK):
     max_data_size = param.Number(default=(256 ** 3) * 2 / 1e6, doc="""
         Maximum data size transfert allowed without subsampling""")
 
+    nan_opacity = param.Number(default=1., bounds=(0., 1.), doc="""
+        Opacity applied to nan values in slices""")
+
     origin = param.Tuple(default=None, length=3, allow_None=True)
 
     render_background = param.Color(default='#52576e', doc="""
@@ -617,7 +655,7 @@ class VTKVolume(AbstractVTK):
         model = VTKVolumePlot(**props)
         if root is None:
             root = model
-        self._link_props(model, ['colormap', 'orientation_widget', 'camera', 'mapper', 'controller_expanded'], doc, root, comm)
+        self._link_props(model, ['colormap', 'orientation_widget', 'camera', 'mapper', 'controller_expanded', 'nan_opacity'], doc, root, comm)
         self._models[root.ref['id']] = (model, parent)
         return model
 
@@ -683,7 +721,7 @@ class VTKVolume(AbstractVTK):
                     dims=sub_array.shape,
                     spacing=self._sub_spacing,
                     origin=self.origin,
-                    data_range=(sub_array.min(), sub_array.max()),
+                    data_range=(np.nanmin(sub_array), np.nanmax(sub_array)),
                     dtype=sub_array.dtype.name)
 
     def _get_volume_data(self):
@@ -736,7 +774,18 @@ class VTKVolume(AbstractVTK):
 
 class VTKJS(AbstractVTK):
     """
-    VTK panes allow rendering vtk scene stored in a vtkjs.
+    The VTKJS pane allow rendering a vtk scene stored in a vtkjs.
+
+    Reference: https://panel.holoviz.org/reference/panes/VTKJS.html
+
+    :Example:
+
+    >>> pn.extension('vtk')
+    >>> VTK(
+    ...    'https://raw.githubusercontent.com/Kitware/vtk-js/master/Data/StanfordDragon.vtkjs',
+    ...     sizing_mode='stretch_width', height=400, enable_keybindings=True,
+    ...     orientation_widget=True
+    ... )
     """
 
     enable_keybindings = param.Boolean(default=False, doc="""
@@ -757,7 +806,7 @@ class VTKJS(AbstractVTK):
 
     @classmethod
     def applies(cls, obj):
-        if isinstance(obj, string_types) and obj.endswith('.vtkjs'):
+        if isinstance(obj, str) and obj.endswith('.vtkjs'):
             return True
 
     def _get_model(self, doc, root=None, parent=None, comm=None):
@@ -778,7 +827,7 @@ class VTKJS(AbstractVTK):
 
     def _get_vtkjs(self):
         if self._vtkjs is None and self.object is not None:
-            if isinstance(self.object, string_types) and self.object.endswith('.vtkjs'):
+            if isinstance(self.object, str) and self.object.endswith('.vtkjs'):
                 if isfile(self.object):
                     with open(self.object, 'rb') as f:
                         vtkjs = f.read()
