@@ -2,16 +2,17 @@
 Defines the Param pane which converts Parameterized classes into a
 set of widgets.
 """
-import os
-import sys
-import json
-import types
 import inspect
 import itertools
+import json
+import os
+import types
 
+from collections.abc import Callable
 from collections import OrderedDict, defaultdict, namedtuple
 from contextlib import contextmanager
-from six import string_types
+from functools import partial
+from packaging.version import Version
 
 import param
 
@@ -21,17 +22,18 @@ from .io import init_doc, state
 from .layout import Column, Panel, Row, Spacer, Tabs
 from .pane.base import PaneBase, ReplacementPane
 from .util import (
-    abbreviated_repr, full_groupby, get_method_owner, is_parameterized,
-    param_name, recursive_parameterized
+    abbreviated_repr, bokeh_version, classproperty, full_groupby,
+    fullpath, get_method_owner, is_parameterized, param_name,
+    recursive_parameterized
 )
 from .reactive import Reactive
 from .viewable import Layoutable, Viewable
 from .widgets import (
-    Button, Checkbox, ColorPicker, DataFrame, DatePicker,
-    DatetimeInput, DateRangeSlider, DiscreteSlider, FileSelector,
-    FloatSlider, IntInput, IntSlider, LiteralInput, MultiSelect,
-    RangeSlider, Select, FloatInput, StaticText, TextInput, Toggle,
-    Widget
+    ArrayInput, Button, Checkbox, ColorPicker, DataFrame, DatePicker,
+    DatetimeInput, DatetimeRangeSlider, DateRangeSlider,
+    DiscreteSlider, FileSelector, FloatSlider, IntInput, IntSlider,
+    LiteralInput, MultiSelect, RangeSlider, Select, FloatInput,
+    StaticText, TextInput, Toggle, Widget
 )
 from .widgets.button import _ButtonBase
 
@@ -75,12 +77,12 @@ def set_values(*parameterizeds, **param_values):
     for parameterized in parameterizeds:
         old_values = {p: getattr(parameterized, p) for p in param_values}
         old.append((parameterized, old_values))
-        parameterized.param.set_param(**param_values)
+        parameterized.param.update(**param_values)
     try:
         yield
     finally:
         for parameterized, old_values in old:
-            parameterized.param.set_param(**old_values)
+            parameterized.param.update(**old_values)
 
 
 class Param(PaneBase):
@@ -136,6 +138,12 @@ class Param(PaneBase):
     show_name = param.Boolean(default=True, doc="""
         Whether to show the parameterized object's name""")
 
+    sort = param.ClassSelector(default=False, class_=(bool, Callable), doc="""
+        If True the widgets will be sorted alphabetically by label.
+        If a callable is provided it will be used to sort the Parameters,
+        for example lambda x: x[1].label[::-1] will sort by the reversed
+        label.""")
+
     width = param.Integer(default=300, allow_None=True, bounds=(0, None), doc="""
         Width of widgetbox the parameter widgets are displayed in.""")
 
@@ -143,12 +151,9 @@ class Param(PaneBase):
         Dictionary of widget overrides, mapping from parameter name
         to widget class.""")
 
-    priority = 0.1
-
-    _unpack = True
-
-    _mapping = {
+    mapping = {
         param.Action:            Button,
+        param.Array:             ArrayInput,
         param.Boolean:           Checkbox,
         param.CalendarDate:      DatePicker,
         param.Color:             ColorPicker,
@@ -173,7 +178,14 @@ class Param(PaneBase):
     }
 
     if hasattr(param, 'Event'):
-        _mapping[param.Event] = Button
+        mapping[param.Event] = Button
+
+    if bokeh_version >= Version('2.4.3'):
+        mapping[param.DateRange] = DatetimeRangeSlider
+
+    priority = 0.1
+
+    _unpack = True
 
     _rerender_params = []
 
@@ -198,7 +210,7 @@ class Param(PaneBase):
         self._updating = []
 
         # Construct Layout
-        kwargs = {p: v for p, v in self.param.get_param_values()
+        kwargs = {p: v for p, v in self.param.values().items()
                   if p in Layoutable.param and v is not None}
         self._widget_box = self.default_layout(**kwargs)
 
@@ -220,16 +232,24 @@ class Param(PaneBase):
             'hide_constant'])
         self._update_widgets()
 
+    @classproperty
+    def _mapping(cls):
+        cls.param.warning(
+            "Param._mapping is now deprecated in favor of the public "
+            "Param.mapping attribute. Update your code accordingly."
+        )
+        return cls.mapping
+
     def __repr__(self, depth=0):
         cls = type(self).__name__
         obj_cls = type(self.object).__name__
         params = [] if self.object is None else list(self.object.param)
         parameters = [k for k in params if k != 'name']
         params = []
-        for p, v in sorted(self.param.get_param_values()):
+        for p, v in sorted(self.param.values().items()):
             if v is self.param[p].default: continue
             elif v is None: continue
-            elif isinstance(v, string_types) and v == '': continue
+            elif isinstance(v, str) and v == '': continue
             elif p == 'object' or (p == 'name' and (v.startswith(obj_cls) or v.startswith(cls))): continue
             elif p == 'parameters' and v == parameters: continue
             try:
@@ -257,7 +277,7 @@ class Param(PaneBase):
                     # Setting object will trigger this method a second time
                     self.object = event.new.cls if event.new.self is None else event.new.self
                     return
-                
+
                 if self._explicit_parameters:
                     parameters = self.parameters
                 elif event.new is None:
@@ -322,7 +342,7 @@ class Param(PaneBase):
                         if e not in existing
                     ]
                 elif change.new:
-                    kwargs = {k: v for k, v in self.param.get_param_values()
+                    kwargs = {k: v for k, v in self.param.values().items()
                               if k not in ['name', 'object', 'parameters']}
                     pane = Param(parameterized, name=parameterized.name,
                                  **kwargs)
@@ -343,13 +363,13 @@ class Param(PaneBase):
                     return
                 elif is_parameterized(change.new):
                     parameterized = change.new
-                    kwargs = {k: v for k, v in self.param.get_param_values()
+                    kwargs = {k: v for k, v in self.param.values().items()
                               if k not in ['name', 'object', 'parameters']}
                     pane = Param(parameterized, name=parameterized.name,
                                  **kwargs)
                     layout[layout.objects.index(existing[0])] = pane
                 else:
-                    layout.pop(existing[0])
+                    layout.remove(existing[0])
 
             watchers = [selector.param.watch(update_pane, 'value')]
             if toggle:
@@ -400,13 +420,13 @@ class Param(PaneBase):
 
         if hasattr(p_obj, 'get_range'):
             options = p_obj.get_range()
-            if not options and value is not None:
-                options = [value]
             # This applies to widgets whose `options` Parameter is a List type,
             # such as AutoCompleteInput.
             if ('options' in widget_class.param
                 and isinstance(widget_class.param['options'], param.List)):
                 options = list(options.values())
+            if not options and value is not None:
+                options = [value]
             kw['options'] = options
         if hasattr(p_obj, 'get_soft_bounds'):
             bounds = p_obj.get_soft_bounds()
@@ -414,11 +434,11 @@ class Param(PaneBase):
                 kw['start'] = bounds[0]
             if bounds[1] is not None:
                 kw['end'] = bounds[1]
-            if ('start' not in kw or 'end' not in kw):
-                # Do not change widget class if _mapping was overridden
+            if (('start' not in kw or 'end' not in kw) and
+                not isinstance(p_obj, (param.Date, param.CalendarDate))):
+                # Do not change widget class if mapping was overridden
                 if not widget_class_overridden:
-                    if (isinstance(p_obj, param.Number) and
-                        not isinstance(p_obj, (param.Date, param.CalendarDate))):
+                    if isinstance(p_obj, param.Number):
                         widget_class = FloatInput
                         if isinstance(p_obj, param.Integer):
                             widget_class = IntInput
@@ -446,7 +466,7 @@ class Param(PaneBase):
                 return
             try:
                 self._updating.append(p_name)
-                self.object.param.set_param(**{p_name: change.new})
+                self.object.param.update(**{p_name: change.new})
             finally:
                 self._updating.remove(p_name)
 
@@ -480,7 +500,7 @@ class Param(PaneBase):
                     self._rerender()
                 elif (change.new < self.display_threshold and
                       widget in self._widget_box.objects):
-                    self._widget_box.pop(widget)
+                    self._widget_box.remove(widget)
                 elif change.new >= self.display_threshold:
                     self._rerender()
                 return
@@ -533,10 +553,10 @@ class Param(PaneBase):
                 self._updating.append(p_name)
                 if change.type == 'triggered':
                     with discard_events(widget):
-                        widget.param.set_param(**updates)
+                        widget.param.update(**updates)
                     widget.param.trigger(*updates)
                 else:
-                    widget.param.set_param(**updates)
+                    widget.param.update(**updates)
             finally:
                 self._updating.remove(p_name)
 
@@ -571,14 +591,21 @@ class Param(PaneBase):
     def _ordered_params(self):
         params = [(p, pobj) for p, pobj in self.object.param.objects('existing').items()
                   if p in self.parameters or p == 'name']
+        if self.sort:
+            if callable(self.sort):
+                key_fn = self.sort
+            else:
+                key_fn = lambda x: x[1].label
+            sorted_params = sorted(params, key=key_fn)
+            sorted_params = [el[0] for el in sorted_params if (el[0] != 'name' or el[0] in self.parameters)]
+            return sorted_params
+
         key_fn = lambda x: x[1].precedence if x[1].precedence is not None else self.default_precedence
         sorted_precedence = sorted(params, key=key_fn)
         filtered = [(k, p) for k, p in sorted_precedence]
         groups = itertools.groupby(filtered, key=key_fn)
         # Params preserve definition order in Python 3.6+
-        dict_ordered_py3 = (sys.version_info.major == 3 and sys.version_info.minor >= 6)
-        dict_ordered = dict_ordered_py3 or (sys.version_info.major > 3)
-        ordered_groups = [list(grp) if dict_ordered else sorted(grp) for (_, grp) in groups]
+        ordered_groups = [list(grp) for (_, grp) in groups]
         ordered_params = [el[0] for group in ordered_groups for el in group
                           if (el[0] != 'name' or el[0] in self.parameters)]
         return ordered_params
@@ -607,7 +634,7 @@ class Param(PaneBase):
                 watchers.append(w)
         self._widgets[p_name] = self.widget(p_name)
         self._rerender()
-    
+
     def _get_widgets(self):
         """Return name,widget boxes for all parameters (i.e., a property sheet)"""
         # Format name specially
@@ -643,10 +670,12 @@ class Param(PaneBase):
     def widget_type(cls, pobj):
         ptype = type(pobj)
         for t in classlist(ptype)[::-1]:
-            if t in cls._mapping:
-                if isinstance(cls._mapping[t], types.FunctionType):
-                    return cls._mapping[t](pobj)
-                return cls._mapping[t]
+            if t not in cls.mapping:
+                continue
+            wtype = cls.mapping[t]
+            if isinstance(wtype, types.FunctionType):
+                return wtype(pobj)
+            return wtype
 
     def select(self, selector=None):
         """
@@ -721,17 +750,16 @@ class ParamMethod(ReplacementPane):
         if not dependencies or not dependencies.get('watch'):
             return
         fn_type = 'method' if type(self) is ParamMethod else 'function'
-        self.param.warning(f"The {fn_type} supplied for Panel to display "
-                           "was declared with `watch=True`, which will "
-                           f"cause the {fn_type} to be called twice for "
-                           "any change in a dependent Parameter. "
-                           "`watch` should be False when Panel is "
-                           "responsible for displaying the result "
-                           f"of the {fn_type} call, while `watch=True` "
-                           f"should be reserved for {fn_type}s that work "
-                           "via side-effects, e.g. by modifying internal  "
-                           "state of a class or global state in an "
-                           "application's namespace.")
+        self.param.warning(
+            f"The {fn_type} supplied for Panel to display was declared "
+            f"with `watch=True`, which will cause the {fn_type} to be "
+            "called twice for any change in a dependent Parameter. "
+            "`watch` should be False when Panel is responsible for "
+            f"displaying the result of the {fn_type} call, while "
+            f"`watch=True` should be reserved for {fn_type}s that work "
+            "via side-effects, e.g. by modifying internal state of a "
+            "class or global state in an application's namespace."
+        )
 
     #----------------------------------------------------------------
     # Callback API
@@ -748,6 +776,13 @@ class ParamMethod(ReplacementPane):
                 kwargs = {n: getattr(dep.owner, dep.name) for n, dep in kw_deps.items()}
         return function(*args, **kwargs)
 
+    async def _eval_async(self, awaitable):
+        try:
+            new_object = await awaitable
+            self._update_inner(new_object)
+        finally:
+            self._inner_layout.loading = False
+
     def _replace_pane(self, *args, force=False):
         self._evaled = bool(self._models) or force or not self.lazy
         if self._evaled:
@@ -757,6 +792,9 @@ class ParamMethod(ReplacementPane):
                     new_object = Spacer()
                 else:
                     new_object = self.eval(self.object)
+                if inspect.isawaitable(new_object):
+                    param.parameterized.async_executor(partial(self._eval_async, new_object))
+                    return
                 self._update_inner(new_object)
             finally:
                 self._inner_layout.loading = False
@@ -775,13 +813,13 @@ class ParamMethod(ReplacementPane):
 
     def _link_object_params(self):
         parameterized = get_method_owner(self.object)
-        params = parameterized.param.params_depended_on(self.object.__name__)
+        params = parameterized.param.method_dependencies(self.object.__name__)
         deps = params
 
         def update_pane(*events):
             # Update nested dependencies if parameterized object events
             if any(is_parameterized(event.new) for event in events):
-                new_deps = parameterized.param.params_depended_on(self.object.__name__)
+                new_deps = parameterized.param.method_dependencies(self.object.__name__)
                 for p in list(deps):
                     if p in new_deps: continue
                     watchers = self._callbacks
@@ -790,8 +828,8 @@ class ParamMethod(ReplacementPane):
                             p.name in w.parameter_names):
                             obj = p.cls if p.inst is None else p.inst
                             obj.param.unwatch(w)
-                            watchers.pop(watchers.index(w))
-                    deps.pop(deps.index(p))
+                            watchers.remove(w)
+                    deps.remove(p)
 
                 new_deps = [dep for dep in new_deps if dep not in deps]
                 for _, params in full_groupby(new_deps, lambda x: (x.inst or x.cls, x.what)):
@@ -844,6 +882,19 @@ class ParamFunction(ParamMethod):
     def _link_object_params(self):
         deps = getattr(self.object, '_dinfo', {})
         dep_params = list(deps.get('dependencies', [])) + list(deps.get('kw', {}).values())
+        if not dep_params:
+            fn = getattr(self.object, '__bound_function__', self.object)
+            fn_name = getattr(fn, '__name__', repr(self.object))
+            self.param.warning(
+                f"The function {fn_name!r} does not have any dependencies "
+                "and will never update. Are you sure you did not intend "
+                "to depend on or bind a parameter or widget to this function? "
+                "If not simply call the function before passing it to Panel. "
+                "Otherwise, when passing a parameter as an argument, "
+                "ensure you pass at least one parameter and reference the "
+                "actual parameter object not the current value, i.e. use "
+                "object.param.parameter not object.parameter."
+            )
         grouped = defaultdict(list)
         for dep in dep_params:
             grouped[id(dep.owner)].append(dep)
@@ -909,7 +960,8 @@ class JSONInit(param.Parameterized):
         if self.json_file or env_var.endswith('.json'):
             try:
                 fname = self.json_file if self.json_file else env_var
-                spec = json.load(open(os.path.abspath(fname), 'r'))
+                with open(fullpath(fname), 'r') as f:
+                    spec = json.load(f)
             except Exception:
                 warnobj.warning('Could not load JSON file %r' % spec)
         else:
@@ -926,7 +978,7 @@ class JSONInit(param.Parameterized):
 
         for name, value in params.items():
             try:
-                parameterized.param.set_param(**{name:value})
+                parameterized.param.update(**{name:value})
             except ValueError as e:
                 warnobj.warning(str(e))
 
@@ -954,3 +1006,10 @@ def link_param_method(root_view, root_model):
 
 
 Viewable._preprocessing_hooks.insert(0, link_param_method)
+
+__all__= (
+    "Param",
+    "ParamFunction",
+    "ParamMethod",
+    "set_values"
+)
