@@ -3,9 +3,10 @@ Declares Syncable and Reactive classes which provides baseclasses
 for Panel components which sync their state with one or more bokeh
 models rendered on the frontend.
 """
+from __future__ import annotations
 
-import difflib
 import datetime as dt
+import difflib
 import logging
 import re
 import sys
@@ -13,28 +14,46 @@ import textwrap
 
 from collections import Counter, defaultdict, namedtuple
 from functools import partial
+from pprint import pformat
+from typing import (
+    TYPE_CHECKING, Any, Callable, Dict, List, Mapping, Optional,
+    Set, Tuple, Type, Union
+)
 
 import bleach
 import numpy as np
 import param
 
+from bokeh.core.property.descriptors import UnsetValueError
 from bokeh.model import DataModel
 from param.parameterized import ParameterizedMetaclass, Watcher
-from pprint import pformat
 
 from .io.document import unlocked
 from .io.model import hold
 from .io.notebook import push
 from .io.state import set_curdoc, state
 from .models.reactive_html import (
-    ReactiveHTML as _BkReactiveHTML, ReactiveHTMLParser
+    DOMEvent, ReactiveHTML as _BkReactiveHTML, ReactiveHTMLParser
 )
 from .util import edit_readonly, escape, updating
 from .viewable import Layoutable, Renderable, Viewable
 
+if TYPE_CHECKING:
+    import pandas as pd
+
+    from bokeh.document import Document
+    from bokeh.events import Event
+    from bokeh.model import Model
+    from bokeh.models.sources import DataDict, Patches
+    from pyviz_comms import Comm
+
+    from .layout.base import Panel
+    from .links import Callback, JSLinkTarget, Link
+
 log = logging.getLogger('panel.reactive')
 
-LinkWatcher = namedtuple("Watcher", Watcher._fields+('target', 'links', 'transformed', 'bidirectional_watcher'))
+_fields = tuple(Watcher._fields+('target', 'links', 'transformed', 'bidirectional_watcher'))
+LinkWatcher: Tuple = namedtuple("Watcher", _fields) # type: ignore
 
 
 class Syncable(Renderable):
@@ -53,28 +72,28 @@ class Syncable(Renderable):
     """
 
     # Timeout if a notebook comm message is swallowed
-    _timeout = 20000
+    _timeout: int = 20000
 
     # Timeout before the first event is processed
-    _debounce = 50
+    _debounce: int = 50
 
     # Property changes which should not be debounced
-    _priority_changes = []
+    _priority_changes: List[str] = []
 
     # Any parameters that require manual updates handling for the models
     # e.g. parameters which affect some sub-model
-    _manual_params = []
+    _manual_params: List[str] = []
 
     # Mapping from parameter name to bokeh model property name
-    _rename = {}
+    _rename: Mapping[str, str | None] = {}
 
     # Allows defining a mapping from model property name to a JS code
     # snippet that transforms the object before serialization
-    _js_transforms = {}
+    _js_transforms: Mapping[str, str] = {}
 
     # Transforms from input value to bokeh property value
-    _source_transforms = {}
-    _target_transforms = {}
+    _source_transforms: Mapping[str, str | None] = {}
+    _target_transforms: Mapping[str, str | None] = {}
 
     __abstract = True
 
@@ -103,7 +122,7 @@ class Syncable(Renderable):
     # Model API
     #----------------------------------------------------------------
 
-    def _process_property_change(self, msg):
+    def _process_property_change(self, msg: Dict[str, Any]) -> Dict[str, Any]:
         """
         Transform bokeh model property changes into parameter updates.
         Should be overridden to provide appropriate mapping between
@@ -114,7 +133,7 @@ class Syncable(Renderable):
         inverted = {v: k for k, v in self._rename.items()}
         return {inverted.get(k, k): v for k, v in msg.items()}
 
-    def _process_param_change(self, msg):
+    def _process_param_change(self, msg: Dict[str, Any]) -> Dict[str, Any]:
         """
         Transform parameter changes into bokeh model property updates.
         Should be overridden to provide appropriate mapping between
@@ -122,7 +141,7 @@ class Syncable(Renderable):
         _rename class level attribute to map between parameter and
         property names.
         """
-        properties = {self._rename.get(k, k): v for k, v in msg.items()
+        properties = {self._rename.get(k) or k: v for k, v in msg.items()
                       if self._rename.get(k, False) is not None}
         if 'width' in properties and self.sizing_mode is None:
             properties['min_width'] = properties['width']
@@ -131,7 +150,7 @@ class Syncable(Renderable):
         return properties
 
     @property
-    def _linkable_params(self):
+    def _linkable_params(self) -> List[str]:
         """
         Parameters that can be linked in JavaScript via source
         transforms.
@@ -140,7 +159,7 @@ class Syncable(Renderable):
                 and self._source_transforms.get(p, False) is not None] + ['loading']
 
     @property
-    def _synced_params(self):
+    def _synced_params(self) -> List[str]:
         """
         Parameters which are synced with properties using transforms
         applied in the _process_param_change method.
@@ -148,17 +167,20 @@ class Syncable(Renderable):
         ignored = ['default_layout', 'loading']
         return [p for p in self.param if p not in self._manual_params+ignored]
 
-    def _init_params(self):
+    def _init_params(self) -> Dict[str, Any]:
         return {k: v for k, v in self.param.values().items()
                 if k in self._synced_params and v is not None}
 
-    def _link_params(self):
+    def _link_params(self) -> None:
         params = self._synced_params
         if params:
             watcher = self.param.watch(self._param_change, params)
             self._callbacks.append(watcher)
 
-    def _link_props(self, model, properties, doc, root, comm=None):
+    def _link_props(
+        self, model: 'Model', properties: List[str] | List[Tuple[str, str]],
+        doc: 'Document', root: 'Model', comm: Optional['Comm'] = None
+    ) -> None:
         from .config import config
         ref = root.ref['id']
         if config.embed:
@@ -179,13 +201,16 @@ class Syncable(Renderable):
             else:
                 m.on_change(p, partial(self._server_change, doc, ref, subpath))
 
-    def _manual_update(self, events, model, doc, root, parent, comm):
+    def _manual_update(
+        self, events: Tuple[param.parameterized.Event, ...], model: 'Model', doc: 'Document',
+        root: 'Model', parent: Optional['Model'], comm: Optional['Comm']
+    ) -> None:
         """
         Method for handling any manual update events, i.e. events triggered
         by changes in the manual params.
         """
 
-    def _update_manual(self, *events):
+    def _update_manual(self, *events: param.parameterized.Event) -> None:
         for ref, (model, parent) in self._models.items():
             if ref not in state._views or ref in state._fake_roots:
                 continue
@@ -202,7 +227,10 @@ class Syncable(Renderable):
                 else:
                     cb()
 
-    def _apply_update(self, events, msg, model, ref):
+    def _apply_update(
+        self, events: Dict[str, param.parameterized.Event], msg: Mapping[str, Any],
+        model: 'Model', ref: str
+    ) -> None:
         if ref not in state._views or ref in state._fake_roots:
             return
         viewable, root, doc, comm = state._views[ref]
@@ -215,12 +243,21 @@ class Syncable(Renderable):
             cb = partial(self._update_model, events, msg, root, model, doc, comm)
             doc.add_next_tick_callback(cb)
 
-    def _update_model(self, events, msg, root, model, doc, comm):
+    def _update_model(
+        self, events: Dict[str, param.parameterized.Event], msg: Dict[str, Any],
+        root: 'Model', model: 'Model', doc: 'Document', comm: Optional['Comm']
+    ) -> None:
         ref = root.ref['id']
-        self._changing[ref] = attrs = [
-            attr for attr, value in msg.items()
-            if not model.lookup(attr).property.matches(getattr(model, attr), value)
-        ]
+        self._changing[ref] = attrs = []
+        for attr, value in msg.items():
+            # Bokeh raises UnsetValueError if the value is Undefined.
+            try:
+                model_val = getattr(model, attr)
+            except UnsetValueError:
+                attrs.append(attr)
+                continue
+            if not model.lookup(attr).property.matches(model_val, value):
+                attrs.append(attr)
         try:
             model.update(**msg)
         finally:
@@ -233,8 +270,10 @@ class Syncable(Renderable):
             elif ref in self._changing:
                 del self._changing[ref]
 
-    def _cleanup(self, root):
+    def _cleanup(self, root: 'Model' | None) -> None:
         super()._cleanup(root)
+        if root is None:
+            return
         ref = root.ref['id']
         self._models.pop(ref, None)
         comm, client_comm = self._comms.pop(ref, (None, None))
@@ -249,22 +288,22 @@ class Syncable(Renderable):
             except Exception:
                 pass
 
-    def _param_change(self, *events):
+    def _param_change(self, *events: param.parameterized.Event) -> None:
         msgs = []
         for event in events:
             msg = self._process_param_change({event.name: event.new})
             if msg:
                 msgs.append(msg)
 
-        events = {event.name: event for event in events}
+        named_events = {event.name: event for event in events}
         msg = {k: v for msg in msgs for k, v in msg.items()}
         if not msg:
             return
 
         for ref, (model, _) in self._models.copy().items():
-            self._apply_update(events, msg, model, ref)
+            self._apply_update(named_events, msg, model, ref)
 
-    def _process_events(self, events):
+    def _process_events(self, events: Dict[str, Any]) -> None:
         self._log('received events %s', events)
         busy = state.busy
         with edit_readonly(state):
@@ -297,7 +336,7 @@ class Syncable(Renderable):
             with edit_readonly(state):
                 state.busy = busy
 
-    def _process_bokeh_event(self, event):
+    def _process_bokeh_event(self, event: 'Event') -> None:
         self._log('received bokeh event %s', event)
         busy = state.busy
         with edit_readonly(state):
@@ -309,21 +348,21 @@ class Syncable(Renderable):
             with edit_readonly(state):
                 state.busy = busy
 
-    async def _change_coroutine(self, doc=None):
+    async def _change_coroutine(self, doc: Optional['Document'] = None) -> None:
         if state._thread_pool:
             state._thread_pool.submit(self._change_event, doc)
         else:
             with set_curdoc(doc):
                 self._change_event(doc)
 
-    async def _event_coroutine(self, event, doc):
+    async def _event_coroutine(self, event, doc: 'Document') -> None:
         if state._thread_pool:
             state._thread_pool.submit(self._process_bokeh_event, event)
         else:
             with set_curdoc(doc):
                 self._process_bokeh_event(event)
 
-    def _change_event(self, doc=None):
+    def _change_event(self, doc: Optional['Document'] = None) -> None:
         try:
             state.curdoc = doc
             events = self._events
@@ -332,11 +371,14 @@ class Syncable(Renderable):
         finally:
             state.curdoc = None
 
-    def _schedule_change(self, doc, comm):
+    def _schedule_change(self, doc: Optional['Document'], comm: Optional['Comm']) -> None:
         with hold(doc, comm=comm):
             self._change_event(doc)
 
-    def _comm_change(self, doc, ref, comm, subpath, attr, old, new):
+    def _comm_change(
+        self, doc: 'Document', ref: str, comm: Optional['Comm'], subpath: str,
+        attr: str, old: Any, new: Any
+    ) -> None:
         if subpath:
             attr = f'{subpath}.{attr}'
         if attr in self._changing.get(ref, []):
@@ -349,13 +391,13 @@ class Syncable(Renderable):
         else:
             self._schedule_change(doc, comm)
 
-    def _comm_event(self, event):
+    def _comm_event(self, event: 'Event') -> None:
         if state._thread_pool:
             state._thread_pool.submit(self._process_bokeh_event, event)
         else:
             self._process_bokeh_event(event)
 
-    def _server_event(self, doc, event):
+    def _server_event(self, doc: 'Document', event: 'Event') -> None:
         if doc.session_context and not state._unblocked(doc):
             doc.add_next_tick_callback(
                 partial(self._event_coroutine, event, doc)
@@ -363,7 +405,10 @@ class Syncable(Renderable):
         else:
             self._comm_event(event)
 
-    def _server_change(self, doc, ref, subpath, attr, old, new):
+    def _server_change(
+        self, doc: 'Document', ref: str, subpath: str, attr: str,
+        old: Any, new: Any
+    ) -> None:
         if subpath:
             attr = f'{subpath}.{attr}'
         if attr in self._changing.get(ref, []):
@@ -399,22 +444,27 @@ class Reactive(Syncable, Viewable):
     # Public API
     #----------------------------------------------------------------
 
-    def link(self, target, callbacks=None, bidirectional=False,  **links):
+    def link(
+        self, target: param.Parameterized, callbacks: Optional[Dict[str, str | Callable]]=None,
+        bidirectional: bool=False, **links: str
+    ) -> Watcher:
         """
-        Links the parameters on this object to attributes on another
-        object in Python. Supports two modes, either specify a mapping
-        between the source and target object parameters as keywords or
+        Links the parameters on this `Reactive` object to attributes on the
+        target `Parameterized` object.
+
+        Supports two modes, either specify a
+        mapping between the source and target object parameters as keywords or
         provide a dictionary of callbacks which maps from the source
         parameter to a callback which is triggered when the parameter
         changes.
 
         Arguments
         ---------
-        target: object
+        target: param.Parameterized
           The target object of the link.
-        callbacks: dict
+        callbacks: dict | None
           Maps from a parameter in the source object to a callback.
-        bidirectional: boolean
+        bidirectional: bool
           Whether to link source and target bi-directionally
         **links: dict
           Maps between parameters on this object to the parameters
@@ -433,7 +483,7 @@ class Reactive(Syncable, Viewable):
                              'separate callbacks for each direction.')
 
         _updating = []
-        def link(*events):
+        def link_cb(*events):
             for event in events:
                 if event.name in _updating: continue
                 _updating.append(event.name)
@@ -445,7 +495,7 @@ class Reactive(Syncable, Viewable):
                 finally:
                     _updating.pop(_updating.index(event.name))
         params = list(callbacks) if callbacks else list(links)
-        cb = self.param.watch(link, params)
+        cb = self.param.watch(link_cb, params)
 
         bidirectional_watcher = None
         if bidirectional:
@@ -470,7 +520,7 @@ class Reactive(Syncable, Viewable):
         self._links.append(link)
         return cb
 
-    def controls(self, parameters=[], jslink=True, **kwargs):
+    def controls(self, parameters: List[str] = [], jslink: bool = True, **kwargs) -> 'Panel':
         """
         Creates a set of widgets which allow manipulating the parameters
         on this instance. By default all parameters which support
@@ -492,8 +542,8 @@ class Reactive(Syncable, Viewable):
         -------
         A layout of the controls
         """
-        from .param import Param
         from .layout import Tabs, WidgetBox
+        from .param import Param
         from .widgets import LiteralInput
 
         if parameters:
@@ -529,7 +579,7 @@ class Reactive(Syncable, Viewable):
             return controls.layout[0]
         return style.layout[0]
 
-    def jscallback(self, args={}, **callbacks):
+    def jscallback(self, args: Mapping[str, Any]={}, **callbacks: str) -> 'Callback':
         """
         Allows defining a JS callback to be triggered when a property
         changes on the source object. The keyword arguments define the
@@ -551,14 +601,20 @@ class Reactive(Syncable, Viewable):
         """
 
         from .links import Callback
-        for k, v in list(callbacks.items()):
-            callbacks[k] = self._rename.get(v, v)
-        return Callback(self, code=callbacks, args=args)
+        renamed = {
+            k: self._rename.get(v, v) for k, v in callbacks.items()
+        }
+        return Callback(self, code=renamed, args=args)
 
-    def jslink(self, target, code=None, args=None, bidirectional=False, **links):
+    def jslink(
+        self, target: 'JSLinkTarget' , code: Dict[str, str] = None, args: Optional[Dict] = None,
+        bidirectional: bool = False, **links: str
+    ) -> 'Link':
         """
-        Links properties on the source object to those on the target
-        object in JS code. Supports two modes, either specify a
+        Links properties on the this Reactive object to those on the
+        target Reactive object in JS code.
+
+        Supports two modes, either specify a
         mapping between the source and target model properties as
         keywords or provide a dictionary of JS code snippets which
         maps from the source parameter to a JS code snippet which is
@@ -566,11 +622,13 @@ class Reactive(Syncable, Viewable):
 
         Arguments
         ----------
-        target: HoloViews object or bokeh Model or panel Viewable
+        target: panel.viewable.Viewable | bokeh.model.Model | holoviews.core.dimension.Dimensioned
           The target to link the value to.
         code: dict
           Custom code which will be executed when the widget value
           changes.
+        args: dict
+          A mapping of objects to make available to the JS callback
         bidirectional: boolean
           Whether to link source and target bi-directionally
         **links: dict
@@ -602,19 +660,22 @@ class Reactive(Syncable, Viewable):
                     bidirectional=bidirectional)
 
 
+TData = Union['pd.DataFrame', 'DataDict']
+
+
 class SyncableData(Reactive):
     """
     A baseclass for components which sync one or more data parameters
     with the frontend via a ColumnDataSource.
     """
 
-    selection = param.List(default=[], doc="""
+    selection = param.List(default=[], class_=int, doc="""
         The currently selected rows in the data.""")
 
     # Parameters which when changed require an update of the data
-    _data_params = []
+    _data_params: List[str] = []
 
-    _rename = {'selection': None}
+    _rename: Mapping[str, str | None] = {'selection': None}
 
     __abstract = True
 
@@ -629,12 +690,12 @@ class SyncableData(Reactive):
         self._validate()
         self._update_cds()
 
-    def _validate(self, *events):
+    def _validate(self, *events: param.parameterized.Event) -> None:
         """
         Allows implementing validation for the data parameters.
         """
 
-    def _get_data(self):
+    def _get_data(self) -> Tuple[TData, 'DataDict']:
         """
         Implemented by subclasses converting data parameter(s) into
         a ColumnDataSource compatible data dictionary.
@@ -648,7 +709,7 @@ class SyncableData(Reactive):
           ColumnDataSource
         """
 
-    def _update_column(self, column, array):
+    def _update_column(self, column: str, array: np.ndarray | List) -> None:
         """
         Implemented by subclasses converting changes in columns to
         changes in the data parameter.
@@ -665,10 +726,13 @@ class SyncableData(Reactive):
         if self._processed is not None:
             self._processed[column] = array
 
-    def _update_data(self, data):
+    def _update_data(self, data: TData) -> None:
         self.param.update(**{self._data_params[0]: data})
 
-    def _manual_update(self, events, model, doc, root, parent, comm):
+    def _manual_update(
+        self, events: Tuple[param.parameterized.Event, ...], model: 'Model',
+        doc: 'Document', root: 'Model', parent: Optional['Model'], comm: 'Comm'
+    ) -> None:
         for event in events:
             if event.type == 'triggered' and self._updating:
                 continue
@@ -676,20 +740,22 @@ class SyncableData(Reactive):
                 getattr(self, '_update_' + event.name)(model)
 
     @updating
-    def _update_cds(self, *events):
+    def _update_cds(self, *events: param.parameterized.Event) -> None:
         self._processed, self._data = self._get_data()
         msg = {'data': self._data}
         for ref, (m, _) in self._models.items():
             self._apply_update(events, msg, m.source, ref)
 
     @updating
-    def _update_selected(self, *events, indices=None):
+    def _update_selected(
+        self, *events: param.parameterized.Event, indices: Optional[List[int]] = None
+    ) -> None:
         indices = self.selection if indices is None else indices
         msg = {'indices': indices}
         for ref, (m, _) in self._models.items():
             self._apply_update(events, msg, m.source.selected, ref)
 
-    def _apply_stream(self, ref, model, stream, rollover):
+    def _apply_stream(self, ref: str, model: 'Model', stream: 'DataDict', rollover: Optional[int]) -> None:
         self._changing[ref] = ['data']
         try:
             model.source.stream(stream, rollover)
@@ -697,7 +763,7 @@ class SyncableData(Reactive):
             del self._changing[ref]
 
     @updating
-    def _stream(self, stream, rollover=None):
+    def _stream(self, stream: 'DataDict', rollover: Optional[int] = None) -> None:
         self._processed, _ = self._get_data()
         for ref, (m, _) in self._models.items():
             if ref not in state._views or ref in state._fake_roots:
@@ -712,7 +778,7 @@ class SyncableData(Reactive):
                 cb = partial(self._apply_stream, ref, m, stream, rollover)
                 doc.add_next_tick_callback(cb)
 
-    def _apply_patch(self, ref, model, patch):
+    def _apply_patch(self, ref: str, model: 'Model', patch: 'Patches') -> None:
         self._changing[ref] = ['data']
         try:
             model.source.patch(patch)
@@ -720,7 +786,7 @@ class SyncableData(Reactive):
             del self._changing[ref]
 
     @updating
-    def _patch(self, patch):
+    def _patch(self, patch: 'Patches') -> None:
         for ref, (m, _) in self._models.items():
             if ref not in state._views or ref in state._fake_roots:
                 continue
@@ -734,7 +800,7 @@ class SyncableData(Reactive):
                 cb = partial(self._apply_patch, ref, m, patch)
                 doc.add_next_tick_callback(cb)
 
-    def _update_manual(self, *events):
+    def _update_manual(self, *events: param.parameterized.Event) -> None:
         """
         Skip events triggered internally
         """
@@ -745,16 +811,19 @@ class SyncableData(Reactive):
             processed_events.append(e)
         super()._update_manual(*processed_events)
 
-    def stream(self, stream_value, rollover=None, reset_index=True):
+    def stream(
+        self, stream_value: 'pd.DataFrame' | 'pd.Series' | Dict,
+        rollover: Optional[int] = None, reset_index: bool = True
+    ) -> None:
         """
         Streams (appends) the `stream_value` provided to the existing
         value in an efficient manner.
 
         Arguments
         ---------
-        stream_value: (Union[pd.DataFrame, pd.Series, Dict])
+        stream_value: (pd.DataFrame | pd.Series | Dict)
           The new value(s) to append to the existing value.
-        rollover: int
+        rollover: (int | None, default=None)
            A maximum column size, above which data from the start of
            the column begins to be discarded. If None, then columns
            will continue to grow unbounded.
@@ -804,7 +873,7 @@ class SyncableData(Reactive):
         if 'pandas' in sys.modules:
             import pandas as pd
         else:
-            pd = None
+            pd = None # type: ignore
         if pd and isinstance(stream_value, pd.DataFrame):
             if isinstance(self._processed, dict):
                 self.stream(stream_value.to_dict(), rollover)
@@ -852,13 +921,13 @@ class SyncableData(Reactive):
         else:
             raise ValueError("The stream value provided is not a DataFrame, Series or Dict!")
 
-    def patch(self, patch_value):
+    def patch(self, patch_value: 'pd.DataFrame' | 'pd.Series' | Dict) -> None:
         """
         Efficiently patches (updates) the existing value with the `patch_value`.
 
         Arguments
         ---------
-        patch_value: (Union[pd.DataFrame, pd.Series, Dict])
+        patch_value: (pd.DataFrame | pd.Series | Dict)
           The value(s) to patch the existing value with.
 
         Raises
@@ -907,10 +976,10 @@ class SyncableData(Reactive):
         if 'pandas' in sys.modules:
             import pandas as pd
         else:
-            pd = None
+            pd = None # type: ignore
         data = getattr(self, self._data_params[0])
+        patch_value_dict: 'Patches' = {}
         if pd and isinstance(patch_value, pd.DataFrame):
-            patch_value_dict = {}
             for column in patch_value.columns:
                 patch_value_dict[column] = []
                 for index in patch_value.index:
@@ -955,15 +1024,18 @@ class ReactiveData(SyncableData):
     def __init__(self, **params):
         super().__init__(**params)
         self._old = None
-    
-    def _update_selection(self, indices):
+
+    def _update_selection(self, indices: List[int]) -> None:
         self.selection = indices
 
-    def _convert_column(self, values, old_values):
+    def _convert_column(
+        self, values: np.ndarray, old_values: np.ndarray | 'pd.Series'
+    ) -> np.ndarray | List:
         dtype = old_values.dtype
+        converted: List | np.ndarray | None = None
         if dtype.kind == 'M':
             if values.dtype.kind in 'if':
-                values = (values * 10e5).astype(dtype)
+                converted = (values * 10e5).astype(dtype)
         elif dtype.kind == 'O':
             if (all(isinstance(ov, dt.date) for ov in old_values) and
                 not all(isinstance(iv, dt.date) for iv in values)):
@@ -974,12 +1046,12 @@ class ReactiveData(SyncableData):
                     elif not isinstance(iv, dt.date):
                         iv = dt.date.fromtimestamp(iv/1000)
                     new_values.append(iv)
-                values = new_values
+                converted = new_values
         else:
-            values = values.astype(dtype)
-        return values
+            converted = values.astype(dtype)
+        return values if converted is None else converted
 
-    def _process_data(self, data):
+    def _process_data(self, data: Mapping[str, List | Dict[int, Any] | np.ndarray]) -> None:
         if self._updating:
             return
 
@@ -987,7 +1059,7 @@ class ReactiveData(SyncableData):
         old_raw, old_data = self._get_data()
         self._old = old_raw = old_raw.copy()
         if hasattr(old_raw, 'columns'):
-            columns = list(old_raw.columns)
+            columns = list(old_raw.columns) # type: ignore
         else:
             columns = list(old_raw)
 
@@ -1003,7 +1075,7 @@ class ReactiveData(SyncableData):
                 np.asarray(values), old_raw[col]
             )
             try:
-                isequal = (old_raw[col] == values).all()
+                isequal = (old_raw[col] == values).all() # type: ignore
             except Exception:
                 isequal = False
             if not isequal:
@@ -1018,7 +1090,7 @@ class ReactiveData(SyncableData):
         self._updating = True
         old_data = getattr(self, self._data_params[0])
         try:
-            if old_data is self.value:
+            if old_data is self.value: # type: ignore
                 with param.discard_events(self):
                     self.value = old_raw
                 self.value = old_data
@@ -1031,7 +1103,7 @@ class ReactiveData(SyncableData):
         if old_data is not self.value:
             self._update_cds()
 
-    def _process_events(self, events):
+    def _process_events(self, events: Dict[str, Any]) -> None:
         if 'data' in events:
             self._process_data(events.pop('data'))
         if 'indices' in events:
@@ -1051,13 +1123,13 @@ class ReactiveHTMLMetaclass(ParameterizedMetaclass):
     HTML attributes.
     """
 
-    _loaded_extensions = set()
+    _loaded_extensions: Set[str] = set()
 
-    _name_counter = Counter()
+    _name_counter: Counter = Counter()
 
-    _script_regex = r"script\([\"|'](.*)[\"|']\)"
+    _script_regex: str = r"script\([\"|'](.*)[\"|']\)"
 
-    def __init__(mcs, name, bases, dict_):
+    def __init__(mcs, name: str, bases: Tuple[Type, ...], dict_: Mapping[str, Any]):
         from .io.datamodel import PARAM_MAPPING, construct_data_model
 
         mcs.__original_doc__ = mcs.__doc__
@@ -1098,7 +1170,7 @@ class ReactiveHTMLMetaclass(ParameterizedMetaclass):
                 "ensure there is a matching </div> tag."
             )
 
-        mcs._node_callbacks = {}
+        mcs._node_callbacks: Dict[str, List[Tuple[str, str]]]  = {}
         mcs._inline_callbacks = []
         for node, attrs in mcs._parser.attrs.items():
             for (attr, parameters, template) in attrs:
@@ -1277,23 +1349,23 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 `input` node in the example above.
     """
 
-    _child_config = {}
+    _child_config: Mapping[str, str] = {}
 
-    _dom_events = {}
+    _dom_events: Mapping[str, List[str]] = {}
 
-    _extension_name = None
+    _extension_name: Optional[str] = None
 
-    _template = ""
+    _template: str = ""
 
-    _scripts = {}
+    _scripts: Mapping[str, str | List[str]] = {}
 
-    _script_assignment = r'data\.([^[^\d\W]\w*)[ ]*[\+,\-,\*,\\,%,\*\*,<<,>>,>>>,&,\^,|,\&\&,\|\|,\?\?]*='
+    _script_assignment: str = r'data\.([^[^\d\W]\w*)[ ]*[\+,\-,\*,\\,%,\*\*,<<,>>,>>>,&,\^,|,\&\&,\|\|,\?\?]*='
 
     __abstract = True
 
-    __css__ = None
-    __javascript__ = None
-    __javascript_modules__ = None
+    __css__: Optional[List[str]] = None
+    __javascript__: Optional[List[str]] = None
+    __javascript_modules__: Optional[List[str]] = None
 
     def __init__(self, **params):
         from .pane import panel
@@ -1324,7 +1396,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         self._event_callbacks = defaultdict(lambda: defaultdict(list))
 
     @classmethod
-    def _loaded(cls):
+    def _loaded(cls) -> bool:
         """
         Whether the component has been loaded.
         """
@@ -1333,24 +1405,27 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             cls._extension_name in ReactiveHTMLMetaclass._loaded_extensions
         )
 
-    def _cleanup(self, root):
+    def _cleanup(self, root: 'Model' | None) -> None:
         for child, panes in self._panes.items():
             for pane in panes:
                 pane._cleanup(root)
         super()._cleanup(root)
 
     @property
-    def _linkable_params(self):
+    def _linkable_params(self) -> List[str]:
         return [p for p in super()._linkable_params if p not in self._parser.children.values()]
 
     @property
     def _child_names(self):
         return {}
 
-    def _process_children(self, doc, root, model, comm, children):
+    def _process_children(
+        self, doc: 'Document', root: 'Model', model: 'Model', comm: Optional['Comm'],
+        children: Mapping[str, str | 'Model' | List['Model']]
+    ) -> Mapping[str, str | 'Model' | List['Model']]:
         return children
 
-    def _init_params(self):
+    def _init_params(self) -> Dict[str, Any]:
         ignored = list(Reactive.param)
         for child in self._parser.children.values():
             if self._child_config.get(child) != 'literal':
@@ -1389,13 +1464,13 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             ]
         return params
 
-    def _get_events(self):
+    def _get_events(self) -> Dict[str, Dict[str, bool]]:
         events = {}
-        for node, node_events in self._dom_events.items():
-            if isinstance(node_events, list):
-                events[node] = {e: True for e in node_events}
+        for node, dom_events in self._dom_events.items():
+            if isinstance(dom_events, list):
+                events[node] = {e: True for e in dom_events}
             else:
-                events[node] = node_events
+                events[node] = dom_events
         for node, evs in self._event_callbacks.items():
             events[node] = node_events = events.get(node, {})
             for e in evs:
@@ -1403,11 +1478,14 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                     node_events[e] = False
         return events
 
-    def _get_children(self, doc, root, model, comm):
+    def _get_children(
+        self, doc: 'Document', root: 'Model', model: 'Model', comm: Optional['Comm']
+    ) -> Mapping[str, str | 'Model' | List['Model']]:
         from .pane import panel
         old_models = model.children
-        new_models = {parent: [] for parent in self._parser.children}
-        new_panes, internal_panes = {}, {}
+        new_models: Dict[str, List['Model']] = {parent: [] for parent in self._parser.children}
+        new_panes: Dict[str, List[Viewable] | Dict[str, Viewable] | None] = {}
+        internal_panes: Dict[str, List[Viewable] | None] = {}
 
         for parent, children_param in self._parser.children.items():
             mode = self._child_config.get(children_param, 'model')
@@ -1430,15 +1508,15 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
 
         for children_param, old_panes in self._panes.items():
             for old_pane in old_panes:
-                if old_pane not in internal_panes.get(children_param, []):
+                if old_pane not in (internal_panes.get(children_param) or []):
                     old_pane._cleanup(root)
 
         for parent, child_panes in new_panes.items():
             children_param = self._parser.children[parent]
             if isinstance(child_panes, dict):
-                child_panes = child_panes.values()
+                child_panes = list(child_panes.values())
             mode = self._child_config.get(children_param, 'model')
-            if mode == 'literal':
+            if mode == 'literal' or child_panes is None:
                 new_models[parent] = children_param
             elif children_param in self._panes:
                 # Find existing models
@@ -1460,7 +1538,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         self._panes = internal_panes
         return self._process_children(doc, root, model, comm, new_models)
 
-    def _get_template(self):
+    def _get_template(self) -> Tuple[str, List[str], Mapping[str, List[Tuple[str, List[str], str]]]]:
         import jinja2
 
         # Replace loop variables with indexed child parameter e.g.:
@@ -1515,16 +1593,16 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             )
 
         # Parse attrs
-        p_attrs = {}
+        p_attrs: Dict[str, List[Tuple[str, List[str], str]]] = {}
         for node, attrs in parser.attrs.items():
-            for (attr, parameters, template) in attrs:
+            for (attr, parameters, tmpl) in attrs:
                 param_attrs = []
                 for p in parameters:
                     if p in self.param or '.' in p:
                         param_attrs.append(p)
                 if node not in p_attrs:
                     p_attrs[node] = []
-                p_attrs[node].append((attr, param_attrs, template))
+                p_attrs[node].append((attr, param_attrs, tmpl))
 
         # Remove child node template syntax
         for parent, child_name in self._parser.children.items():
@@ -1535,7 +1613,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 html = html.replace('${%s}' % child_name, '')
         return html, parser.nodes, p_attrs
 
-    def _linked_properties(self):
+    def _linked_properties(self) -> List[str]:
         linked_properties = [p for pss in self._attrs.values() for _, ps, _ in pss for p in ps]
         for scripts in self._scripts.values():
             if not isinstance(scripts, list):
@@ -1549,7 +1627,10 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
                 linked_properties.append(children_param)
         return linked_properties
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
+    def _get_model(
+        self, doc: 'Document', root: Optional['Model'] = None,
+        parent: Optional['Model'] = None, comm: Optional['Comm'] = None
+    ) -> 'Model':
         properties = self._process_param_change(self._init_params())
         model = _BkReactiveHTML(**properties)
         if comm and not self._loaded():
@@ -1586,7 +1667,9 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         self._models[root.ref['id']] = (model, parent)
         return model
 
-    def _process_event(self, event):
+    def _process_event(self, event: 'Event') -> None:
+        if not isinstance(event, DOMEvent):
+            return
         cb = getattr(self, f"_{event.node}_{event.data['type']}", None)
         if cb is not None:
             cb(event)
@@ -1603,7 +1686,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         for cb in event_cbs:
             cb(event)
 
-    def _set_on_model(self, msg, root, model):
+    def _set_on_model(self, msg: Mapping[str, Any], root: 'Model', model: 'Model') -> None:
         if not msg:
             return
         old = self._changing.get(root.ref['id'], [])
@@ -1619,7 +1702,10 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
             else:
                 del self._changing[root.ref['id']]
 
-    def _update_model(self, events, msg, root, model, doc, comm):
+    def _update_model(
+        self, events: Dict[str, param.parameterized.Event], msg: Dict[str, Any],
+        root: 'Model', model: 'Model', doc: 'Document', comm: Optional['Comm']
+    ) -> None:
         child_params = self._parser.children.values()
         new_children, model_msg, data_msg  = {}, {}, {}
         for prop, v in list(msg.items()):
@@ -1656,7 +1742,7 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         self._set_on_model(model_msg, root, model)
         self._set_on_model(data_msg, root, model.data)
 
-    def on_event(self, node, event, callback):
+    def on_event(self, node: str, event: str, callback: Callable) -> None:
         """
         Registers a callback to be executed when the specified DOM
         event is triggered on the named node. Note that the named node
@@ -1680,3 +1766,9 @@ class ReactiveHTML(Reactive, metaclass=ReactiveHTMLMetaclass):
         events = self._get_events()
         for ref, (model, _) in self._models.items():
             self._apply_update([], {'events': events}, model, ref)
+
+__all__ = (
+    "Reactive",
+    "ReactiveHTML",
+    "ReactiveData"
+)
