@@ -375,7 +375,7 @@ def test_tabs_iadd(document, comm, tabs):
     assert tab3.child is div4
     assert tab4.child is div3
 
-    
+
 def test_tabs_extend_uses_object_name(document, comm, tabs):
     model = tabs.get_root(document, comm=comm)
     tab1_before, tab2_before = model.tabs
@@ -677,3 +677,15 @@ def test_tabs_clear(document, comm):
     assert tabs._names == []
     assert len(model.tabs) == 0
     assert p1._models == p2._models == {}
+
+
+def test_tabs_pane_update(document, comm):
+    div1 = Div()
+    div2 = Div()
+    tabs = Tabs(div1, div2)
+
+    model = tabs.get_root(document, comm=comm)
+
+    new_div = Div()
+    tabs[1].object = new_div
+    assert model.tabs[1].child is new_div
