@@ -1,6 +1,4 @@
 import os
-import sys
-import pytest
 
 from base64 import b64decode, b64encode
 from pathlib import Path
@@ -77,7 +75,6 @@ def test_load_from_byteio():
     image_data = image_pane._data()
     assert b'PNG' in image_data
 
-@pytest.mark.skipif(sys.version_info.major <= 2, reason="Doesn't work with python 2")
 def test_load_from_stringio():
     """Testing a loading a image from a StringIO"""
     memory = StringIO()
@@ -98,6 +95,15 @@ def test_loading_a_image_from_url():
     image_data = image_pane._data()
     assert b'PNG' in image_data
 
+def test_image_from_bytes():
+    path = os.path.dirname(__file__)
+    with open(os.path.join(path, '../test_data/logo.png'), 'rb') as f:
+        img = f.read()
+
+    image_pane = PNG(img)
+    image_data = image_pane._data()
+    assert b'PNG' in image_data
+
 def test_loading_a_image_from_pathlib():
     """Tests the loading of a image from a pathlib"""
     filepath = Path(__file__).parent.parent / "test_data" / "logo.png"
@@ -108,28 +114,30 @@ def test_loading_a_image_from_pathlib():
 
 def test_image_alt_text(document, comm):
     """Tests the loading of a image from a url"""
-    url = 'https://file-examples-com.github.io/uploads/2017/10/file_example_PNG_500kB.png'
+    url = 'https://raw.githubusercontent.com/holoviz/panel/master/doc/_static/logo.png'
 
     image_pane = PNG(url, embed=False, alt_text="Some alt text")
     model = image_pane.get_root(document, comm)
 
     assert 'alt=&quot;Some alt text&quot;' in model.text
 
+
 def test_image_link_url(document, comm):
     """Tests the loading of a image from a url"""
-    url = 'https://file-examples-com.github.io/uploads/2017/10/file_example_PNG_500kB.png'
+    url = 'https://raw.githubusercontent.com/holoviz/panel/master/doc/_static/logo.png'
 
     image_pane = PNG(url, embed=False, link_url="http://anaconda.org")
     model = image_pane.get_root(document, comm)
 
     assert model.text.startswith('&lt;a href=&quot;http://anaconda.org&quot;')
 
+
 def test_pdf_embed(document, comm):
     pdf_pane = PDF('https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf')
     model = pdf_pane.get_root(document, comm)
 
     assert model.text.startswith("&lt;embed src=&quot;data:application/pdf;base64,")
-    
+
 
 def test_pdf_no_embed(document, comm):
     url = 'https://www.w3.org/WAI/ER/tests/xhtml/testfiles/resources/pdf/dummy.pdf'
