@@ -240,6 +240,7 @@ def test_server_session_info():
 
     state.curdoc = None
     html._server_destroy(session_context)
+    state._destroy_session(session_context)
     assert state.session_info['live'] == 0
 
 
@@ -676,7 +677,6 @@ def test_server_component_custom_resources_with_subpath_and_prefix_relative_url(
 
     r = requests.get(f"http://localhost:{port}/prefix/subpath/component")
     content = r.content.decode('utf-8')
-    print(content)
     assert 'href="../components/panel.tests.test_server/CustomComponent/__css__/assets/custom.css"' in content
 
 
