@@ -1,8 +1,7 @@
 """
 These that verify Templates are working correctly.
 """
-from distutils.version import LooseVersion
-from panel.layout.grid import GridSpec
+from packaging.version import Version
 
 try:
     import holoviews as hv
@@ -12,10 +11,13 @@ except Exception:
 import param
 import pytest
 
-latest_param = pytest.mark.skipif(LooseVersion(param.__version__) < '1.10.0a4',
+latest_param = pytest.mark.skipif(Version(param.__version__) < Version('1.10.0a4'),
                                   reason="requires param>=1.10.0a4")
 
-from panel.layout import Row
+from bokeh.document import Document
+from bokeh.io.doc import patch_curdoc
+
+from panel.layout import GridSpec, Row
 from panel.pane import HoloViews, Markdown
 from panel.template import (
     BootstrapTemplate, GoldenTemplate, MaterialTemplate, ReactTemplate,
@@ -140,6 +142,17 @@ def test_basic_template(template, document, comm):
     assert tvars['header'] == False
 
 
+def test_template_server_title():
+    tmpl = VanillaTemplate(title='Main title')
+
+    doc = Document()
+
+    with patch_curdoc(doc):
+        doc = tmpl.server_doc(title='Ignored title')
+
+    assert doc.title == 'Main title'
+
+
 def test_react_template(document, comm):
     tmplt = ReactTemplate(title='BasicTemplate', header_background='blue', header_color='red')
 
@@ -162,12 +175,17 @@ def test_react_template(document, comm):
     markdown2 = Markdown('# Some title')
     tmplt.main[:4, 6:] = markdown2
 
-    assert tvars['layouts'] == {
-        'lg': [{'h': 4, 'i': '1', 'w': 6, 'x': 0, 'y': 0},
-               {'h': 4, 'i': '2', 'w': 6, 'x': 6, 'y': 0}],
-        'md': [{'h': 4, 'i': '1', 'w': 6, 'x': 0, 'y': 0},
-               {'h': 4, 'i': '2', 'w': 6, 'x': 6, 'y': 0}]
-    }
+    layouts = {'lg': [{'h': 4, 'i': '1', 'w': 6, 'x': 0, 'y': 0},
+                      {'h': 4, 'i': '2', 'w': 6, 'x': 6, 'y': 0}],
+               'md': [{'h': 4, 'i': '1', 'w': 6, 'x': 0, 'y': 0},
+                      {'h': 4, 'i': '2', 'w': 6, 'x': 6, 'y': 0}]
+               }
+
+    for size in layouts:
+        for layout in layouts[size]:
+            layout.update({'minW': 0, 'minH': 0, 'maxW': 'Infinity', 'maxH': 'Infinity'})
+
+    assert tvars['layouts'] == layouts
 
 @pytest.mark.parametrize(["template_class"], [(t,) for t in LIST_TEMPLATES])
 def test_list_template_insert_order(template_class):
@@ -179,7 +197,7 @@ def test_list_template_insert_order(template_class):
 
     template.main.extend([2, 3])
 
-    objs = list(template._render_items.values())[2:]
+    objs = list(template._render_items.values())[3:]
     ((obj1, tag1), (obj2, tag2), (obj3, tag3), (obj4, tag4)) = objs
 
     assert tag1 == tag2 == tag3 == tag4 == ['main']
@@ -208,7 +226,7 @@ def test_grid_template_override():
     template.main[0, 0] = item
     template.main[0, 0] = override
 
-    objs = list(template._render_items.values())[2:]
+    objs = list(template._render_items.values())[3:]
     assert len(objs) == 1
     ((obj, tags),) = objs
 
