@@ -11,6 +11,8 @@ def test_progress_bounds():
     assert progress.param.value.bounds == (-1, 200)
     progress.value = 120
     assert progress.value == 120
+    progress.value = -1
+    assert progress.value == -1
 
 
 def test_file_download_label():
@@ -63,13 +65,13 @@ def test_file_download_file():
 
     with pytest.raises(ValueError):
         FileDownload(embed=True)
-    
+
     with pytest.raises(FileNotFoundError):
         FileDownload("nofile", embed=True)
 
     with pytest.raises(ValueError):
         FileDownload(666, embed=True)
-    
+
     file_download = FileDownload("nofile")
     with pytest.raises(FileNotFoundError):
         file_download._clicks += 1
@@ -80,9 +82,9 @@ def test_file_download_callback():
 
     with pytest.raises(ValueError):
         file_download._clicks += 1
-    
+
     file_download = FileDownload(callback=lambda: StringIO("data"), filename="abc.py")
-    
+
     assert file_download.data is None
 
     file_download._clicks += 1
