@@ -8,19 +8,25 @@ and become viewable including:
 * Viewable: Defines methods to view the component in the
   notebook, on the server or in static exports
 """
-import datetime as dt
+from __future__ import annotations
+
+import asyncio
 import logging
 import sys
+import threading
 import traceback
 import uuid
 
 from functools import partial
+from typing import (
+    TYPE_CHECKING, Any, Callable, IO, List, Mapping, Optional
+)
 
-import param
+import param # type: ignore
 
-from bokeh.document.document import Document as _Document
+from bokeh.document import Document
 from bokeh.io import curdoc as _curdoc
-from pyviz_comms import JupyterCommManager
+from pyviz_comms import Comm, JupyterCommManager # type: ignore
 
 from .config import config, panel_extension
 from .io import serve
@@ -35,6 +41,13 @@ from .io.save import save
 from .io.state import state
 from .util import escape, param_reprs
 
+if TYPE_CHECKING:
+    from bokeh.model import Model
+    from bokeh.server.contexts import BokehSessionContext
+    from bokeh.server.server import Server
+
+    from .io.location import Location
+
 
 class Layoutable(param.Parameterized):
     """
@@ -236,15 +249,20 @@ class ServableMixin(object):
     Mixin to define methods shared by objects which can served.
     """
 
-    def _modify_doc(self, server_id, title, doc, location):
+    def _modify_doc(
+        self, server_id: str, title: str, doc: Document, location: Optional['Location']
+    ) -> Document:
         """
         Callback to handle FunctionHandler document creation.
         """
         if server_id:
             state._servers[server_id][2].append(doc)
-        return self.server_doc(doc, title, location)
+        return self.server_doc(doc, title, location) # type: ignore
 
-    def _add_location(self, doc, location, root=None):
+    def _add_location(
+        self, doc: Document, location: Optional['Location' | bool],
+        root: Optional['Model'] = None
+    ) -> 'Location':
         from .io.location import Location
         if isinstance(location, Location):
             loc = location
@@ -254,14 +272,14 @@ class ServableMixin(object):
             loc = Location()
         state._locations[doc] = loc
         if root is None:
-            loc_model = loc._get_root(doc)
+            loc_model = loc.get_root(doc)
         else:
             loc_model = loc._get_model(doc, root)
         loc_model.name = 'location'
         doc.add_root(loc_model)
         return loc
 
-    def _on_msg(self, ref, manager, msg):
+    def _on_msg(self, ref: str, manager, msg) -> None:
         """
         Handles Protocol messages arriving from the client comm.
         """
@@ -270,12 +288,12 @@ class ServableMixin(object):
         held = doc.callbacks.hold_value
         patch = manager.assemble(msg)
         doc.hold()
-        patch.apply_to_document(doc, comm.id)
+        patch.apply_to_document(doc, comm.id if comm else None)
         doc.unhold()
         if held:
             doc.hold(held)
 
-    def _on_error(self, ref, error):
+    def _on_error(self, ref: str, error: Exception) -> None:
         if ref not in state._handles or config.console_output in [None, 'disable']:
             return
         handle, accumulator = state._handles[ref]
@@ -287,7 +305,7 @@ class ServableMixin(object):
         if accumulator:
             handle.update({'text/html': '\n'.join(accumulator)}, raw=True)
 
-    def _on_stdout(self, ref, stdout):
+    def _on_stdout(self, ref: str, stdout: Any) -> None:
         if ref not in state._handles or config.console_output is [None, 'disable']:
             return
         handle, accumulator = state._handles[ref]
@@ -303,11 +321,16 @@ class ServableMixin(object):
     # Public API
     #----------------------------------------------------------------
 
-    def servable(self, title=None, location=True, area='main'):
+    def servable(
+        self, title: Optional[str] = None, location: bool | 'Location' = True,
+        area: str = 'main', target: Optional[str] = None
+    ) -> 'ServableMixin':
         """
-        Serves the object if in a `panel serve` context and returns
-        the Panel object to allow it to display itself in a notebook
-        context.
+        Serves the object or adds it to the configured
+        pn.state.template if in a `panel serve` context, writes to the
+        DOM if in a pyodide context and returns the Panel object to
+        allow it to display itself in a notebook context.
+
         Arguments
         ---------
         title : str
@@ -315,9 +338,14 @@ class ServableMixin(object):
         location : boolean or panel.io.location.Location
           Whether to create a Location component to observe and
           set the URL location.
-        area: str
+        area: str (deprecated)
           The area of a template to add the component too. Only has an
           effect if pn.config.template has been set.
+        target: str
+          Target area to write to. If a template has been configured
+          on pn.config.template this refers to the target area in the
+          template while in pyodide this refers to the ID of the DOM
+          node to write to.
 
         Returns
         -------
@@ -329,7 +357,9 @@ class ServableMixin(object):
                 if isinstance(handler, logging.StreamHandler):
                     handler.setLevel(logging.WARN)
             if config.template:
+                area = target or area or 'main'
                 template = state.template
+                assert template is not None
                 if template.title == template.param.title.default and title:
                     template.title = title
                 if area == 'main':
@@ -341,17 +371,29 @@ class ServableMixin(object):
                 elif area == 'header':
                     template.header.append(self)
             else:
-                self.server_doc(title=title, location=location)
+                self.server_doc(title=title, location=location) # type: ignore
+        elif state._is_pyodide:
+            from .io.pyodide import write
+            if target:
+                out = target
+            elif hasattr(sys.stdout, '_out'):
+                out = sys.stdout._out # type: ignore
+            else:
+                raise ValueError("Could not determine target node to write to.")
+            asyncio.create_task(write(out, self))
         return self
 
-    def show(self, title=None, port=0, address=None, websocket_origin=None,
-             threaded=False, verbose=True, open=True, location=True, **kwargs):
+    def show(
+        self, title: Optional[str] = None, port: int = 0, address: Optional[str] = None,
+        websocket_origin: Optional[str] = None, threaded: bool = False, verbose: bool = True,
+        open: bool = True, location: bool | 'Location' = True, **kwargs
+    ) -> threading.Thread | 'Server':
         """
         Starts a Bokeh server and displays the Viewable in a new tab.
 
         Arguments
         ---------
-        title : str
+        title : str | None
           A string title to give the Document (if served as an app)
         port: int (optional, default=0)
           Allows specifying a specific port
@@ -406,10 +448,13 @@ class Renderable(param.Parameterized):
         self._found_links = set()
         self._logger = logging.getLogger(f'{__name__}.{type(self).__name__}')
 
-    def _log(self, msg, *args, level='debug'):
+    def _log(self, msg: str, *args, level: str = 'debug') -> None:
         getattr(self._logger, level)(f'Session %s {msg}', id(state.curdoc), *args)
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
+    def _get_model(
+        self, doc: Document, root: Optional['Model'] = None,
+        parent: Optional['Model'] = None, comm: Optional[Comm] = None
+    ) -> 'Model':
         """
         Converts the objects being wrapped by the viewable into a
         bokeh model that can be composed in a bokeh layout.
@@ -431,7 +476,7 @@ class Renderable(param.Parameterized):
         """
         raise NotImplementedError
 
-    def _cleanup(self, root):
+    def _cleanup(self, root: 'Model' | None) -> None:
         """
         Clean up method which is called when a Viewable is destroyed.
 
@@ -440,11 +485,13 @@ class Renderable(param.Parameterized):
         root: bokeh.model.Model
           Bokeh model for the view being cleaned up
         """
+        if root is None:
+            return
         ref = root.ref['id']
         if ref in state._handles:
             del state._handles[ref]
 
-    def _preprocess(self, root):
+    def _preprocess(self, root: 'Model') -> None:
         """
         Applies preprocessing hooks to the model.
         """
@@ -452,9 +499,9 @@ class Renderable(param.Parameterized):
         for hook in hooks:
             hook(self, root)
 
-    def _render_model(self, doc=None, comm=None):
+    def _render_model(self, doc: Optional[Document] = None, comm: Optional[Comm] = None) -> 'Model':
         if doc is None:
-            doc = _Document()
+            doc = Document()
         if comm is None:
             comm = state._comm_manager.get_server_comm()
         model = self.get_root(doc, comm)
@@ -470,34 +517,27 @@ class Renderable(param.Parameterized):
             add_to_doc(model, doc)
         return model
 
-    def _init_params(self):
+    def _init_params(self) -> Mapping[str, Any]:
         return {k: v for k, v in self.param.values().items() if v is not None}
 
-    def _server_destroy(self, session_context):
+    def _server_destroy(self, session_context: 'BokehSessionContext') -> None:
         """
         Server lifecycle hook triggered when session is destroyed.
         """
-        session_id = session_context.id
-        sessions = state.session_info['sessions']
-        if session_id in sessions and sessions[session_id]['ended'] is None:
-            session = sessions[session_id]
-            if session['rendered'] is not None:
-                state.session_info['live'] -= 1
-            session['ended'] = dt.datetime.now().timestamp()
-            state.param.trigger('session_info')
         doc = session_context._document
+        if doc not in self._documents:
+            return
         root = self._documents[doc]
         ref = root.ref['id']
         self._cleanup(root)
         del self._documents[doc]
         if ref in state._views:
             del state._views[ref]
-        if doc in state._locations:
-            loc = state._locations[doc]
-            loc._cleanup(root)
-            del state._locations[doc]
 
-    def get_root(self, doc=None, comm=None, preprocess=True):
+    def get_root(
+        self, doc: Optional[Document] = None, comm: Optional[Comm] = None,
+        preprocess: bool = True
+    ) -> 'Model':
         """
         Returns the root model and applies pre-processing hooks
 
@@ -537,7 +577,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         Whether or not the Viewable is loading. If True a loading spinner
         is shown on top of the Viewable.""")
 
-    _preprocessing_hooks = []
+    _preprocessing_hooks: List[Callable[['Viewable', 'Model'], None]] = []
 
     def __init__(self, **params):
         hooks = params.pop('hooks', [])
@@ -547,26 +587,25 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         watcher = self.param.watch(self._update_loading, 'loading')
         self._callbacks.append(watcher)
 
-    def _update_loading(self, *_):
+    def _update_loading(self, *_) -> None:
         if self.loading:
             start_loading_spinner(self)
         else:
             stop_loading_spinner(self)
 
-    def __repr__(self, depth=0):
+    def __repr__(self, depth: int = 0) -> str:
         return '{cls}({params})'.format(cls=type(self).__name__,
                                         params=', '.join(param_reprs(self)))
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.__repr__()
 
     def _repr_mimebundle_(self, include=None, exclude=None):
         loaded = panel_extension._loaded
         if not loaded and 'holoviews' in sys.modules:
-            import holoviews as hv
+            import holoviews as hv # type: ignore
             loaded = hv.extension._loaded
 
-
         if config.comms in ('vscode', 'ipywidgets'):
             widget = ipywidget(self)
             if hasattr(widget, '_repr_mimebundle_'):
@@ -584,7 +623,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
                     'model_id': widget._model_id
                 }
             if config.comms == 'vscode':
-                from IPython.display import display
+                from IPython.display import display # type: ignore
                 display(data, raw=True)
                 return {'text/html': '<div style="display: none"></div>'}, {}
             return data, {}
@@ -601,7 +640,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
             load_notebook(config.inline)
 
         try:
-            from IPython import get_ipython
+            from IPython import get_ipython # type: ignore
             assert get_ipython().kernel is not None
             state._comm_manager = JupyterCommManager
         except Exception:
@@ -617,7 +656,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         from IPython.display import display
         from .models.comm_manager import CommManager
 
-        doc = _Document()
+        doc = Document()
         comm = state._comm_manager.get_server_comm()
         model = self._render_model(doc, comm)
         ref = model.ref['id']
@@ -642,7 +681,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
     # Public API
     #----------------------------------------------------------------
 
-    def clone(self, **params):
+    def clone(self, **params) -> 'Viewable':
         """
         Makes a copy of the object sharing the same parameters.
 
@@ -658,13 +697,15 @@ class Viewable(Renderable, Layoutable, ServableMixin):
                      if not self.param[p].readonly}
         return type(self)(**dict(inherited, **params))
 
-    def pprint(self):
+    def pprint(self) -> None:
         """
         Prints a compositional repr of the class.
         """
         print(self)
 
-    def select(self, selector=None):
+    def select(
+        self, selector: Optional[type | Callable[['Viewable'], bool]] = None
+    ) -> List['Viewable']:
         """
         Iterates over the Viewable and any potential children in the
         applying the Selector.
@@ -686,7 +727,7 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         else:
             return []
 
-    def app(self, notebook_url="localhost:8888", port=0):
+    def app(self, notebook_url: str = "localhost:8888", port: int = 0) -> 'Server':
         """
         Displays a bokeh server app inline in the notebook.
 
@@ -699,8 +740,11 @@ class Viewable(Renderable, Layoutable, ServableMixin):
         """
         return show_server(self, notebook_url, port)
 
-    def embed(self, max_states=1000, max_opts=3, json=False, json_prefix='',
-              save_path='./', load_path=None, progress=False, states={}):
+    def embed(
+        self, max_states: int = 1000, max_opts: int = 3, json: bool = False,
+        json_prefix: str = '', save_path: str = './', load_path: Optional[str] = None,
+        progress: bool = False, states={}
+    ) -> None:
         """
         Renders a static version of a panel in a notebook by evaluating
         the set of states defined by the widgets in the model. Note
@@ -731,7 +775,8 @@ class Viewable(Renderable, Layoutable, ServableMixin):
             load_path, progress, states
         )
 
-    def save(self, filename, title=None, resources=None, template=None,
+    def save(self, filename: str | IO, title: Optional[str] = None,
+             resources=None, template=None,
              template_variables=None, embed=False, max_states=1000,
              max_opts=3, embed_json=False, json_prefix='', save_path='./',
              load_path=None, progress=True, embed_states={}, as_png=None,
@@ -778,7 +823,10 @@ class Viewable(Renderable, Layoutable, ServableMixin):
                     embed_json, json_prefix, save_path, load_path,
                     progress, embed_states, as_png, **kwargs)
 
-    def server_doc(self, doc=None, title=None, location=True):
+    def server_doc(
+        self, doc: Optional[Document] = None, title: Optional[str] = None,
+        location: bool | 'Location' = True
+    ) -> Document:
         """
         Returns a serveable bokeh Document with the panel attached
 
@@ -803,11 +851,12 @@ class Viewable(Renderable, Layoutable, ServableMixin):
             title = title or 'Panel Application'
             doc.title = title
         model = self.get_root(doc)
-        if hasattr(doc, 'on_session_destroyed'):
-            doc.on_session_destroyed(self._server_destroy)
-            self._documents[doc] = model
+        doc.on_session_destroyed(state._destroy_session)
+        doc.on_session_destroyed(self._server_destroy) # type: ignore
+        self._documents[doc] = model
         add_to_doc(model, doc)
-        if location: self._add_location(doc, location, model)
+        if location:
+            self._add_location(doc, location, model)
         return doc
 
 
@@ -835,13 +884,19 @@ class Viewer(param.Parameterized):
 
         return view
 
-    def servable(self, title=None, location=True):
-        return self._create_view().servable(title, location)
+    def servable(
+        self, title: Optional[str]=None, location: bool | 'Location' = True,
+        area: str = 'main', target: Optional[str] = None
+    ) -> 'Viewer':
+        return self._create_view().servable(title, location, area, target)
 
     servable.__doc__ = ServableMixin.servable.__doc__
 
-    def show(self, title=None, port=0, address=None, websocket_origin=None,
-             threaded=False, verbose=True, open=True, location=True, **kwargs):
+    def show(
+        self, title: Optional[str] = None, port: int = 0, address: Optional[str] = None,
+        websocket_origin: Optional[str] = None, threaded: bool = False, verbose: bool = True,
+        open: bool = True, location: bool | 'Location' = True, **kwargs
+    ) -> threading.Thread | 'Server':
         return self._create_view().show(
             title, port, address, websocket_origin, threaded,
             verbose, open, location, **kwargs
@@ -851,3 +906,10 @@ class Viewer(param.Parameterized):
 
     def _repr_mimebundle_(self, include=None, exclude=None):
         return self._create_view()._repr_mimebundle_(include, exclude)
+
+
+__all__ = (
+    "Layoutable",
+    "Viewable",
+    "Viewer"
+)
