@@ -3,13 +3,26 @@ Defines the Widget base class which provides bi-directional
 communication between the rendered dashboard and the Widget
 parameters.
 """
+from __future__ import annotations
+
 import math
 
-import param
+from typing import (
+    TYPE_CHECKING, Any, Callable, List, Mapping, Optional, Tuple
+)
+
+import param # type: ignore
 
 from ..layout import Row
 from ..reactive import Reactive
-from ..viewable import Layoutable
+from ..viewable import Layoutable, Viewable
+
+if TYPE_CHECKING:
+    from bokeh.document import Document
+    from bokeh.model import Model
+    from pyviz_comms import Comm
+
+    from ..layout import Panel
 
 
 class Widget(Reactive):
@@ -32,13 +45,13 @@ class Widget(Reactive):
         be specified as a two-tuple of the form (vertical, horizontal)
         or a four-tuple (top, right, bottom, left).""")
 
-    _rename = {'name': 'title'}
-    
+    _rename: Mapping[str, str | None] = {'name': 'title'}
+
     # Whether the widget supports embedding
-    _supports_embed = False
+    _supports_embed: bool = False
 
     # Declares the Bokeh model type of the widget
-    _widget_type = None
+    _widget_type: 'Model' = None
 
     __abstract = True
 
@@ -54,18 +67,18 @@ class Widget(Reactive):
         super().__init__(**params)
 
     @classmethod
-    def from_param(cls, parameter, **params):
+    def from_param(cls, parameter: param.Parameter, **params) -> Viewable:
         """
         Construct a widget from a Parameter and link the two
         bi-directionally.
-        
+
         Parameters
         ----------
         parameter: param.Parameter
           A parameter to create the widget from.
         params: dict
           Keyword arguments to be passed to the widget constructor
-        
+
         Returns
         -------
         Widget instance linked to the supplied parameter
@@ -77,7 +90,10 @@ class Widget(Reactive):
         )
         return layout[0]
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
+    def _get_model(
+        self, doc: 'Document', root: Optional['Model'] = None,
+        parent: Optional['Model'] = None, comm: Optional['Comm'] = None
+    ) -> 'Model':
         model = self._widget_type(**self._process_param_change(self._init_params()))
         if root is None:
             root = model
@@ -88,11 +104,13 @@ class Widget(Reactive):
         self._link_props(model, properties, doc, root, comm)
         return model
 
-    def _filter_properties(self, properties):
+    def _filter_properties(self, properties: List[str]) -> List[str]:
         ignored = list(Layoutable.param)+['loading']
         return [p for p in properties if p not in ignored]
 
-    def _get_embed_state(self, root, values=None, max_opts=3):
+    def _get_embed_state(
+        self, root: 'Model', values: Optional[List[Any]] = None, max_opts: int = 3
+    ) -> Tuple['Widget', 'Model', List[Any], Callable[['Model'], Any], str, str]:
         """
         Returns the bokeh model and a discrete set of value states
         for the widget.
@@ -129,9 +147,9 @@ class CompositeWidget(Widget):
     widgets
     """
 
-    __abstract = True
+    _composite_type: 'Panel' = Row
 
-    _composite_type = Row
+    __abstract = True
 
     def __init__(self, **params):
         super().__init__(**params)
@@ -144,11 +162,13 @@ class CompositeWidget(Widget):
         self._models = self._composite._models
         self.param.watch(self._update_layout_params, layout_params)
 
-    def _update_layout_params(self, *events):
+    def _update_layout_params(self, *events: param.parameterized.Event) -> None:
         updates = {event.name: event.new for event in events}
         self._composite.param.update(**updates)
 
-    def select(self, selector=None):
+    def select(
+        self, selector: Optional[type | Callable[['Viewable'], bool]] = None
+    ) -> List[Viewable]:
         """
         Iterates over the Viewable and any potential children in the
         applying the Selector.
@@ -168,20 +188,23 @@ class CompositeWidget(Widget):
             objects += obj.select(selector)
         return objects
 
-    def _cleanup(self, root):
+    def _cleanup(self, root: 'Model') -> None:
         self._composite._cleanup(root)
         super()._cleanup(root)
 
-    def _get_model(self, doc, root=None, parent=None, comm=None):
+    def _get_model(
+        self, doc: 'Document', root: Optional['Model'] = None,
+        parent: Optional['Model'] = None, comm: Optional['Comm'] = None
+    ) -> 'Model':
         model = self._composite._get_model(doc, root, parent, comm)
         if root is None:
             root = parent = model
         self._models[root.ref['id']] = (model, parent)
         return model
 
-    def __contains__(self, object):
+    def __contains__(self, object: Any) -> bool:
         return object in self._composite.objects
 
     @property
-    def _synced_params(self):
+    def _synced_params(self) -> List[str]:
         return []
