@@ -1,3 +1,20 @@
+"""
+Indicators
+==========
+
+Indicators can be used to indicate status or progress
+
+Check out the Panel gallery of indicators
+https://panel.holoviz.org/reference/index.html#indicators for inspiration.
+
+How to use indicators
+---------------------
+
+>>> pn.indicators.Number(
+...    name='Rate', value=72, format='{value}%',
+...    colors=[(80, 'green'), (100, 'red')]
+... )
+"""
 import math
 import os
 import sys
@@ -52,7 +69,7 @@ class BooleanStatus(BooleanIndicator):
     """
     The `BooleanStatus` is a boolean indicator providing a visual
     representation of a boolean status as filled or non-filled circle.
-    
+
     If the value is set to `True` the indicator will be filled while
     setting it to `False` will cause it to be non-filled.
 
@@ -97,7 +114,7 @@ class LoadingSpinner(BooleanIndicator):
     """
     The `LoadingSpinner` is a boolean indicator providing a visual
     representation of the loading status.
-    
+
     If the value is set to `True` the spinner will rotate while
     setting it to `False` will disable the rotating segment.
 
@@ -156,7 +173,7 @@ class Progress(ValueIndicator):
     """
     The `Progress` widget displays the progress towards some target
     based on the current `value` and the `max` value.
-    
+
     If no `value` is set, the `Progress` widget is in indeterminate
     mode and will animate depending on whether it is `active` or
     not. A more beautiful indicator for this use case is the
@@ -1055,7 +1072,7 @@ MARGIN = {
 
 
 class ptqdm(_tqdm):
-    
+
     def __init__(self, *args, **kwargs):
         self._indicator = kwargs.pop('indicator')
         super().__init__(*args, **kwargs)
@@ -1215,3 +1232,18 @@ class Tqdm(Indicator):
         """Resets the parameters"""
         self.value = self.param.value.default
         self.text = self.param.text.default
+
+__all__ = [
+    "BooleanIndicator",
+    "BooleanStatus",
+    "Dial",
+    "Gauge",
+    "LinearGauge",
+    "LoadingSpinner",
+    "Number",
+    "Progress",
+    "String",
+    "Tqdm",
+    "Trend",
+    "ValueIndicator",
+]
