@@ -1,8 +1,6 @@
 """
 Defines Player widgets which offer media-player like controls.
 """
-from __future__ import absolute_import, division, unicode_literals
-
 import param
 
 from ..models.widgets import Player as _BkPlayer
@@ -25,6 +23,10 @@ class PlayerBase(Widget):
     show_loop_controls = param.Boolean(default=True, doc="""
         Whether the loop controls radio buttons are shown""")
 
+    direction = param.Integer(0, doc="""
+        Current play direction of the Player (-1: playing in reverse,
+        0: paused, 1: playing)""")
+
     height = param.Integer(default=80)
 
     width = param.Integer(default=510)
@@ -35,13 +37,29 @@ class PlayerBase(Widget):
 
     __abstract = True
 
+    def play(self):
+        self.direction = 1
+
+    def pause(self):
+        self.direction = 0
+
+    def reverse(self):
+        self.direction = -1
+
+
 
 class Player(PlayerBase):
     """
-    The Player provides controls to play and skip through a number of
+    The `Player` provides controls to play and skip through a number of
     frames defined by explicit start and end values.  The speed at
-    which the widget plays is defined by the interval, but it is also
-    possible to skip frames using the step parameter.
+    which the widget plays is defined by the `interval`, but it is also
+    possible to skip frames using the `step` parameter.
+
+    Reference: https://panel.holoviz.org/reference/widgets/Player.html
+
+    :Example:
+
+    >>> Player(name='Player', start=0, end=100, value=32, loop_policy='loop')
     """
 
     start = param.Integer(default=0, doc="Lower bound on the slider value")
@@ -60,25 +78,37 @@ class Player(PlayerBase):
             params['end'] = params.pop('length')-1
         elif params.get('start', 0) > 0 and not 'value' in params:
             params['value'] = params['start']
-        super(Player, self).__init__(**params)
+        super().__init__(**params)
 
-    def _get_embed_state(self, root, max_opts=3):
-        return (self, self._models[root.ref['id']][0], range(self.start, self.end, self.step),
+    def _get_embed_state(self, root, values=None, max_opts=3):
+        if values is None:
+            values = list(range(self.start, self.end, self.step))
+        return (self, self._models[root.ref['id']][0], values,
                 lambda x: x.value, 'value', 'cb_obj.value')
 
 
 
 class DiscretePlayer(PlayerBase, SelectBase):
     """
-    The DiscretePlayer provides controls to iterate through a list of
+    The `DiscretePlayer` provides controls to iterate through a list of
     discrete options.  The speed at which the widget plays is defined
-    by the interval, but it is also possible to skip items using the
-    step parameter.
+    by the `interval`, but it is also possible to skip items using the
+    `step` parameter.
+
+    Reference: https://panel.holoviz.org/reference/widgets/DiscretePlayer.html
+
+    :Example:
+
+    >>> DiscretePlayer(
+    ...     name='Discrete Player',
+    ...     options=[2, 4, 8, 16, 32, 64, 128], value=32,
+    ...     loop_policy='loop'
+    ... )
     """
 
     interval = param.Integer(default=500, doc="Interval between updates")
 
-    value = param.Parameter()
+    value = param.Parameter(doc="Current player value")
 
     _rename = {'name': None, 'options': None}
 
@@ -97,7 +127,7 @@ class DiscretePlayer(PlayerBase, SelectBase):
                 msg['value'] = indexOf(value, values)
             elif values:
                 self.value = values[0]
-        return super(DiscretePlayer, self)._process_param_change(msg)
+        return super()._process_param_change(msg)
 
     def _process_property_change(self, msg):
         if 'value' in msg:
