@@ -213,8 +213,8 @@ class BaseTable(ReactiveData, Widget):
             indexes = list(self._processed.index)
             selection = []
             for sel in self.selection:
-                iv = old_processed.index[sel]
                 try:
+                    iv = old_processed.index[sel]
                     idx = indexes.index(iv)
                     selection.append(idx)
                 except Exception:
@@ -447,7 +447,6 @@ class BaseTable(ReactiveData, Widget):
     def _get_data(self):
         import pandas as pd
         df = self._filter_dataframe(self.value)
-        df = self._sort_df(df)
         if df is None:
             return [], {}
         if isinstance(self.value.index, pd.MultiIndex):
@@ -494,7 +493,7 @@ class BaseTable(ReactiveData, Widget):
 
         Arguments
         ---------
-        stream_value: (Union[pd.DataFrame, pd.Series, Dict])
+        stream_value: (pd.DataFrame | pd.Series | Dict)
           The new value(s) to append to the existing value.
         rollover: int
            A maximum column size, above which data from the start of
@@ -598,7 +597,7 @@ class BaseTable(ReactiveData, Widget):
 
         Arguments
         ---------
-        patch_value: (Union[pd.DataFrame, pd.Series, Dict])
+        patch_value: (pd.DataFrame | pd.Series | Dict)
           The value(s) to patch the existing value with.
         as_index: boolean
           Whether to treat the patch index as DataFrame indexes (True)
@@ -690,7 +689,8 @@ class BaseTable(ReactiveData, Widget):
         Returns the current view of the table after filtering and
         sorting are applied.
         """
-        return self._processed
+        df = self._processed
+        return self._sort_df(df)
 
     @property
     def selected_dataframe(self):
@@ -698,18 +698,17 @@ class BaseTable(ReactiveData, Widget):
         Returns a DataFrame of the currently selected rows.
         """
         if not self.selection:
-            return self._processed.iloc[:0]
-        return self._processed.iloc[self.selection]
-
+            return self.current_view.iloc[:0]
+        return self.current_view.iloc[self.selection]
 
 
 class DataFrame(BaseTable):
     """
     The `DataFrame` widget allows displaying and editing a pandas DataFrame.
-    
+
     Note that editing is not possible for multi-indexed DataFrames, in which
-    case you will need to reduce the DataFrame to a single index. 
-    
+    case you will need to reduce the DataFrame to a single index.
+
     Also note that the `DataFrame` widget will eventually be replaced with the
     `Tabulator` widget, and so new code should be written to use `Tabulator`
     instead.
@@ -784,6 +783,11 @@ class DataFrame(BaseTable):
 
     _source_transforms = {'hierarchical': None}
 
+    _rename = {
+        'disabled': 'editable', 'selection': None, 'sorters': None,
+        'text_align': None
+    }
+
     @property
     def _widget_type(self):
         return DataCube if self.hierarchical else DataTable
@@ -1013,7 +1017,9 @@ class Tabulator(BaseTable):
         if self.value is None or self._explicit_pagination:
             return
         with param.parameterized.discard_events(self):
-            if self._MAX_ROW_LIMITS[0] < len(self.value) <= self._MAX_ROW_LIMITS[1]:
+            if self.hierarchical:
+                pass
+            elif self._MAX_ROW_LIMITS[0] < len(self.value) <= self._MAX_ROW_LIMITS[1]:
                 self.pagination = 'local'
             elif len(self.value) > self._MAX_ROW_LIMITS[1]:
                 self.pagination = 'remote'
@@ -1041,25 +1047,26 @@ class Tabulator(BaseTable):
         super()._cleanup(root)
 
     def _process_event(self, event):
+        event_col = self._renamed_cols.get(event.column, event.column)
         if self.pagination == 'remote':
             nrows = self.page_size
-            event.row = (self.page-1)*nrows
-        if event.column not in self.buttons:
-            if event.column not in self._processed.columns:
+            event.row = event.row+(self.page-1)*nrows
+        if event_col not in self.buttons:
+            if event_col not in self._processed.columns:
                 event.value = self._processed.index[event.row]
             else:
-                event.value = self._processed[event.column].iloc[event.row]
+                event.value = self._processed[event_col].iloc[event.row]
         if event.event_name == 'table-edit':
             if self._old is not None:
-                event.old = self._old[event.column].iloc[event.row]
+                event.old = self._old[event_col].iloc[event.row]
             for cb in self._on_edit_callbacks:
-                cb(event)
+                state.execute(partial(cb, event))
             self._update_style()
         else:
             for cb in self._on_click_callbacks.get(None, []):
-                cb(event)
-            for cb in self._on_click_callbacks.get(event.column, []):
-                cb(event)
+                state.execute(partial(cb, event))
+            for cb in self._on_click_callbacks.get(event_col, []):
+                state.execute(partial(cb, event))
 
     def _get_theme(self, theme, resources=None):
         from ..io.resources import RESOURCE_MODE
@@ -1145,7 +1152,7 @@ class Tabulator(BaseTable):
             styler = self._computed_styler
         if styler is None:
             return {}
-        offset = len(self.indexes) + int(self.selectable in ('checkbox', 'checkbox-single')) + int(bool(self.row_content))
+        offset = 1 + len(self.indexes) + int(self.selectable in ('checkbox', 'checkbox-single')) + int(bool(self.row_content))
         if self.pagination == 'remote':
             start = (self.page-1)*self.page_size
             end = start + self.page_size
@@ -1374,7 +1381,8 @@ class Tabulator(BaseTable):
             'hidden_columns': self.hidden_columns,
             'editable': not self.disabled,
             'select_mode': selectable,
-            'selectable_rows': self._get_selectable()
+            'selectable_rows': self._get_selectable(),
+            'sorters': self.sorters
         })
         process = {'theme': self.theme, 'frozen_rows': self.frozen_rows}
         props.update(self._process_param_change(process))
@@ -1645,3 +1653,14 @@ class Tabulator(BaseTable):
         if column not in self._on_click_callbacks:
             self._on_click_callbacks[column] = []
         self._on_click_callbacks[column].append(callback)
+
+    @property
+    def current_view(self):
+        """
+        Returns the current view of the table after filtering and
+        sorting are applied.
+        """
+        df = self._processed
+        if self.pagination == 'remote':
+            return df
+        return self._sort_df(df)
